@@ -36,7 +36,7 @@
 #include <libxml/tree.h>
 #include <string.h>
 
-#include "public/cpelang.h"
+#include "public/cpe_lang.h"
 #include "cpelang_priv.h"
 
 #include "common/util.h"
@@ -59,6 +59,7 @@ OSCAP_ITERATOR_GEN(cpe_testexpr)
 struct cpe_lang_model {
 	struct oscap_list *platforms;	// list of items
 	struct oscap_htable *item;	// item by ID
+	char* origin_file; // from which file the lang model was loaded
 };
 OSCAP_IGETTER_GEN(cpe_platform, cpe_lang_model, platforms)
 OSCAP_HGETTER_STRUCT(cpe_platform, cpe_lang_model, item)
@@ -87,12 +88,16 @@ OSCAP_GETTER(const struct cpe_testexpr*, cpe_platform, expr)
 #define TAG_PLATFORM_STR        BAD_CAST "platform"
 #define TAG_LOGICAL_TEST_STR    BAD_CAST "logical-test"
 #define TAG_FACT_REF_STR        BAD_CAST "fact-ref"
+#define TAG_CHECK_FACT_REF_STR  BAD_CAST "check-fact-ref"
 #define TAG_REMARK_STR          BAD_CAST "remark"
 #define ATTR_TITLE_STR      BAD_CAST "title"
 #define ATTR_NAME_STR       BAD_CAST "name"
 #define ATTR_OPERATOR_STR   BAD_CAST "operator"
 #define ATTR_NEGATE_STR     BAD_CAST "negate"
 #define ATTR_ID_STR         BAD_CAST "id"
+#define ATTR_HREF_STR       BAD_CAST "href"
+#define ATTR_SYSTEM_STR     BAD_CAST "system"
+#define ATTR_ID_REF_STR     BAD_CAST "id-ref"
 #define VAL_AND_STR     BAD_CAST "AND"
 #define VAL_OR_STR      BAD_CAST "OR"
 #define VAL_FALSE_STR   BAD_CAST "false"
@@ -155,6 +160,37 @@ static int xmlTextReaderNextElement(xmlTextReaderPtr reader)
 	return ret;
 }
 
+/* Function that jump to next XML starting element.
+ *
+ * This function makes sure we don't go past end tag of given element
+ * */
+static int xmlTextReaderNextElementWE(xmlTextReaderPtr reader, xmlChar* end_tag)
+{
+
+	__attribute__nonnull__(reader);
+
+	int ret;
+	do {
+		ret = xmlTextReaderRead(reader);
+		// if end of file
+		if (ret < 1)
+			break;
+
+		if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_END_ELEMENT) {
+			if (!xmlStrcmp(xmlTextReaderConstLocalName(reader), end_tag)) {
+				ret = 0;
+				break;
+			}
+		}
+	} while (xmlTextReaderNodeType(reader) != XML_READER_TYPE_ELEMENT);
+
+	if (ret == -1) {
+		oscap_setxmlerr(xmlCtxtGetLastError(reader));
+		/* TODO: Should we end here as fatal ? */
+	}
+
+	return ret;
+}
 
 const struct cpe_testexpr *cpe_testexpr_get_next(const struct cpe_testexpr *expr)
 {
@@ -172,26 +208,41 @@ const struct cpe_testexpr *cpe_testexpr_get_next(const struct cpe_testexpr *expr
 
 struct cpe_testexpr *cpe_testexpr_new()
 {
-
 	struct cpe_testexpr *ret;
 
 	ret = oscap_calloc(1, sizeof(struct cpe_testexpr));
 	if (ret == NULL)
 		return NULL;
 
+	ret->oper = CPE_LANG_OPER_AND;
 	ret->meta.expr = NULL;
-	ret->meta.cpe = NULL;
 
 	return ret;
 }
 
 struct cpe_testexpr * cpe_testexpr_clone(struct cpe_testexpr * old_expr)
 {
-        struct cpe_testexpr * new_expr = cpe_testexpr_new();
-        new_expr->oper = old_expr->oper;
-        new_expr->meta.cpe = cpe_name_clone(old_expr->meta.cpe);
-        new_expr->meta.expr = oscap_list_clone(old_expr->meta.expr, (oscap_clone_func) cpe_testexpr_clone);
-        return new_expr;
+	struct cpe_testexpr * new_expr = cpe_testexpr_new();
+	new_expr->oper = old_expr->oper;
+
+	switch (new_expr->oper & CPE_LANG_OPER_MASK) {
+	case CPE_LANG_OPER_AND:
+	case CPE_LANG_OPER_OR:
+		new_expr->meta.expr = oscap_list_clone(old_expr->meta.expr, (oscap_clone_func) cpe_testexpr_clone);
+		break;
+	case CPE_LANG_OPER_MATCH:
+		new_expr->meta.cpe = cpe_name_clone(old_expr->meta.cpe);
+		break;
+	case CPE_LANG_OPER_CHECK:
+		new_expr->meta.check.system = oscap_strdup(old_expr->meta.check.system);
+		new_expr->meta.check.href = oscap_strdup(old_expr->meta.check.href);
+		new_expr->meta.check.id = oscap_strdup(old_expr->meta.check.id);
+		break;
+	default:
+		break;
+	}
+
+	return new_expr;
 }
 
 struct cpe_lang_model *cpe_lang_model_new()
@@ -205,6 +256,7 @@ struct cpe_lang_model *cpe_lang_model_new()
 
 	ret->platforms = oscap_list_new();
 	ret->item = oscap_htable_new();
+	ret->origin_file = NULL;
 
 	return ret;
 }
@@ -253,8 +305,8 @@ static bool cpe_validate_xml(const char *filename)
 	doc = xmlCtxtReadFile(ctxt, filename, NULL, XML_PARSE_DTDATTR);
 	/* check if parsing suceeded */
 	if (doc == NULL) {
-		xmlFreeParserCtxt(ctxt);
 		oscap_setxmlerr(xmlCtxtGetLastError(ctxt));
+		xmlFreeParserCtxt(ctxt);
 		return false;
 	}
 	/* check if validation suceeded */
@@ -284,7 +336,7 @@ struct cpe_lang_model *cpe_lang_model_parse_xml(const char *file)
 		xmlTextReaderNextNode(reader);
 		ret = cpe_lang_model_parse(reader);
 	} else {
-		oscap_seterr(OSCAP_EFAMILY_GLIBC, errno, "Unable to open file.");
+		oscap_seterr(OSCAP_EFAMILY_GLIBC, "Unable to open file: '%s'", file);
 	}
 	xmlFreeTextReader(reader);
 
@@ -307,14 +359,14 @@ struct cpe_lang_model *cpe_lang_model_parse(xmlTextReaderPtr reader)
 			return NULL;
 
 		// skip nodes until new element
-		xmlTextReaderNextElement(reader);
+		xmlTextReaderNextElementWE(reader, TAG_PLATFORM_SPEC_STR);
 
 		while (xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_PLATFORM_STR) == 0) {
 
 			platform = cpe_platform_parse(reader);
 			if (platform)
 				cpe_lang_model_add_platform(ret, platform);
-			xmlTextReaderNextElement(reader);
+			xmlTextReaderNextElementWE(reader, TAG_PLATFORM_SPEC_STR);
 		}
 	}
 
@@ -357,9 +409,11 @@ struct cpe_platform *cpe_platform_parse(xmlTextReaderPtr reader)
 		} else
 		    if (!xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_LOGICAL_TEST_STR) &&
 			xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT) {
+			/* Maybe we shall not allocate this in constructor? */
+			cpe_testexpr_free(ret->expr);
 			ret->expr = cpe_testexpr_parse(reader);
 		} else if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT)
-			oscap_seterr(OSCAP_EFAMILY_OSCAP, OSCAP_EXMLELEM, "Unknown XML element in platform");
+			oscap_seterr(OSCAP_EFAMILY_OSCAP, "Unknown XML element in platform");
 		// get the next node
 		xmlTextReaderNextNode(reader);
 	}
@@ -388,6 +442,14 @@ struct cpe_testexpr *cpe_testexpr_parse(xmlTextReaderPtr reader)
 		ret->meta.cpe = cpe_name_new((char *)temp);
 		xmlFree(temp);
 		return ret;
+	// it's check-fact-ref only, fill the structure and return it
+	} else if (!xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_CHECK_FACT_REF_STR) &&
+	    xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT) {
+		ret->oper = CPE_LANG_OPER_CHECK;
+		ret->meta.check.system = (char*)xmlTextReaderGetAttribute(reader, ATTR_SYSTEM_STR);
+		ret->meta.check.href = (char*)xmlTextReaderGetAttribute(reader, ATTR_HREF_STR);
+		ret->meta.check.id = (char*)xmlTextReaderGetAttribute(reader, ATTR_ID_REF_STR);
+		return ret;
 	} else if (!xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_LOGICAL_TEST_STR) &&
 		   xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT) {
 		// it's logical-test, fill the structure and go to next node
@@ -412,7 +474,7 @@ struct cpe_testexpr *cpe_testexpr_parse(xmlTextReaderPtr reader)
 			ret->oper |= CPE_LANG_OPER_NOT;
 		xmlFree(temp);
 	} else if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT)
-		oscap_seterr(OSCAP_EFAMILY_OSCAP, OSCAP_EXMLELEM, "Unknown XML element in test expression");
+		oscap_seterr(OSCAP_EFAMILY_OSCAP, "Unknown XML element in test expression");
 
 	// go to next node
 	// skip to next node
@@ -432,6 +494,10 @@ struct cpe_testexpr *cpe_testexpr_parse(xmlTextReaderPtr reader)
 		}
 		elem_cnt++;
 
+		// We assume that the expression is a logical one (meaning that it
+		// can have subexpressions).
+		// TODO: Enforce that assumption and don't rely on just validation.
+
 		// .. and the next node is logical-test element, we need recursive call
 		if (!xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_LOGICAL_TEST_STR) &&
 		    xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT) {
@@ -441,8 +507,8 @@ struct cpe_testexpr *cpe_testexpr_parse(xmlTextReaderPtr reader)
                                 return ret;
                         } else if (xmlTextReaderDepth(reader) == depth) continue;
 
-		} else		// .. or it's fact-ref only
-		if (!xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_FACT_REF_STR) &&
+		}
+		else if (!xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_FACT_REF_STR) &&
 			    xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT) {
 			// fill the structure
 			struct cpe_testexpr *subexpr = cpe_testexpr_new();
@@ -452,8 +518,16 @@ struct cpe_testexpr *cpe_testexpr_parse(xmlTextReaderPtr reader)
                         //printf("FACT-REF: %s\n", temp);
 			xmlFree(temp);
 			oscap_list_add(ret->meta.expr, subexpr);
+		} else if (!xmlStrcmp(xmlTextReaderConstLocalName(reader), TAG_CHECK_FACT_REF_STR) &&
+				xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT) {
+			struct cpe_testexpr *subexpr = cpe_testexpr_new();
+			subexpr->oper = CPE_LANG_OPER_CHECK;
+			subexpr->meta.check.system = (char*)xmlTextReaderGetAttribute(reader, ATTR_SYSTEM_STR);
+			subexpr->meta.check.href = (char*)xmlTextReaderGetAttribute(reader, ATTR_HREF_STR);
+			subexpr->meta.check.id = (char*)xmlTextReaderGetAttribute(reader, ATTR_ID_REF_STR);
+			oscap_list_add(ret->meta.expr, subexpr);
 		} else if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT) {
-			oscap_seterr(OSCAP_EFAMILY_OSCAP, OSCAP_EXMLELEM, "Unknown XML element in test expression");
+			oscap_seterr(OSCAP_EFAMILY_OSCAP, "Unknown XML element in test expression");
 		}
 		xmlTextReaderNextNode(reader);
 	}
@@ -483,7 +557,7 @@ static char *parse_text_element(xmlTextReaderPtr reader, char *name)
 			string = (char *)xmlTextReaderValue(reader);
 			break;
 		default:
-			oscap_seterr(OSCAP_EFAMILY_OSCAP, OSCAP_EXMLELEM, "Unknown XML element in platform");
+			oscap_seterr(OSCAP_EFAMILY_OSCAP, "Unknown XML element in platform");
 			break;
 		}
 	}
@@ -566,39 +640,46 @@ void cpe_testexpr_export(const struct cpe_testexpr *expr, xmlTextWriterPtr write
 
 	if (expr->oper == CPE_LANG_OPER_MATCH) {
 		xmlTextWriterStartElementNS(writer, NULL, TAG_FACT_REF_STR, NULL);
-		xmlTextWriterWriteAttribute(writer, ATTR_NAME_STR, BAD_CAST cpe_name_get_uri(expr->meta.cpe));
+		xmlTextWriterWriteAttribute(writer, ATTR_NAME_STR, BAD_CAST cpe_name_get_as_str(expr->meta.cpe));
 		xmlTextWriterEndElement(writer);
-		return;
-	} else {
+	}
+	else if (expr->oper == CPE_LANG_OPER_CHECK) {
+		xmlTextWriterStartElementNS(writer, NULL, TAG_CHECK_FACT_REF_STR, NULL);
+		xmlTextWriterWriteAttribute(writer, ATTR_SYSTEM_STR, BAD_CAST expr->meta.check.system);
+		xmlTextWriterWriteAttribute(writer, ATTR_HREF_STR, BAD_CAST expr->meta.check.href);
+		xmlTextWriterWriteAttribute(writer, ATTR_ID_REF_STR, BAD_CAST expr->meta.check.id);
+		xmlTextWriterEndElement(writer);
+	}
+	else {
 		xmlTextWriterStartElementNS(writer, NULL, TAG_LOGICAL_TEST_STR, CPELANG_NS);
+
+		if (expr->oper == CPE_LANG_OPER_AND) {
+			xmlTextWriterWriteAttribute(writer, ATTR_OPERATOR_STR, VAL_AND_STR);
+			xmlTextWriterWriteAttribute(writer, ATTR_NEGATE_STR, VAL_FALSE_STR);
+		} else if (expr->oper == CPE_LANG_OPER_OR) {
+			xmlTextWriterWriteAttribute(writer, ATTR_OPERATOR_STR, VAL_OR_STR);
+			xmlTextWriterWriteAttribute(writer, ATTR_NEGATE_STR, VAL_FALSE_STR);
+		} else if (expr->oper == CPE_LANG_OPER_NOR) {
+			xmlTextWriterWriteAttribute(writer, ATTR_OPERATOR_STR, VAL_OR_STR);
+			xmlTextWriterWriteAttribute(writer, ATTR_NEGATE_STR, VAL_TRUE_STR);
+		} else if (expr->oper == CPE_LANG_OPER_NAND) {
+			xmlTextWriterWriteAttribute(writer, ATTR_OPERATOR_STR, VAL_AND_STR);
+			xmlTextWriterWriteAttribute(writer, ATTR_NEGATE_STR, VAL_TRUE_STR);
+		} else {
+			/* can this happen? */
+			return;
+		}
+
+		if (expr->meta.expr == NULL)
+			return;
+		OSCAP_FOREACH(cpe_testexpr, subexpr, oscap_iterator_new(expr->meta.expr),
+			cpe_testexpr_export(subexpr, writer);
+		);
+		xmlTextWriterEndElement(writer);
 	}
 
-	if (expr->oper == CPE_LANG_OPER_AND) {
-		xmlTextWriterWriteAttribute(writer, ATTR_OPERATOR_STR, VAL_AND_STR);
-		xmlTextWriterWriteAttribute(writer, ATTR_NEGATE_STR, VAL_FALSE_STR);
-	} else if (expr->oper == CPE_LANG_OPER_OR) {
-		xmlTextWriterWriteAttribute(writer, ATTR_OPERATOR_STR, VAL_OR_STR);
-		xmlTextWriterWriteAttribute(writer, ATTR_NEGATE_STR, VAL_FALSE_STR);
-	} else if (expr->oper == CPE_LANG_OPER_NOR) {
-		xmlTextWriterWriteAttribute(writer, ATTR_OPERATOR_STR, VAL_OR_STR);
-		xmlTextWriterWriteAttribute(writer, ATTR_NEGATE_STR, VAL_TRUE_STR);
-	} else if (expr->oper == CPE_LANG_OPER_NAND) {
-		xmlTextWriterWriteAttribute(writer, ATTR_OPERATOR_STR, VAL_AND_STR);
-		xmlTextWriterWriteAttribute(writer, ATTR_NEGATE_STR, VAL_TRUE_STR);
-	} else {
-		/* can this happen? */
-		return;
-	}
-
-	if (expr->meta.expr == NULL)
-		return;
-	OSCAP_FOREACH(cpe_testexpr, subexpr, oscap_iterator_new(expr->meta.expr),
-		cpe_testexpr_export(subexpr, writer);
-	);
-	xmlTextWriterEndElement(writer);
 	if (xmlGetLastError() != NULL)
 		oscap_setxmlerr(xmlGetLastError());
-
 }
 
 /* End of private export functions
@@ -614,8 +695,10 @@ void cpe_lang_model_free(struct cpe_lang_model *platformspec)
 	if (platformspec == NULL)
 		return;
 
-	oscap_htable_free(platformspec->item, NULL);
+	oscap_htable_free0(platformspec->item);
 	oscap_list_free(platformspec->platforms, (oscap_destruct_func) cpe_platform_free);
+	oscap_free(platformspec->origin_file);
+
 	oscap_free(platformspec);
 }
 
@@ -645,6 +728,14 @@ static void cpe_testexpr_meta_free(struct cpe_testexpr *expr)
 		cpe_name_free(expr->meta.cpe);
 		expr->meta.cpe = NULL;
 		break;
+	case CPE_LANG_OPER_CHECK:
+		oscap_free(expr->meta.check.system);
+		expr->meta.check.system = NULL;
+		oscap_free(expr->meta.check.href);
+		expr->meta.check.href = NULL;
+		oscap_free(expr->meta.check.id);
+		expr->meta.check.id = NULL;
+		break;
 	default:
 		break;
 	}
@@ -684,6 +775,30 @@ const struct cpe_name *cpe_testexpr_get_meta_cpe(const struct cpe_testexpr *expr
 	return expr->meta.cpe;
 }
 
+const char* cpe_testexpr_get_meta_check_system(const struct cpe_testexpr *expr)
+{
+	assert(expr != NULL);
+	if ((expr->oper & CPE_LANG_OPER_MASK) != CPE_LANG_OPER_CHECK)
+		return NULL;
+	return expr->meta.check.system;
+}
+
+const char* cpe_testexpr_get_meta_check_href(const struct cpe_testexpr *expr)
+{
+	assert(expr != NULL);
+	if ((expr->oper & CPE_LANG_OPER_MASK) != CPE_LANG_OPER_CHECK)
+		return NULL;
+	return expr->meta.check.href;
+}
+
+const char* cpe_testexpr_get_meta_check_id(const struct cpe_testexpr *expr)
+{
+	assert(expr != NULL);
+	if ((expr->oper & CPE_LANG_OPER_MASK) != CPE_LANG_OPER_CHECK)
+		return NULL;
+	return expr->meta.check.id;
+}
+
 bool cpe_testexpr_set_oper(struct cpe_testexpr *expr, cpe_lang_oper_t oper)
 {
 	assert(expr != NULL);
@@ -746,3 +861,16 @@ bool cpe_platform_set_expr(struct cpe_platform *platform, struct cpe_testexpr *e
 	platform->expr = expr;
 	return true;
 }
+
+bool cpe_lang_model_set_origin_file(struct cpe_lang_model* lang_model, const char* origin_file)
+{
+	oscap_free(lang_model->origin_file);
+	lang_model->origin_file = oscap_strdup(origin_file);
+
+	return true;
+}
+
+const char* cpe_lang_model_get_origin_file(const struct cpe_lang_model* lang_model)
+{
+	return lang_model->origin_file;
+}
