@@ -25,6 +25,7 @@
  *
  * Authors:
  *      "Peter Vrabec" <pvrabec@redhat.com>
+ *      Šimon Lukašík
  */
 
 #ifdef HAVE_CONFIG_H
@@ -43,17 +44,16 @@
 #include "oval_system_characteristics_impl.h"
 #include "oval_probe_impl.h"
 #include "oval_results_impl.h"
+#include "common/list.h"
 #include "common/util.h"
 #include "common/debug_priv.h"
 #include "common/_error.h"
-#include "common/reporter_priv.h"
-
-#ifdef ENABLE_XCCDF
 #include "oval_agent_xccdf_api.h"
-#endif
 
 struct oval_agent_session {
 	char *filename;
+	char *product_name;
+
 	struct oval_definition_model * def_model;
 	struct oval_variable_model *cur_var_model;
 	struct oval_syschar_model    * sys_model;
@@ -63,7 +63,6 @@ struct oval_agent_session {
 };
 
 
-#ifdef ENABLE_XCCDF
 /**
  * Specification of structure for transformation of OVAL Result type
  * to XCCDF result type.
@@ -71,26 +70,23 @@ struct oval_agent_session {
 struct oval_result_to_xccdf_spec {
 	oval_result_t oval;
 	xccdf_test_result_type_t xccdf;
+	xccdf_test_result_type_t reversed_xccdf;
 };
 
 /**
  * Array of transformation rules from OVAL Result type to XCCDF result type
  */
 static const struct oval_result_to_xccdf_spec XCCDF_OVAL_RESULTS_MAP[] = {
-	{OVAL_RESULT_TRUE, XCCDF_RESULT_PASS},
-	{OVAL_RESULT_FALSE, XCCDF_RESULT_FAIL},
-	{OVAL_RESULT_UNKNOWN, XCCDF_RESULT_UNKNOWN},
-	{OVAL_RESULT_ERROR, XCCDF_RESULT_ERROR},
-	{OVAL_RESULT_NOT_EVALUATED, XCCDF_RESULT_NOT_CHECKED},
-	{OVAL_RESULT_NOT_APPLICABLE, XCCDF_RESULT_NOT_APPLICABLE},
-	{0, 0}
+	{OVAL_RESULT_TRUE, XCCDF_RESULT_PASS, XCCDF_RESULT_FAIL},
+	{OVAL_RESULT_FALSE, XCCDF_RESULT_FAIL, XCCDF_RESULT_PASS},
+	{OVAL_RESULT_UNKNOWN, XCCDF_RESULT_UNKNOWN, XCCDF_RESULT_UNKNOWN},
+	{OVAL_RESULT_ERROR, XCCDF_RESULT_ERROR, XCCDF_RESULT_ERROR},
+	{OVAL_RESULT_NOT_EVALUATED, XCCDF_RESULT_NOT_CHECKED, XCCDF_RESULT_NOT_CHECKED},
+	{OVAL_RESULT_NOT_APPLICABLE, XCCDF_RESULT_NOT_APPLICABLE, XCCDF_RESULT_NOT_APPLICABLE},
+	{0, 0, 0}
 };
 
-#endif
-
-
 oval_agent_session_t * oval_agent_new_session(struct oval_definition_model *model, const char * name) {
-
 	oval_agent_session_t *ag_sess;
 	struct oval_sysinfo *sysinfo;
 	int ret;
@@ -120,9 +116,29 @@ oval_agent_session_t * oval_agent_new_session(struct oval_definition_model *mode
 	ag_sess->sys_models[0] = ag_sess->sys_model;
 	ag_sess->sys_models[1] = NULL;
 	ag_sess->res_model = oval_results_model_new(model, ag_sess->sys_models);
+
+	ag_sess->product_name = NULL;
+
 	return ag_sess;
 }
 
+struct oval_definition_model* oval_agent_get_definition_model(oval_agent_session_t* ag_sess)
+{
+	return ag_sess->def_model;
+}
+
+void oval_agent_set_product_name(oval_agent_session_t *ag_sess, char * product_name)
+{
+	struct oval_generator *generator;
+	ag_sess->product_name = strdup(product_name);
+
+	generator = oval_syschar_model_get_generator(ag_sess->sys_models[0]);
+	oval_generator_set_product_name(generator, product_name);
+
+	generator = oval_results_model_get_generator(ag_sess->res_model);
+	oval_generator_set_product_name(generator, product_name);
+}
+
 int oval_agent_eval_definition(oval_agent_session_t *ag_sess, const char *id)
 {
 	int ret;
@@ -155,8 +171,7 @@ int oval_agent_get_definition_result(oval_agent_session_t *ag_sess, const char *
 	rsystem_it = oval_results_model_get_systems(rmodel);
 	if (!oval_result_system_iterator_has_more(rsystem_it)) {
 		oval_result_system_iterator_free(rsystem_it);
-                dE("No results system in agent sessin.");
-                oscap_seterr(OSCAP_EFAMILY_OSCAP, OVAL_EOVALINT, "Missing result system.");
+		oscap_seterr(OSCAP_EFAMILY_OSCAP, "No results system in agent session.");
                 return -1;
 	}
 
@@ -164,10 +179,7 @@ int oval_agent_get_definition_result(oval_agent_session_t *ag_sess, const char *
 	oval_result_system_iterator_free(rsystem_it);
 	rdef = oval_result_system_get_definition(rsystem, id);
         if (rdef == NULL) {
-                char msg[100];
-                snprintf(msg, sizeof(msg), "No definition with ID: %s in result model.", id);
-                dE(msg);
-                oscap_seterr(OSCAP_EFAMILY_OSCAP, OVAL_EOVALINT, msg);
+                oscap_seterr(OSCAP_EFAMILY_OSCAP, "No definition with ID: %s in result model.", id);
                 return -1;
         }
 
@@ -176,6 +188,26 @@ int oval_agent_get_definition_result(oval_agent_session_t *ag_sess, const char *
 	return 0;
 }
 
+struct oval_result_definition * oval_agent_get_result_definition(oval_agent_session_t *ag_sess, const char *id) {
+	struct oval_results_model *rmodel;
+	struct oval_result_system_iterator *rsystem_it;
+	struct oval_result_system *rsystem;
+	struct oval_result_definition *rdef;
+
+	rmodel = oval_agent_get_results_model(ag_sess);
+	rsystem_it = oval_results_model_get_systems(rmodel);
+	if (!oval_result_system_iterator_has_more(rsystem_it)) {
+		oval_result_system_iterator_free(rsystem_it);
+                return NULL;
+	}
+
+	rsystem = oval_result_system_iterator_next(rsystem_it);
+	oval_result_system_iterator_free(rsystem_it);
+	rdef = oval_result_system_get_definition(rsystem, id);
+
+	return rdef;
+}
+
 int oval_agent_reset_session(oval_agent_session_t * ag_sess) {
 	ag_sess->cur_var_model = NULL;
 	oval_definition_model_clear_external_variables(ag_sess->def_model);
@@ -187,6 +219,14 @@ int oval_agent_reset_session(oval_agent_session_t * ag_sess) {
 	oval_results_model_free(ag_sess->res_model);
 	ag_sess->res_model = oval_results_model_new(ag_sess->def_model, ag_sess->sys_models);
 
+	/* Apply product name to new results_model */
+	if (ag_sess->product_name) {
+		struct oval_generator *generator;
+
+	        generator = oval_results_model_get_generator(ag_sess->res_model);
+        	oval_generator_set_product_name(generator, ag_sess->product_name);
+	}
+
 	oval_probe_session_destroy(ag_sess->psess);
 	ag_sess->psess = oval_probe_session_new(ag_sess->sys_model);
 
@@ -201,12 +241,11 @@ int oval_agent_abort_session(oval_agent_session_t *ag_sess)
 	return oval_probe_session_abort(ag_sess->psess);
 }
 
-int oval_agent_eval_system(oval_agent_session_t * ag_sess, oscap_reporter cb, void *arg) {
+int oval_agent_eval_system(oval_agent_session_t * ag_sess, agent_reporter cb, void *arg) {
 	struct oval_definition *oval_def;
 	struct oval_definition_iterator *oval_def_it;
 	char   *id;
 	int ret = 0;
-	oval_result_t result;
 
 	oval_def_it = oval_definition_model_get_definitions(ag_sess->def_model);
 	while (oval_definition_iterator_has_more(oval_def_it)) {
@@ -216,26 +255,14 @@ int oval_agent_eval_system(oval_agent_session_t * ag_sess, oscap_reporter cb, vo
 		/* probe and eval */
 		ret = oval_agent_eval_definition(ag_sess, id);
 		if (ret==-1) {
-			return -1;
+			goto cleanup;
 		}
 
 		/* callback */
                 if (cb != NULL) {
-			struct oscap_reporter_message * msg = oscap_reporter_message_new_fmt(
-				OSCAP_REPORTER_FAMILY_OVAL, /* FAMILY */
-				0,                           /* CODE */
-				"%s",
-				oval_definition_get_description(oval_def));
-			oscap_reporter_message_set_user1str(msg, id);
-			ret = oval_agent_get_definition_result(ag_sess, id, &result);
-			if (ret == -1) {
-				oscap_reporter_message_free(msg);
-				goto cleanup;
-			}
-
-			oscap_reporter_message_set_user2num(msg, result);
-			oscap_reporter_message_set_user3str(msg, oval_definition_get_title(oval_def));
-			ret = oscap_reporter_report(cb, msg, arg);
+			struct oval_result_definition * res_def = oval_agent_get_result_definition(ag_sess, id);
+			ret = cb(res_def,arg);
+			/* stop? */
 			if (ret!=0)
 				goto cleanup;
 		}
@@ -265,6 +292,8 @@ const char * oval_agent_get_filename(oval_agent_session_t * ag_sess) {
 }
 
 void oval_agent_destroy_session(oval_agent_session_t * ag_sess) {
+	if (ag_sess->product_name)
+		oscap_free(ag_sess->product_name);
 	oval_probe_session_destroy(ag_sess->psess);
 	oval_syschar_model_free(ag_sess->sys_model);
 	oval_results_model_free(ag_sess->res_model);
@@ -274,60 +303,100 @@ void oval_agent_destroy_session(oval_agent_session_t * ag_sess) {
 }
 
 
-#ifdef ENABLE_XCCDF
 /**
  * Function for OVAL Result type -> XCCDF result type transformation
  * @param id OVAL_RESULT_* type
  * @return xccdf_test_result_type_t
  */
-static xccdf_test_result_type_t xccdf_get_result_from_oval(oval_result_t id)
+static xccdf_test_result_type_t xccdf_get_result_from_oval(oval_definition_class_t class, oval_result_t id)
 {
 
 	const struct oval_result_to_xccdf_spec *mapptr;
 
 	for (mapptr = XCCDF_OVAL_RESULTS_MAP; mapptr->oval != 0; ++mapptr) {
 		if (id == mapptr->oval)
-			return mapptr->xccdf;
+			// SP800-126r2: Deriving XCCDF Check Results from OVAL Definition Results
+			return (class == OVAL_CLASS_VULNERABILITY || class == OVAL_CLASS_PATCH) ? mapptr->reversed_xccdf : mapptr->xccdf;
 	}
 
 	return XCCDF_RESULT_UNKNOWN;
 }
 
-int oval_agent_resolve_variables(struct oval_agent_session * session, struct xccdf_value_binding_iterator *it)
+/**
+ * Transform the value_bindings to intermediary mapping.
+ * @param it XCCDF value binding iterator
+ * @return dict structure which for each OVAL variable name contains a list of binded values
+ */
+static struct oscap_htable *_binding_iterator_to_dict(struct xccdf_value_binding_iterator *it)
 {
-    bool conflict = false;
-    int retval = 0;
-    struct oval_value_iterator * value_it;
+	struct oscap_htable *dict = oscap_htable_new();
+	while (xccdf_value_binding_iterator_has_more(it)) {
+		struct xccdf_value_binding *binding = xccdf_value_binding_iterator_next(it);
+		const char *var_name = xccdf_value_binding_get_name(binding);
+		const char *var_val = xccdf_value_binding_get_setvalue(binding);
+		if (var_val == NULL)
+			var_val = xccdf_value_binding_get_value(binding);
+		struct oscap_stringlist *list = (struct oscap_stringlist *) oscap_htable_get(dict, var_name);
+		if (list == NULL) {
+			list = oscap_stringlist_new();
+			oscap_htable_add(dict, var_name, list);
+		}
 
-    if (!xccdf_value_binding_iterator_has_more(it))
-        return 0;
+		if (!oscap_list_contains((struct oscap_list *) list, (char *) var_val, (oscap_cmp_func) oscap_streq)) {
+			oscap_stringlist_add_string(list, var_val);
+		}
+	}
+	xccdf_value_binding_iterator_reset(it);
+	return dict;
+}
 
-    /* Get the definition model from OVAL agent session */
-    struct oval_definition_model *def_model =
-        oval_results_model_get_definition_model(oval_agent_get_results_model(session));
+/**
+ * Ensure that the newly binded values do not pose conflict with existing value set.
+ * The lists should be equal or the existing must be empty.
+ */
+static bool _stringlist_conflicts_with_value_it(struct oscap_stringlist *slist, struct oval_value_iterator *val_it)
+{
+	int multival_count = 0;
+	if (!oval_value_iterator_has_more(val_it))
+		return false;
+	while (oval_value_iterator_has_more(val_it)) {
+		struct oval_value *o_value = oval_value_iterator_next(val_it);
+		char *o_value_text = oval_value_get_text(o_value);
 
-    /* Check the conflict */
-    while (xccdf_value_binding_iterator_has_more(it)) {
-        struct xccdf_value_binding *binding = xccdf_value_binding_iterator_next(it);
-        struct oval_variable *variable = oval_definition_model_get_variable(def_model, xccdf_value_binding_get_name(binding));
-        /* Do we have comflict ? */
-        if (variable != NULL) {
-            value_it = oval_variable_get_values(variable);
-            char * value = xccdf_value_binding_get_setvalue(binding);
-            if (value == NULL) value = xccdf_value_binding_get_value(binding);
-            if (oval_value_iterator_has_more(value_it)) {
-                struct oval_value * o_value = oval_value_iterator_next(value_it);
-                if (strcmp(oval_value_get_text(o_value), value)) {
-                    conflict = true;
-                    oscap_dlprintf(DBG_W, "Variable conflict: %s has different values %s != %s\n", xccdf_value_binding_get_name(binding), oval_value_get_text(o_value), value);
-                }
-                else oscap_dlprintf(DBG_W, "Variable %s has the same value, skipping\n", xccdf_value_binding_get_name(binding));
-            }
-            oval_value_iterator_free(value_it);
-        }
-    }
+		if (!oscap_list_contains((struct oscap_list *) slist, o_value_text, (oscap_cmp_func) oscap_streq))
+			return true;
+		multival_count++;
+	}
+	// It is conflict if the numbers do not match
+	return oscap_list_get_itemcount((struct oscap_list *) slist) != multival_count;
+}
+
+/**
+ * Finds out, if the new batch of variable bindings compel new variable model
+ * (so-called multiset). Creates new variable model if needed.
+ */
+static void _oval_agent_resolve_variables_conflict(struct oval_agent_session *session, struct xccdf_value_binding_iterator *it)
+{
+	const char *var_name = NULL;
+	struct oscap_stringlist *value_list = NULL;
+	bool conflict = false;
+	struct oscap_htable *dict = _binding_iterator_to_dict(it);
+	struct oscap_htable_iterator *hit = oscap_htable_iterator_new(dict);
+	struct oval_definition_model *def_model =
+			oval_results_model_get_definition_model(oval_agent_get_results_model(session));
+	while (!conflict && oscap_htable_iterator_has_more(hit)) {
+		oscap_htable_iterator_next_kv(hit, &var_name, (void*) &value_list);
+		struct oval_variable *variable = oval_definition_model_get_variable(def_model, var_name);
+		if (variable != NULL) {
+			struct oval_value_iterator *value_it = oval_variable_get_values(variable);
+			if (_stringlist_conflicts_with_value_it(value_list, value_it))
+				conflict = true;
+			oval_value_iterator_free(value_it);
+		}
+	}
+	oscap_htable_iterator_free(hit);
+	oscap_htable_free(dict, (oscap_destruct_func) oscap_stringlist_free);
 
-    xccdf_value_binding_iterator_reset(it);
     if (conflict) {
         /* We have a conflict, clear session and external variables */
         oval_agent_reset_session(session);
@@ -336,7 +405,28 @@ int oval_agent_resolve_variables(struct oval_agent_session * session, struct xcc
     if (!session->cur_var_model) {
 	    session->cur_var_model = oval_variable_model_new();
 	    oval_definition_model_bind_variable_model(def_model, session->cur_var_model);
+	    /* Apply generator template to new model */
+	    if (session->product_name) {
+			struct oval_generator *generator;
+
+			generator = oval_variable_model_get_generator(session->cur_var_model);
+			oval_generator_set_product_name(generator, session->product_name);
+	    }
     }
+}
+
+int oval_agent_resolve_variables(struct oval_agent_session * session, struct xccdf_value_binding_iterator *it)
+{
+	int retval = 0;
+
+	if (!xccdf_value_binding_iterator_has_more(it))
+		return 0;
+
+	_oval_agent_resolve_variables_conflict(session, it);
+
+	/* Get the definition model from OVAL agent session */
+	struct oval_definition_model *def_model =
+		oval_results_model_get_definition_model(oval_agent_get_results_model(session));
 
     /* Iterate through variable bindings and add variables into the variable model */
     while (xccdf_value_binding_iterator_has_more(it)) {
@@ -347,18 +437,15 @@ int oval_agent_resolve_variables(struct oval_agent_session * session, struct xcc
         struct oval_variable *variable = oval_definition_model_get_variable(def_model, name);
         if (variable != NULL) {
                 oval_datatype_t o_type = oval_variable_get_datatype(variable);
-                /* TODO: check of variable type ? */
-                if (oval_variable_model_has_variable(session->cur_var_model, name))
-			oscap_dlprintf(DBG_E, "External variable %s in conflict! Probably content failure.\n", name);
-                /* Add variable to variable model */
-                value_it = oval_variable_get_values(variable);
-		if (!oval_value_iterator_has_more(value_it)) {
+		if (!oval_variable_contains_value(variable, value)) {
+			/* Add variable to variable model */
 			oval_variable_model_add(session->cur_var_model, name, "Unknown", o_type, value);
 			oval_variable_bind_ext_var(variable, session->cur_var_model, name);
+			oscap_dlprintf(DBG_I, "Adding external variable %s.\n", name);
 		} else {
-			oscap_dlprintf(DBG_W, "External variable %s in conflict but with same value.\n", name);
+			/* Skip this variable (we assume it has same values otherwise conflict was detected) */
+			oscap_dlprintf(DBG_W, "Skipping external variable %s.\n", name);
 		}
-                oval_value_iterator_free(value_it);
         } else {
                 oscap_dlprintf(DBG_W, "Variable %s does not exist, skipping.\n", name);
         }
@@ -367,26 +454,47 @@ int oval_agent_resolve_variables(struct oval_agent_session * session, struct xcc
     return retval;
 }
 
-static int oval_agent_callback(const struct oscap_reporter_message *msg, void *arg)
+static int
+oval_agent_eval_multi_check(oval_agent_session_t *sess)
 {
+	// This is special handling of evaluation when the process is driven by XCCDF
+	// and xccdf:check/@multi-check=false. We need extra function for this to apply
+	// the XCCDF Truth Table for AND.
+	struct oval_definition *oval_def;
+	struct oval_definition_iterator *oval_def_it;
+	const char *id;
+	oval_result_t oval_result;
+	xccdf_test_result_type_t xccdf_result;
+	xccdf_test_result_type_t final_result = 0;
 
-        /*printf("Evalutated definition %s: %s\n",
-                oscap_reporter_message_get_user1str(msg),
-                oval_result_get_text(oscap_reporter_message_get_user2num(msg)));*/
-	switch ((oval_result_t) oscap_reporter_message_get_user2num(msg)) {
-            case OVAL_RESULT_FALSE:
-            case OVAL_RESULT_UNKNOWN:
-                    (*((int *)arg))++;
-                    break;
-            default:
-                    break;
+	oval_def_it = oval_definition_model_get_definitions(sess->def_model);
+	if (!oval_definition_iterator_has_more(oval_def_it)) {
+		// We are evaluating oval, which has no definitions. We are in state
+		// which is not explicitly covered in SCAP 1.2 Specification, we are
+		// better to report error.
+		final_result = XCCDF_RESULT_ERROR;
 	}
+	while (oval_definition_iterator_has_more(oval_def_it)) {
+		oval_def = oval_definition_iterator_next(oval_def_it);
+		id = oval_definition_get_id(oval_def);
 
-	return 0;
+		// Evaluate definition.
+		assume_r(oval_agent_eval_definition(sess, id) != -1, -1);
+		assume_r(oval_agent_get_definition_result(sess, id, &oval_result) != -1, -1);
+		// Get XCCDF equivalent of the oval result.
+		xccdf_result = xccdf_get_result_from_oval(oval_definition_get_class(oval_def), oval_result);
+		// AND as described in (NISTIR-7275r4): Table 12: Truth Table for AND
+		final_result = (final_result == 0) ? xccdf_result :
+			xccdf_test_result_resolve_and_operation(final_result, xccdf_result);
+	}
+	oval_definition_iterator_free(oval_def_it);
+	return final_result;
 }
 
 xccdf_test_result_type_t oval_agent_eval_rule(struct xccdf_policy *policy, const char *rule_id, const char *id,
-			       const char * href, struct xccdf_value_binding_iterator *it, void *usr)
+			       const char * href, struct xccdf_value_binding_iterator *it,
+			       struct xccdf_check_import_iterator * check_import_it,
+			       void *usr)
 {
         __attribute__nonnull__(usr);
 
@@ -401,70 +509,51 @@ xccdf_test_result_type_t oval_agent_eval_rule(struct xccdf_policy *policy, const
         if (retval != 0) return XCCDF_RESULT_UNKNOWN;
 
         if (id != NULL) {
+		struct oval_definition *definition = oval_definition_model_get_definition(oval_results_model_get_definition_model(oval_agent_get_results_model(sess)), id);
             /* If there is no such OVAL definition, return XCCDF_RESUL_NOT_CHECKED. XDCCDF should look for alternative definition in this case. */
-            if (oval_definition_model_get_definition(oval_results_model_get_definition_model(oval_agent_get_results_model(sess)), id) == NULL)
+            if (definition == NULL)
                     return XCCDF_RESULT_NOT_CHECKED;
             /* Evaluate OVAL definition */
 	    oval_agent_eval_definition(sess, id);
 	    oval_agent_get_definition_result(sess, id, &result);
+		return xccdf_get_result_from_oval(oval_definition_get_class(definition), result);
         } else {
-            int res = 0;
-            oval_agent_eval_system(sess, oval_agent_callback, (void *) &res);
-            if (res == 0) return XCCDF_RESULT_PASS;
-            else return XCCDF_RESULT_FAIL;
+		return oval_agent_eval_multi_check(sess);
         }
-
-	return xccdf_get_result_from_oval(result);
 }
 
+static void *
+_oval_agent_list_definitions(void *usr, xccdf_policy_engine_query_t query_type, void *query_data)
+{
+	__attribute__nonnull__(usr);
+	struct oval_agent_session *sess = (struct oval_agent_session *) usr;
+	if (query_type != POLICY_ENGINE_QUERY_NAMES_FOR_HREF || (query_data != NULL && strcmp(sess->filename, (const char *) query_data)))
+		return NULL;
+	struct oval_definition_iterator *iterator = oval_definition_model_get_definitions(sess->def_model);
+	struct oscap_stringlist *result = oscap_stringlist_new();
+	struct oval_definition *oval_def;
+
+	while (oval_definition_iterator_has_more(iterator)) {
+		oval_def = oval_definition_iterator_next(iterator);
+		oscap_stringlist_add_string(result, oval_definition_get_id(oval_def));
+	}
+
+	oval_definition_iterator_free(iterator);
+	return result;
+}
 
 bool xccdf_policy_model_register_engine_oval(struct xccdf_policy_model * model, struct oval_agent_session * usr)
 {
 
-    return xccdf_policy_model_register_engine_callback(model, "http://oval.mitre.org/XMLSchema/oval-definitions-5", oval_agent_eval_rule, (void *) usr);
+    return xccdf_policy_model_register_engine_and_query_callback(model, "http://oval.mitre.org/XMLSchema/oval-definitions-5",
+		oval_agent_eval_rule, (void *) usr, _oval_agent_list_definitions);
 }
 
 void oval_agent_export_sysinfo_to_xccdf_result(struct oval_agent_session * sess, struct xccdf_result * ritem)
 {
-	struct oval_syschar_model *sys_model = NULL;
-	struct oval_results_model *res_model = NULL;
-	struct oval_result_system *re_system = NULL;
-	struct oval_result_system_iterator *re_system_it = NULL;
-	struct oval_sysinfo *sysinfo = NULL;
+	// just a delegate to maintain API and ABI stability,
+	// this has been implemented elsewhere and this function is deprecated!
 
-	__attribute__nonnull__(sess);
-
-        /* Get all models we will need */
-	res_model = oval_agent_get_results_model(sess);
-
-        /* Get the very first system */
-        re_system_it = oval_results_model_get_systems(res_model);
-	re_system = oval_result_system_iterator_next(re_system_it);
-        oval_result_system_iterator_free(re_system_it);
-
-	sys_model = oval_result_system_get_syschar_model(re_system);
-	sysinfo = oval_syschar_model_get_sysinfo(sys_model);
-
-        /* Set the test system */
-        xccdf_result_add_target(ritem, oval_sysinfo_get_primary_host_name(sysinfo));
-
-	struct xccdf_target_fact *fact = NULL;
-	struct oval_sysint *sysint = NULL;
-
-        /* Add info for all interfaces */
-	struct oval_sysint_iterator *sysint_it = oval_sysinfo_get_interfaces(sysinfo);
-	while (oval_sysint_iterator_has_more(sysint_it)) {
-		sysint = oval_sysint_iterator_next(sysint_it);
-		xccdf_result_add_target_address(ritem, oval_sysint_get_ip_address(sysint));
-
-		if (oval_sysint_get_mac_address(sysint) != NULL) {
-			fact = xccdf_target_fact_new();
-			xccdf_target_fact_set_name(fact, "urn:xccdf:fact:ethernet:MAC");
-			xccdf_target_fact_set_string(fact, oval_sysint_get_mac_address(sysint));
-			xccdf_result_add_target_fact(ritem, fact);
-		}
-	}
-	oval_sysint_iterator_free(sysint_it);
+	xccdf_result_fill_sysinfo(ritem);
 }
 
-#endif
