@@ -199,6 +199,7 @@ void oval_component_to_print(struct oval_component *component, char *indent, int
 	OVAL_FUNCTION_ESCAPE_REGEX = OVAL_FUNCTION + 7,
 	OVAL_FUNCTION_REGEX_CAPTURE = OVAL_FUNCTION + 8,
 	OVAL_FUNCTION_ARITHMETIC = OVAL_FUNCTION + 9
+	OVAL_FUNCTION_COUNT = OVAL_FUNCTION + 10,
  */
 static const struct oscap_string_map _OVAL_COMPONENT_MAP[] = {
 	{OVAL_COMPONENT_LITERAL, "literal_component"},
@@ -217,6 +218,8 @@ static const struct oscap_string_map _OVAL_FUNCTION_MAP[] = {
 	{OVAL_FUNCTION_ESCAPE_REGEX, "escape_regex"},
 	{OVAL_FUNCTION_REGEX_CAPTURE, "regex_capture"},
 	{OVAL_FUNCTION_ARITHMETIC, "arithmetic"},
+	{OVAL_FUNCTION_COUNT, "count"},
+	{OVAL_FUNCTION_UNIQUE, "unique"},
 	{0, NULL}
 };
 
@@ -840,10 +843,13 @@ void oval_component_free(struct oval_component *component)
 		};
 		break;
 	case OVAL_FUNCTION_CONCAT:
+	case OVAL_FUNCTION_COUNT:
+	case OVAL_FUNCTION_UNIQUE:
 	case OVAL_FUNCTION_SUBSTRING:
 	case OVAL_FUNCTION_TIMEDIF:
 	case OVAL_FUNCTION_ESCAPE_REGEX:
 	case OVAL_FUNCTION_ARITHMETIC:
+	case OVAL_FUNCTION_LAST:
 	case OVAL_COMPONENT_VARREF:
 	case OVAL_COMPONENT_FUNCTION:
 	case OVAL_COMPONENT_UNKNOWN:
@@ -1071,6 +1077,12 @@ int oval_component_parse_tag(xmlTextReaderPtr reader,
 	} else if (strcmp(tagname, "concat") == 0) {
 		component = oval_component_new(model, OVAL_FUNCTION_CONCAT);
 		return_code = _oval_component_parse_FUNCTION_tag(reader, context, component);
+	} else if (strcmp(tagname, "count") == 0) {
+		component = oval_component_new(model, OVAL_FUNCTION_COUNT);
+		return_code = _oval_component_parse_FUNCTION_tag(reader, context, component);
+	} else if (strcmp(tagname, "unique") == 0) {
+		component = oval_component_new(model, OVAL_FUNCTION_UNIQUE);
+		return_code = _oval_component_parse_FUNCTION_tag(reader, context, component);
 	} else if (strcmp(tagname, "end") == 0) {
 		component = oval_component_new(model, OVAL_FUNCTION_END);
 		return_code = _oval_component_parse_BEGEND_tag(reader, context, component);
@@ -1238,7 +1250,6 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_OBJECTREF(oval_ar
 									 struct oval_component *component,
 									 struct oval_collection *value_collection)
 {
-#ifdef ENABLE_PROBES
 	__attribute__nonnull__(component);
 
 	oval_syschar_collection_flag_t flag = SYSCHAR_FLAG_ERROR;
@@ -1321,11 +1332,6 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_OBJECTREF(oval_ar
 	}
 
 	return flag;
-#else
-	oscap_seterr(OSCAP_EFAMILY_OSCAP, OSCAP_ENOTIMPL, "This feature is not implemented, compiled without probes support.");
-	oval_syschar_collection_flag_t flag = SYSCHAR_FLAG_ERROR;
-	return flag;
-#endif
 }
 
 #define _HAS_VALUES(flag) (flag==SYSCHAR_FLAG_COMPLETE || flag==SYSCHAR_FLAG_INCOMPLETE)
@@ -1340,8 +1346,7 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_VARREF(oval_argu_
 	struct oval_variable *variable = varref->variable;
 
 	if (!variable) {
-		oscap_dlprintf(DBG_E, "No variable bound to VARREF Component.\n");
-		oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EOVALINT, "No variable bound to VARREF componenet");
+		oscap_seterr(OSCAP_EFAMILY_OVAL, "No variable bound to VARREF componenet");
 		return flag;
 	}
 
@@ -1397,8 +1402,7 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_BEGIN(oval_argu_t
 		}
 		oval_component_iterator_free(subcomps);
 	} else {
-		oscap_dlprintf(DBG_E, "No prefix specified for begin function.\n");
-		oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EOVALINT, "No prefix specified for component evaluation");
+		oscap_seterr(OSCAP_EFAMILY_OVAL, "No prefix specified for component evaluation");
 	}
 	return flag;
 }
@@ -1434,8 +1438,7 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_END(oval_argu_t *
 		}
 		oval_component_iterator_free(subcomps);
 	} else {
-		oscap_dlprintf(DBG_E, "No suffix specified for end function.\n");
-		oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EOVALINT, "No suffix specified for component evaluation");
+		oscap_seterr(OSCAP_EFAMILY_OVAL, "No suffix specified for component evaluation");
 	}
 	return flag;
 }
@@ -1445,7 +1448,6 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_CONCAT(oval_argu_
 								      struct oval_collection *value_collection)
 {
 	int idx0 = 0;
-	int passnum = 0;
 
 	oval_syschar_collection_flag_t flag = SYSCHAR_FLAG_UNKNOWN;
 	struct oval_component_iterator *subcomps = oval_component_get_function_components(component);
@@ -1484,7 +1486,7 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_CONCAT(oval_argu_
 				texts[idx0] = NULL;
 			}
 		}
-		for (passnum = 1; passnum - 1 < catnum; passnum++) {
+		for (int passnum = 1; passnum - 1 < catnum; passnum++) {
 			int len_cat = 1;
 			for (idx0 = 0; idx0 < len_subcomps; idx0++)
 				if (texts[idx0])
@@ -1526,6 +1528,100 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_CONCAT(oval_argu_
 	return flag;
 }
 
+static oval_syschar_collection_flag_t _oval_component_evaluate_COUNT(oval_argu_t *argu,
+								      struct oval_component *component,
+								      struct oval_collection *value_collection)
+{
+	int count = 0;
+	int idx0 = 0;
+
+	oval_syschar_collection_flag_t flag = SYSCHAR_FLAG_UNKNOWN;
+	struct oval_component_iterator *subcomps = oval_component_get_function_components(component);
+	int len_subcomps = oval_component_iterator_remaining(subcomps);
+	struct oval_collection *component_colls[len_subcomps];
+	for (idx0 = 0; oval_component_iterator_has_more(subcomps); idx0++) {
+		struct oval_collection *subcoll = oval_collection_new();
+		struct oval_component *subcomp = oval_component_iterator_next(subcomps);
+		oval_syschar_collection_flag_t subflag = oval_component_eval_common(argu, subcomp, subcoll);
+		flag = _AGG_FLAG(flag, subflag);
+		component_colls[idx0] = subcoll;
+	}
+	bool not_finished = (len_subcomps > 0) && _HAS_VALUES(flag);
+	if (not_finished) {
+		for (idx0 = 0; idx0 < len_subcomps; idx0++) {
+			struct oval_value_iterator *comp_values =
+			    (struct oval_value_iterator *)oval_collection_iterator(component_colls[idx0]);
+			while (oval_value_iterator_has_more(comp_values)) {
+				if (oval_value_get_text(oval_value_iterator_next(comp_values)))
+					count++;
+			}
+			oval_value_iterator_free(comp_values);
+
+		}
+	}
+	char count_str[128];
+	snprintf(count_str, sizeof(count_str), "%d", count);
+	struct oval_value *value = oval_value_new(OVAL_DATATYPE_INTEGER, count_str);
+	oval_collection_add(value_collection, value);
+
+	oval_component_iterator_free(subcomps);
+
+	for (idx0 = 0; idx0 < len_subcomps; ++idx0)
+	  oval_collection_free_items(component_colls[idx0], (oscap_destruct_func) oval_value_free);
+
+	return flag;
+}
+
+static oval_syschar_collection_flag_t _oval_component_evaluate_UNIQUE(oval_argu_t *argu,
+								      struct oval_component *component,
+								      struct oval_collection *value_collection)
+{
+	int idx0 = 0;
+	oval_syschar_collection_flag_t flag = SYSCHAR_FLAG_UNKNOWN;
+	struct oval_component_iterator *subcomps = oval_component_get_function_components(component);
+	int len_subcomps = oval_component_iterator_remaining(subcomps);
+	struct oval_collection *component_colls[len_subcomps];
+
+	for (idx0 = 0; oval_component_iterator_has_more(subcomps); idx0++) {
+		struct oval_collection *subcoll = oval_collection_new();
+		struct oval_component *subcomp = oval_component_iterator_next(subcomps);
+		oval_syschar_collection_flag_t subflag = oval_component_eval_common(argu, subcomp, subcoll);
+		flag = _AGG_FLAG(flag, subflag);
+		component_colls[idx0] = subcoll;
+	}
+
+	oval_component_iterator_free(subcomps);
+
+	bool not_finished = (len_subcomps > 0) && _HAS_VALUES(flag);
+	struct oval_string_map *valmap = oval_string_map_new();
+
+	if (not_finished) {
+		for (idx0 = 0; idx0 < len_subcomps; idx0++) {
+			struct oval_value_iterator *comp_values =
+			    (struct oval_value_iterator *)oval_collection_iterator(component_colls[idx0]);
+
+			while (oval_value_iterator_has_more(comp_values)) {
+				char *valtxt;
+				if ((valtxt = oval_value_get_text(oval_value_iterator_next(comp_values))) != NULL) {
+				  if (oval_string_map_get_value(valmap, valtxt) == NULL) {
+					struct oval_value *value = oval_value_new(OVAL_DATATYPE_STRING, valtxt);
+					oval_string_map_put(valmap, valtxt, value);
+				  }
+				}
+			}
+			oval_value_iterator_free(comp_values);
+		}
+	}
+
+	oval_string_map_collect_values(valmap, value_collection);
+	oval_string_map_free(valmap, NULL);
+
+	for (idx0 = 0; idx0 < len_subcomps; ++idx0)
+	  oval_collection_free_items(component_colls[idx0], (oscap_destruct_func) oval_value_free);
+
+	return flag;
+}
+
 static oval_syschar_collection_flag_t _oval_component_evaluate_SPLIT(oval_argu_t *argu,
 								     struct oval_component *component,
 								     struct oval_collection *value_collection)
@@ -2011,8 +2107,7 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_ARITHMETIC_rec(st
 		} else if (dt == OVAL_DATATYPE_FLOAT) {
 			new_val = (double) oval_value_get_float(ov);
 		} else {
-			oscap_dlprintf(DBG_E, "Unexpected value type: %s.\n", oval_datatype_get_text(dt));
-			oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EOVALINT, "Unexpected value type");
+			oscap_seterr(OSCAP_EFAMILY_OVAL, "Unexpected value type: %s.\n", oval_datatype_get_text(dt));
 			oval_value_iterator_free(val_itr);
 			return SYSCHAR_FLAG_ERROR;
 		}
@@ -2022,8 +2117,7 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_ARITHMETIC_rec(st
 		} else if (op == OVAL_ARITHMETIC_MULTIPLY) {
 			new_val *= val;
 		} else {
-			oscap_dlprintf(DBG_E, "Unexpected arithmetic operation: %s.\n", oval_arithmetic_operation_get_text(op));
-			oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EOVALINT, "Unexpected arithmetic operation");
+			oscap_seterr(OSCAP_EFAMILY_OVAL, "Unexpected arithmetic operation: %s.\n", oval_arithmetic_operation_get_text(op));
 			oval_value_iterator_free(val_itr);
 			return SYSCHAR_FLAG_ERROR;
 		}
@@ -2049,8 +2143,7 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_ARITHMETIC(oval_a
 
 	op = oval_component_get_arithmetic_operation(component);
 	if (op != OVAL_ARITHMETIC_ADD && op != OVAL_ARITHMETIC_MULTIPLY) {
-		oscap_dlprintf(DBG_E, "Unexpected arithmetic operation: %s.\n", oval_arithmetic_operation_get_text(op));
-		oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EOVALINT, "Unexpected arithmetic operation");
+		oscap_seterr(OSCAP_EFAMILY_OVAL, "Unexpected arithmetic operation: %s.\n", oval_arithmetic_operation_get_text(op));
 		return SYSCHAR_FLAG_ERROR;
 	}
 
@@ -2084,8 +2177,7 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_ARITHMETIC(oval_a
 		} else if (datatype == OVAL_DATATYPE_FLOAT) {
 			val = (double) oval_value_get_float(ov);
 		} else {
-			oscap_dlprintf(DBG_E, "Unexpected value type: %s.\n", oval_datatype_get_text(datatype));
-			oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EOVALINT, "Unexpected value type");
+			oscap_seterr(OSCAP_EFAMILY_OVAL, "Unexpected value type: %s.\n", oval_datatype_get_text(datatype));
 			flag = SYSCHAR_FLAG_ERROR;
 			goto cleanup;
 		}
@@ -2108,7 +2200,7 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_ARITHMETIC(oval_a
 typedef oval_syschar_collection_flag_t(_oval_component_evaluator)
  (oval_argu_t *, struct oval_component *, struct oval_collection *);
 
-static _oval_component_evaluator *_component_evaluators[15] = {
+static _oval_component_evaluator *_component_evaluators[] = {
 	NULL,
 	_oval_component_evaluate_LITERAL,
 	_oval_component_evaluate_OBJECTREF,
@@ -2123,6 +2215,8 @@ static _oval_component_evaluator *_component_evaluators[15] = {
 	_oval_component_evaluate_ESCAPE_REGEX,
 	_oval_component_evaluate_REGEX_CAPTURE,
 	_oval_component_evaluate_ARITHMETIC,
+	_oval_component_evaluate_COUNT,
+	_oval_component_evaluate_UNIQUE,
 	NULL
 };
 
@@ -2134,14 +2228,13 @@ static oval_syschar_collection_flag_t oval_component_eval_common(oval_argu_t *ar
 
 	oval_component_type_t type = component->type;
 	int evidx = (type > OVAL_FUNCTION) ? type - OVAL_FUNCTION + OVAL_COMPONENT_FUNCTION : type;
-	_oval_component_evaluator *evaluator = (OVAL_COMPONENT_LITERAL <= type && type <= OVAL_FUNCTION_ARITHMETIC)
+	_oval_component_evaluator *evaluator = (OVAL_COMPONENT_LITERAL <= type && type <= OVAL_FUNCTION_LAST)
 	    ? _component_evaluators[evidx] : NULL;
 	oval_syschar_collection_flag_t flag = SYSCHAR_FLAG_ERROR;
 	if (evaluator) {
 		flag = (*evaluator) (argu, component, value_collection);
 	} else {
-		oscap_dlprintf(DBG_E, "Component type %d not supported.\n", type);
-		oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EOVALINT, "Component type not supported");
+		oscap_seterr(OSCAP_EFAMILY_OVAL, "Component type %d not supported.\n", type);
 	}
 	return flag;
 }
