@@ -44,7 +44,6 @@
 #include "common/util.h"
 #include "common/debug_priv.h"
 #include "common/_error.h"
-#include "common/reporter_priv.h"
 #include "common/elements.h"
 
 typedef struct oval_definition_model {
@@ -55,54 +54,14 @@ typedef struct oval_definition_model {
 	struct oval_string_map *state_map;
 	struct oval_string_map *variable_map;
 	struct oval_collection *bound_variable_models;
-	xmlDoc *metadata_doc;
         char *schema;
 } oval_definition_model_t;
 
-/* TODO: Use this to generate links
-static char * _oval_generate_schema_location(const char * version)
-{
-
-        char * schema = oscap_alloc(sizeof(char)*(strlen(OVAL_DEF_SCHEMA_LOCATION_DEF_PX)+strlen(version)+strlen(OVAL_DEF_SCHEMA_LOCATION_DEF_SX)+2));
-        strncat(schema, OVAL_DEF_SCHEMA_LOCATION_DEF_PX, strlen(OVAL_DEF_SCHEMA_LOCATION_DEF_PX));
-        strncat(schema, version, strlen(version));
-        strncat(schema, OVAL_DEF_SCHEMA_LOCATION_DEF_SX, strlen(OVAL_DEF_SCHEMA_LOCATION_DEF_SX));
-        strncat(schema, " ", 1);
-
-        oscap_realloc(schema, sizeof(char)*(strlen(schema)+strlen(OVAL_DEF_SCHEMA_LOCATION_IND_PX)+strlen(version)+strlen(OVAL_DEF_SCHEMA_LOCATION_IND_SX)+1));
-        strncat(schema, OVAL_DEF_SCHEMA_LOCATION_IND_PX, strlen(OVAL_DEF_SCHEMA_LOCATION_IND_PX));
-        strncat(schema, version, strlen(version));
-        strncat(schema, OVAL_DEF_SCHEMA_LOCATION_IND_SX, strlen(OVAL_DEF_SCHEMA_LOCATION_IND_SX));
-        strncat(schema, " ", 1);
-
-        oscap_realloc(schema, sizeof(char)*(strlen(schema)+strlen(OVAL_DEF_SCHEMA_LOCATION_UNX_PX)+strlen(version)+strlen(OVAL_DEF_SCHEMA_LOCATION_UNX_SX)+1));
-        strncat(schema, OVAL_DEF_SCHEMA_LOCATION_UNX_PX, strlen(OVAL_DEF_SCHEMA_LOCATION_UNX_PX));
-        strncat(schema, version, strlen(version));
-        strncat(schema, OVAL_DEF_SCHEMA_LOCATION_UNX_SX, strlen(OVAL_DEF_SCHEMA_LOCATION_UNX_SX));
-        strncat(schema, " ", 1);
-
-        oscap_realloc(schema, sizeof(char)*(strlen(schema)+strlen(OVAL_DEF_SCHEMA_LOCATION_LNX_PX)+strlen(version)+strlen(OVAL_DEF_SCHEMA_LOCATION_LNX_SX)+1));
-        strncat(schema, OVAL_DEF_SCHEMA_LOCATION_LNX_PX, strlen(OVAL_DEF_SCHEMA_LOCATION_LNX_PX));
-        strncat(schema, version, strlen(version));
-        strncat(schema, OVAL_DEF_SCHEMA_LOCATION_LNX_SX, strlen(OVAL_DEF_SCHEMA_LOCATION_LNX_SX));
-        strncat(schema, " ", 1);
-
-        oscap_realloc(schema, sizeof(char)*(strlen(schema)+strlen(OVAL_DEF_SCHEMA_LOCATION_CMN_PX)+strlen(version)+strlen(OVAL_DEF_SCHEMA_LOCATION_CMN_SX)+1));
-        strncat(schema, OVAL_DEF_SCHEMA_LOCATION_CMN_PX, strlen(OVAL_DEF_SCHEMA_LOCATION_CMN_PX));
-        strncat(schema, version, strlen(version));
-        strncat(schema, OVAL_DEF_SCHEMA_LOCATION_CMN_SX, strlen(OVAL_DEF_SCHEMA_LOCATION_CMN_SX));
-
-
-        return schema;
-}*/
-
 /* failed   - NULL
  * success  - oval_definition_model
  * */
 struct oval_definition_model *oval_definition_model_new()
 {
-	xmlNode *root;
-	xmlNs *ns;
 	oval_definition_model_t *newmodel = (oval_definition_model_t *) oscap_alloc(sizeof(oval_definition_model_t));
 	if (newmodel == NULL)
 		return NULL;
@@ -116,19 +75,13 @@ struct oval_definition_model *oval_definition_model_new()
 	newmodel->bound_variable_models = NULL;
         newmodel->schema = strdup(OVAL_DEF_SCHEMA_LOCATION);
 
-	root = xmlNewNode(NULL, BAD_CAST "root");
-	ns = xmlNewNs(root, OVAL_DEFINITIONS_NAMESPACE, NULL);
-	xmlSetNs(root, ns);
-	newmodel->metadata_doc = xmlNewDoc(BAD_CAST "1.0");
-	xmlDocSetRootElement(newmodel->metadata_doc, root);
-
 	return newmodel;
 }
 
 typedef void (*_oval_clone_func) (void *, struct oval_definition_model *);
 
-static void _oval_definition_model_clone(struct oval_string_map *oldmap, 
-					 struct oval_definition_model *newmodel, 
+static void _oval_definition_model_clone(struct oval_string_map *oldmap,
+					 struct oval_definition_model *newmodel,
 					 _oval_clone_func cloner)
 {
 	struct oval_string_iterator *keys = (struct oval_string_iterator *)oval_string_map_keys(oldmap);
@@ -159,7 +112,6 @@ struct oval_definition_model *oval_definition_model_clone(struct oval_definition
 	_oval_definition_model_clone(oldmodel->test_map, newmodel, (_oval_clone_func) oval_test_clone);
 	_oval_definition_model_clone
 	    (oldmodel->variable_map, newmodel, (_oval_clone_func) oval_variable_clone);
-	newmodel->metadata_doc = xmlCopyDoc(oldmodel->metadata_doc, 1);
         newmodel->schema = oscap_strdup(oldmodel->schema);
 	return newmodel;
 }
@@ -177,8 +129,6 @@ void oval_definition_model_free(struct oval_definition_model *model)
 		oval_collection_free_items(model->bound_variable_models,
 					   (oscap_destruct_func) oval_variable_model_free);
 
-	xmlFreeDoc(model->metadata_doc);
-
         if (model->schema != NULL)
             oscap_free(model->schema);
 
@@ -187,7 +137,6 @@ void oval_definition_model_free(struct oval_definition_model *model)
 	model->state_map = NULL;
 	model->test_map = NULL;
 	model->variable_map = NULL;
-	model->metadata_doc = NULL;
         model->schema = NULL;
 
 	oval_generator_free(model->generator);
@@ -275,9 +224,7 @@ int oval_definition_model_merge(struct oval_definition_model *model, const char
 
 	xmlTextReader *reader = xmlNewTextReaderFilename(file);
 	if (reader == NULL) {
-		if(errno)
-			oscap_seterr(OSCAP_EFAMILY_GLIBC, errno, strerror(errno));
-		oscap_dlprintf(DBG_E, "Unable to open file.\n");
+		oscap_seterr(OSCAP_EFAMILY_GLIBC, "%s '%s'", strerror(errno), file);
 		return -1;
 	}
 
@@ -288,7 +235,7 @@ int oval_definition_model_merge(struct oval_definition_model *model, const char
 	context.user_data = NULL;
 	xmlTextReaderSetErrorHandler(reader, &libxml_error_handler, &context);
 	/* jump into oval_definitions */
-	xmlTextReaderRead(reader);
+	while (xmlTextReaderRead(reader) == 1 && xmlTextReaderNodeType(reader) != XML_READER_TYPE_ELEMENT) ;
 	/* start parsing */
 	ret = oval_definition_model_parse(reader, &context);
 	xmlFreeTextReader(reader);
@@ -482,20 +429,15 @@ struct oval_test *oval_definition_model_get_new_test(struct oval_definition_mode
 	return test;
 }
 
-xmlDoc *oval_definition_model_get_metadata_doc(struct oval_definition_model *model)
-{
-	return model->metadata_doc;
-}
-
 xmlNode *oval_definition_model_to_dom(struct oval_definition_model *definition_model, xmlDocPtr doc, xmlNode * parent)
 {
 
 	xmlNodePtr root_node = NULL;
 
 	if (parent) { /* result file */
-		root_node = xmlNewTextChild(parent, NULL, BAD_CAST "oval_definitions", NULL);
+		root_node = xmlNewTextChild(parent, NULL, BAD_CAST OVAL_ROOT_ELM_DEFINITIONS, NULL);
 	} else {      /* definitions file, we are the root */
-		root_node = xmlNewNode(NULL, BAD_CAST "oval_definitions");
+		root_node = xmlNewNode(NULL, BAD_CAST OVAL_ROOT_ELM_DEFINITIONS);
 		xmlDocSetRootElement(doc, root_node);
 	}
 	xmlNewProp(root_node, BAD_CAST "xsi:schemaLocation", BAD_CAST definition_model->schema);
@@ -586,8 +528,6 @@ int oval_definition_model_export(struct oval_definition_model *model, const char
 
 	__attribute__nonnull__(model);
 
-	int retcode = 0;
-
 	LIBXML_TEST_VERSION;
 
 	xmlDocPtr doc = xmlNewDoc(BAD_CAST "1.0");
@@ -597,14 +537,7 @@ int oval_definition_model_export(struct oval_definition_model *model, const char
 	}
 
 	oval_definition_model_to_dom(model, doc, NULL);
-	retcode = xmlSaveFormatFileEnc(file, doc, "UTF-8", 1);
-
-	if (retcode < 1)
-		oscap_setxmlerr(xmlGetLastError());
-
-	xmlFreeDoc(doc);
-
-	return retcode;
+	return oscap_xml_save_filename(file, doc);
 }
 
 static void _fp_set_recurse(struct oval_definition_model *model, struct oval_setobject *set, char *set_id)
