@@ -154,7 +154,6 @@ int oval_generator_parse_tag(xmlTextReader *, struct oval_parser_context *, void
 xmlNode *oval_generator_to_dom(struct oval_generator *, xmlDocPtr, xmlNode *);
 
 /* definition_model */
-xmlDoc *oval_definition_model_get_metadata_doc(struct oval_definition_model *);
 xmlNode *oval_definition_model_to_dom(struct oval_definition_model *definition_model, xmlDocPtr doc, xmlNode * parent);
 void oval_definition_model_optimize_by_filter_propagation(struct oval_definition_model *);
 
@@ -175,6 +174,7 @@ void oval_definition_model_set_schema(struct oval_definition_model *model, const
 /* variable model */
 struct oval_collection *oval_variable_model_get_values_ref(struct oval_variable_model *, char *);
 int oval_variable_bind_ext_var(struct oval_variable *, struct oval_variable_model *, char *);
+bool oval_variable_contains_value(struct oval_variable *variable, const char* o_value_text);
 
 //Synthetic object subtype for probing system info.
 #define  OVAL_INDEPENDENT_SYSCHAR_SUBTYPE 7999
