@@ -38,6 +38,8 @@
 #include "oval_definitions_impl.h"
 #include "oval_collection_impl.h"
 #include "oval_agent_api_impl.h"
+#include "oval_parser_impl.h"
+
 #include "common/util.h"
 #include "common/debug_priv.h"
 #include "common/_error.h"
@@ -357,10 +359,9 @@ xmlNode *oval_entity_to_dom(struct oval_entity *entity, xmlDoc * doc, xmlNode *
 	bool mask = oval_entity_get_mask(entity);
 
 	/* omit the value and operation used for testing in oval_results if mask=true */
-	if(mask && !xmlStrcmp(root_node->name, (const xmlChar *) "oval_results")){
+	if (mask && !xmlStrcmp(root_node->name, BAD_CAST OVAL_ROOT_ELM_RESULTS)) {
 		entity_node = xmlNewTextChild(parent, ent_ns, BAD_CAST tagname, BAD_CAST "");
-	}
-	else {
+	} else {
 		entity_node = xmlNewTextChild(parent, ent_ns, BAD_CAST tagname, BAD_CAST content);
 		oval_operation_t operation = oval_entity_get_operation(entity);
 		if (operation != OVAL_OPERATION_EQUALS)
