@@ -335,6 +335,7 @@ static const struct oscap_string_map OVAL_COMPONENT_TYPE_MAP[] = {
 	{OVAL_FUNCTION_ESCAPE_REGEX, "escape_regex"},
 	{OVAL_FUNCTION_REGEX_CAPTURE, "regex_capture"},
 	{OVAL_FUNCTION_ARITHMETIC, "arithmetic"},
+	{OVAL_FUNCTION_COUNT, "count"},
 	{OVAL_ENUMERATION_INVALID, NULL}
 };
 
@@ -368,12 +369,20 @@ oval_family_t oval_family_parse(xmlTextReaderPtr reader)
 	char *namespace = (char *)xmlTextReaderNamespaceUri(reader);
 	char *family_text = strrchr(namespace, '#');
 	if (family_text == NULL) {
+		dW("No OVAL family for namespace: %s\n", namespace);
 		oscap_free(namespace);
 		return OVAL_FAMILY_UNKNOWN;
 	}
+
 	int ret = oscap_string_to_enum(OVAL_FAMILY_MAP, ++family_text);
+
+	if (ret == OVAL_ENUMERATION_INVALID) {
+		dW("Unknown OVAL family: %s\n", family_text);
+		ret = OVAL_FAMILY_UNKNOWN;
+	}
+
 	oscap_free(namespace);
-	return (ret != OVAL_ENUMERATION_INVALID ? ret : OVAL_FAMILY_UNKNOWN);
+	return ret;
 }
 
 const char *oval_family_get_text(oval_family_t family)
@@ -404,7 +413,9 @@ static const struct oscap_string_map OVAL_SUBTYPE_CATOS_MAP[] = {
 
 static const struct oscap_string_map OVAL_SUBTYPE_ESX_MAP[] = {
 	{OVAL_ESX_PATCH, "patch"},
+	{OVAL_ESX_PATCH56, "patch56"},
 	{OVAL_ESX_VERSION, "version"},
+	{OVAL_ESX_VISDKMANAGEDOBJECT, "visdkmanagedobject"},
 	{OVAL_SUBTYPE_UNKNOWN, NULL}
 };
 
@@ -419,6 +430,9 @@ static const struct oscap_string_map OVAL_SUBTYPE_HPUX_MAP[] = {
 	{OVAL_HPUX_PATCH, "patch"},
 	{OVAL_HPUX_SOFTWARE_LIST, "softwarelist"},
 	{OVAL_HPUX_TRUSTED_ACCOUNTS, "trustedaccounts"},
+	{OVAL_HPUX_NDD, "ndd"},
+	{OVAL_HPUX_SWLIST, "swlist"},
+	{OVAL_HPUX_TRUSTED, "trusted"},
 	{OVAL_SUBTYPE_UNKNOWN, NULL}
 };
 
@@ -455,11 +469,14 @@ static const struct oscap_string_map OVAL_SUBTYPE_IOS_MAP[] = {
 static const struct oscap_string_map OVAL_SUBTYPE_LINUX_MAP[] = {
 	{OVAL_LINUX_DPKG_INFO, "dpkginfo"},
 	{OVAL_LINUX_INET_LISTENING_SERVERS, "inetlisteningservers"},
+	{OVAL_LINUX_INET_LISTENING_SERVER, "inetlisteningserver"},
 	{OVAL_LINUX_RPM_INFO, "rpminfo"},
 	{OVAL_LINUX_SLACKWARE_PKG_INFO_TEST, "slackwarepkginfo"},
         {OVAL_LINUX_PARTITION, "partition"},
         {OVAL_LINUX_IFLISTENERS, "iflisteners"},
         {OVAL_LINUX_RPMVERIFY, "rpmverify"},
+        {OVAL_LINUX_RPMVERIFYFILE, "rpmverifyfile"},
+        {OVAL_LINUX_RPMVERIFYPACKAGE, "rpmverifypackage"},
         {OVAL_LINUX_SELINUXBOOLEAN, "selinuxboolean"},
         {OVAL_LINUX_SELINUXSECURITYCONTEXT, "selinuxsecuritycontext"},
 	{OVAL_SUBTYPE_UNKNOWN, NULL}
@@ -483,7 +500,10 @@ static const struct oscap_string_map OVAL_SUBTYPE_SOLARIS_MAP[] = {
 	{OVAL_SOLARIS_ISAINFO, "isainfo"},
 	{OVAL_SOLARIS_PACKAGE, "package"},
 	{OVAL_SOLARIS_PATCH, "patch"},
+	{OVAL_SOLARIS_PATCH54, "patch54"},
 	{OVAL_SOLARIS_SMF, "smf"},
+	{OVAL_SOLARIS_NDD, "ndd"},
+	{OVAL_SOLARIS_PACKAGECHECK, "packagecheck"},
 	{OVAL_SUBTYPE_UNKNOWN, NULL}
 };
 
@@ -607,6 +627,9 @@ oval_subtype_t oval_subtype_parse(xmlTextReaderPtr reader)
 
 	subtype = oscap_string_to_enum(map, tagname);
 
+	if (subtype == OVAL_ENUMERATION_INVALID)
+		dW("Unknown OVAL family subtype: %s\n", tagname);
+
  cleanup:
 	oscap_free(tagname);
 	return subtype;
