@@ -42,6 +42,7 @@
 #include "common/debug_priv.h"
 #include "common/_error.h"
 #include "common/elements.h"
+#include "common/public/oscap.h"
 
 void libxml_error_handler(void *user, const char *message, xmlParserSeverities severity, xmlTextReaderLocatorPtr locator)
 {
@@ -69,6 +70,87 @@ int oval_parser_parse_tag(xmlTextReaderPtr reader, struct oval_parser_context *c
 	return ret;
 }
 
+
+char *oval_determine_document_schema_version(const char *document, oscap_document_type_t doc_type)
+{
+	xmlTextReaderPtr reader;
+	const char *root_name;
+	const char* elm_name;
+	int depth;
+	xmlChar *version = NULL;
+
+	reader = xmlReaderForFile(document, NULL, 0);
+	if (!reader) {
+		oscap_seterr(OSCAP_EFAMILY_GLIBC, "Unable to open file: '%s'", document);
+		return NULL;
+	}
+
+	/* find root element */
+	while (xmlTextReaderRead(reader) == 1
+	       && xmlTextReaderNodeType(reader) != XML_READER_TYPE_ELEMENT);
+	/* verify document type */
+	switch (doc_type) {
+	case OSCAP_DOCUMENT_OVAL_DEFINITIONS:
+		root_name = OVAL_ROOT_ELM_DEFINITIONS;
+		break;
+	case OSCAP_DOCUMENT_OVAL_DIRECTIVES:
+		root_name = OVAL_ROOT_ELM_DIRECTIVES;
+		break;
+	case OSCAP_DOCUMENT_OVAL_RESULTS:
+		root_name = OVAL_ROOT_ELM_RESULTS;
+		break;
+	case OSCAP_DOCUMENT_OVAL_SYSCHAR:
+		root_name = OVAL_ROOT_ELM_SYSCHARS;
+		break;
+	case OSCAP_DOCUMENT_OVAL_VARIABLES:
+		root_name = OVAL_ROOT_ELM_VARIABLES;
+		break;
+	default:
+		oscap_seterr(OSCAP_EFAMILY_OVAL, "Unknown document type: %d.", doc_type);
+		xmlFreeTextReader(reader);
+		return NULL;
+	}
+	/* verify root element's name */
+	elm_name = (const char *) xmlTextReaderConstLocalName(reader);
+	if (!elm_name) {
+		oscap_setxmlerr(xmlGetLastError());
+		xmlFreeTextReader(reader);
+		return NULL;
+	}
+	if (strcmp(root_name, elm_name)) {
+		oscap_seterr(OSCAP_EFAMILY_OSCAP, "Document type doesn't match root element's name: '%s'.", elm_name);
+		xmlFreeTextReader(reader);
+		return NULL;
+	}
+	/* find generator */
+	while (xmlTextReaderRead(reader) == 1
+	       && xmlTextReaderNodeType(reader) != XML_READER_TYPE_ELEMENT);
+	elm_name = (const char *) xmlTextReaderConstLocalName(reader);
+	if (!elm_name || strcmp(elm_name, "generator")) {
+		oscap_seterr(OSCAP_EFAMILY_OSCAP, "Unexpected element: '%s'.", elm_name);
+		xmlFreeTextReader(reader);
+		return NULL;
+	}
+	/* find schema_version */
+	depth = xmlTextReaderDepth(reader);
+	while (xmlTextReaderRead(reader) == 1 && xmlTextReaderDepth(reader) > depth) {
+		if (xmlTextReaderNodeType(reader) != XML_READER_TYPE_ELEMENT)
+			continue;
+
+		elm_name = (const char *) xmlTextReaderConstLocalName(reader);
+		if (!strcmp(elm_name, "schema_version")) {
+			version = xmlTextReaderReadString(reader);
+			break;
+		}
+	}
+
+	xmlFreeTextReader(reader);
+	char* ret = strdup((const char*)version);
+	xmlFree(version);
+
+	return ret;
+}
+
 /*
  * -1 error; 0 OK; 1 warning
  */
