@@ -47,6 +47,7 @@
 #include "_oval_probe_session.h"
 #include "_oval_probe_handler.h"
 #include "oval_probe_meta.h"
+#include "oval_probe_ext.h"
 
 oval_probe_meta_t OSCAP_GSYM(__probe_meta)[] = {
         { OVAL_SUBTYPE_SYSINFO, "system_info", &oval_probe_sys_handler, OVAL_PROBEMETA_EXTERNAL, "probe_system_info" },
@@ -64,10 +65,13 @@ oval_probe_meta_t OSCAP_GSYM(__probe_meta)[] = {
         OVAL_PROBE_EXTERNAL(OVAL_INDEPENDENT_FILE_HASH58, "filehash58"),
         OVAL_PROBE_EXTERNAL(OVAL_LINUX_DPKG_INFO, "dpkginfo"),
         OVAL_PROBE_EXTERNAL(OVAL_LINUX_INET_LISTENING_SERVERS, "inetlisteningservers"),
+        OVAL_PROBE_EXTERNAL(OVAL_LINUX_INET_LISTENING_SERVER, "inetlisteningserver"),
         OVAL_PROBE_EXTERNAL(OVAL_LINUX_RPM_INFO, "rpminfo"),
         OVAL_PROBE_EXTERNAL(OVAL_LINUX_PARTITION, "partition"),
         OVAL_PROBE_EXTERNAL(OVAL_LINUX_IFLISTENERS, "iflisteners"),
         OVAL_PROBE_EXTERNAL(OVAL_LINUX_RPMVERIFY, "rpmverify"),
+        OVAL_PROBE_EXTERNAL(OVAL_LINUX_RPMVERIFYFILE, "rpmverifyfile"),
+        OVAL_PROBE_EXTERNAL(OVAL_LINUX_RPMVERIFYPACKAGE, "rpmverifypackage"),
         OVAL_PROBE_EXTERNAL(OVAL_LINUX_SELINUXBOOLEAN, "selinuxboolean"),
         OVAL_PROBE_EXTERNAL(OVAL_LINUX_SELINUXSECURITYCONTEXT, "selinuxsecuritycontext"),
         OVAL_PROBE_EXTERNAL(OVAL_SOLARIS_ISAINFO, "isainfo"),
@@ -105,7 +109,6 @@ static int __n2s_tbl_cmp(const char *name, oval_subtypedsc_t *dsc)
         return strcmp(name, dsc->name);
 }
 
-#if defined(ENABLE_PROBES)
 /*
  * Library side entity name cache. Initialization needs to be
  * thread-safe and is done by oval_probe_session_new. Freeing
@@ -114,7 +117,6 @@ static int __n2s_tbl_cmp(const char *name, oval_subtypedsc_t *dsc)
  */
 probe_ncache_t  *OSCAP_GSYM(ncache) = NULL;
 struct id_desc_t OSCAP_GSYM(id_desc);
-#endif
 
 #if defined(OSCAP_THREAD_SAFE)
 # include <pthread.h>
@@ -413,7 +415,7 @@ int oval_probe_query_object(oval_probe_session_t *psess, struct oval_object *obj
         if (ph == NULL) {
                 char *msg = "OVAL object not supported.";
 
-		dW(msg);
+		dW("%s\n", msg);
 		oval_syschar_add_new_message(sysc, msg, OVAL_MESSAGE_LEVEL_WARNING);
 		oval_syschar_set_flag(sysc, SYSCHAR_FLAG_NOT_COLLECTED);
 
@@ -443,12 +445,12 @@ int oval_probe_query_sysinfo(oval_probe_session_t *sess, struct oval_sysinfo **o
         ph = oval_probe_handler_get(sess->ph, OVAL_SUBTYPE_SYSINFO);
 
         if (ph == NULL) {
-                oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBENOTSUPP, "OVAL object not supported");
+                oscap_seterr (OSCAP_EFAMILY_OVAL, "OVAL object not supported");
 		return(-1);
         }
 
         if (ph->func == NULL) {
-                oscap_seterr (OSCAP_EFAMILY_OVAL, OVAL_EPROBENOTSUPP, "OVAL object not correctly defined");
+                oscap_seterr (OSCAP_EFAMILY_OVAL, "OVAL object not correctly defined");
 		return(-1);
         }
 
@@ -475,10 +477,7 @@ int oval_probe_query_definition(oval_probe_session_t *sess, const char *id) {
         definition_model = oval_syschar_model_get_definition_model(syschar_model);
 	definition = oval_definition_model_get_definition(definition_model, id);
 	if (definition == NULL) {
-                char msg[100];
-                snprintf(msg, sizeof(msg), "No definition with ID: %s in definition model.", id);
-                dE(msg);
-                oscap_seterr(OSCAP_EFAMILY_OSCAP, OVAL_EOVALINT, msg);
+                oscap_seterr(OSCAP_EFAMILY_OSCAP, "No definition with ID: %s in definition model.", id);
 		return -1;
 	}
 
@@ -640,14 +639,10 @@ void oval_probe_meta_list(FILE *output, int flags)
 	    }
 	}
 
-	fprintf(output, "%-32s %-32s %c",
-		meta[i].stype, meta[i].pname,
-		meta[i].flags & OVAL_PROBEMETA_EXTERNAL ? 'E' : '.');
+	fprintf(output, "%-28s %-28s", meta[i].stype, meta[i].pname);
 
 	if (flags & OVAL_PROBEMETA_LIST_VERBOSE) {
-	    fprintf(output, " %-5u %s\n",
-		    meta[i].otype,
-		    meta[i].flags & OVAL_PROBEMETA_EXTERNAL ? probe_path : "");
+	    fprintf(output, " %-5u %s\n", meta[i].otype, probe_path);
 	} else
 	    fprintf(output, "\n");
     }
