@@ -41,6 +41,8 @@
 #include "oval_results_impl.h"
 #include "oval_collection_impl.h"
 #include "oval_string_map_impl.h"
+#include "oval_parser_impl.h"
+
 #include "common/debug_priv.h"
 #include "common/_error.h"
 #include "common/util.h"
@@ -279,7 +281,7 @@ static int oval_result_system_parse(xmlTextReaderPtr reader, struct oval_parser_
                 return_code = oval_parser_parse_tag(reader, context, oval_result_definition_parse_tag, sys);
 	} else if (strcmp((const char *)localName, "tests") == 0) {
 		return_code = oval_parser_parse_tag(reader, context, oval_result_test_parse_tag, sys);
-	} else if (strcmp((const char *)localName, "oval_system_characteristics") == 0) {
+	} else if (strcmp((const char *)localName, OVAL_ROOT_ELM_SYSCHARS) == 0) {
 		return_code = oval_syschar_model_parse(reader, context);
 	} else {
                 dW("Skipping tag: %s\n", localName);
@@ -359,10 +361,7 @@ int oval_result_system_eval_definition(struct oval_result_system *sys, const cha
 	definition_model = oval_results_model_get_definition_model(res_model);
 	oval_definition = oval_definition_model_get_definition(definition_model, id);
 	if (oval_definition == NULL) {
-		char msg[100];
-		snprintf(msg, sizeof(msg), "No definition with ID: %s in definition model.", id);
-		dE(msg);
-		oscap_seterr(OSCAP_EFAMILY_OSCAP, OVAL_EOVALINT, msg);
+		oscap_seterr(OSCAP_EFAMILY_OSCAP, "No definition with ID: %s in definition model.", id);
 		return -1;
 	}
 
