@@ -47,12 +47,11 @@
 #include "probes/public/probe-api.h"
 #include "oval_definitions_impl.h"
 #include "oval_system_characteristics_impl.h"
+#include "oval_string_map_impl.h"
 #include "common/debug_priv.h"
 #include "common/_error.h"
+#include "public/oval_version.h"
 
-#ifndef _A
-# define _A(x) assert(x)
-#endif
 
 SEXP_t *oval_value_to_sexp(struct oval_value *val, oval_datatype_t dtype)
 {
@@ -84,13 +83,11 @@ SEXP_t *oval_value_to_sexp(struct oval_value *val, oval_datatype_t dtype)
 	case OVAL_DATATYPE_FILESET_REVISION:
 	case OVAL_DATATYPE_IOS_VERSION:
 		// todo:
-		oscap_dlprintf(DBG_E, "Unsupported datatype: %s.\n", dtype);
-		oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EOVALINT, "Unsupported datatype");
+		oscap_seterr(OSCAP_EFAMILY_OVAL, "Unsupported datatype: %s.", dtype);
 		val_sexp = NULL;
 		break;
 	default:
-		oscap_dlprintf(DBG_E, "Unknown datatype: %s.\n", dtype);
-		oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EOVALINT, "Unknown datatype");
+		oscap_seterr(OSCAP_EFAMILY_OVAL, "Unknown datatype: %s.", dtype);
 		val_sexp = NULL;
 		break;
 	}
@@ -111,6 +108,11 @@ static SEXP_t *oval_entity_to_sexp(struct oval_entity *ent)
 				 r2 = SEXP_number_newu_32(oval_entity_get_operation(ent)), NULL);
 	SEXP_vfree(r0, r1, r2, NULL);
 
+        if (oval_entity_get_mask(ent)) {
+            SEXP_list_add(elm_name, r0 = SEXP_string_new("mask", 4));
+            SEXP_free(r0);
+        }
+
 	elm = SEXP_list_new(NULL);
 	datatype = oval_entity_get_datatype(ent);
 	probe_ent_setdatatype(elm, datatype);
@@ -186,7 +188,7 @@ static int oval_varref_attr_to_sexp(void *sess, struct oval_entity *entity, stru
 	}
 
 	if (ret) {
-		dW(msg);
+		dW("%s\n", msg);
 		oval_syschar_add_new_message(syschar, msg, OVAL_MESSAGE_LEVEL_WARNING);
 		oval_syschar_set_flag(syschar, SYSCHAR_FLAG_DOES_NOT_EXIST);
 		return ret;
@@ -252,11 +254,9 @@ static int oval_varref_elm_to_sexp(void *sess, struct oval_variable *var, oval_d
 		val = oval_value_iterator_next(val_itr);
 		vs = oval_value_to_sexp(val, dt);
 		if (vs == NULL) {
-			oscap_dlprintf(DBG_E, "Failed to convert OVAL value to SEXP: "
-				       "datatype: %s, text: %s.\n",
-				       oval_datatype_get_text(dt),
-				       oval_value_get_text(val));
-			oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EOVALINT, "Failed to convert OVAL value to SEXP");
+			oscap_seterr(OSCAP_EFAMILY_OVAL, "Failed to convert OVAL value to SEXP: "
+                                       "datatype: %s, text: %s.", oval_datatype_get_text(dt),
+                                       oval_value_get_text(val));
 			oval_value_iterator_free(val_itr);
 			SEXP_free(val_lst);
 			return -1;
@@ -396,9 +396,9 @@ static SEXP_t *oval_behaviors_to_sexp(struct oval_behavior_iterator *bit)
 int oval_object_to_sexp(void *sess, const char *typestr, struct oval_syschar *syschar, SEXP_t **out_sexp)
 {
 	unsigned int ent_cnt, varref_cnt;
-	int ret = -1;
-	SEXP_t *obj_sexp, *obj_name, *elm, *varrefs, *ent_lst, *lst, *stmp;
-	SEXP_t *r0, *r1, *r2;
+	int ret;
+	SEXP_t *obj_sexp, *elm, *varrefs, *ent_lst, *lst, *stmp;
+	SEXP_t *r0, *r1, *r2, *obj_attr, sm0, sm1;
 
 	struct oval_object *object;
 	struct oval_object_content_iterator *cit;
@@ -406,21 +406,32 @@ int oval_object_to_sexp(void *sess, const char *typestr, struct oval_syschar *sy
 	struct oval_object_content *content;
 	struct oval_entity *entity;
 
+	oval_version_t obj_over;
+	char obj_name[128];
+	const char *obj_id;
+
 	object = oval_syschar_get_object(syschar);
 
 	/*
 	 * Object name & attributes (id)
 	 */
-	obj_name = SEXP_list_new(r0 = SEXP_string_newf("%s_object", typestr),
-				 r1 = SEXP_string_new(":id", 3),
-				 r2 = SEXP_string_newf("%s", oval_object_get_id(object)), NULL);
-	SEXP_free(r0);
-	SEXP_free(r1);
-	SEXP_free(r2);
+	ret = snprintf(obj_name, sizeof obj_name, "%s_object", typestr);
+	if (ret<0 || (unsigned int) ret > sizeof obj_name) {
+		dE("obj_name length too short");
+		return -1;
+	}
 
-	obj_sexp = SEXP_list_new(obj_name, NULL);
+	obj_over = oval_object_get_schema_version(object);
+	obj_id   = oval_object_get_id(object);
+	obj_attr = probe_attr_creat("id", SEXP_string_new_r(&sm0, obj_id, strlen(obj_id)),
+	                            "oval_version", SEXP_number_newu_32_r(&sm1, obj_over),
+	                            NULL);
 
-	SEXP_free(obj_name);
+	obj_sexp = probe_obj_new(obj_name, obj_attr);
+
+	SEXP_free_r(&sm0);
+	SEXP_free_r(&sm1);
+	SEXP_free(obj_attr);
 
 	/*
 	 * Object content
@@ -452,26 +463,23 @@ int oval_object_to_sexp(void *sess, const char *typestr, struct oval_syschar *sy
 						   r0 = SEXP_number_newu_32(ochk));
 				SEXP_free(r0);
 			}
-
+			ret = 0;
 			vr_type = oval_entity_get_varref_type(entity);
 			if (vr_type == OVAL_ENTITY_VARREF_ATTRIBUTE) {
 				ret = oval_varref_attr_to_sexp(sess, entity, syschar, &stmp);
-				if (ret != 0) {
-					SEXP_free(elm);
-					elm = NULL;
-					break;
-				}
 
-				if (varrefs == NULL) {
-					varrefs = SEXP_list_new(NULL);
-				}
-				SEXP_list_add(varrefs, stmp);
-                                SEXP_free(stmp);
-				// todo: don't add duplicates
-				++varref_cnt;
+				if (ret == 0) {
+					if (varrefs == NULL)
+						varrefs = SEXP_list_new(NULL);
 
-				lst = obj_sexp;
-				++ent_cnt;
+					SEXP_list_add(varrefs, stmp);
+					SEXP_free(stmp);
+					// todo: don't add duplicates
+					++varref_cnt;
+
+					lst = obj_sexp;
+					++ent_cnt;
+				}
 			} else if (vr_type == OVAL_ENTITY_VARREF_ELEMENT) {
 				SEXP_t *val_lst;
 				struct oval_variable *var;
@@ -479,17 +487,31 @@ int oval_object_to_sexp(void *sess, const char *typestr, struct oval_syschar *sy
 
 				var = oval_entity_get_variable(entity);
 				dt = oval_entity_get_datatype(entity);
-
 				ret = oval_varref_elm_to_sexp(sess, var, dt, &val_lst);
-				if (ret != 0) {
-					SEXP_free(elm);
-					elm = NULL;
-					break;
-				}
 
-				SEXP_list_add(elm, val_lst);
-				SEXP_free(val_lst);
+				if (ret == 0) {
+					SEXP_list_add(elm, val_lst);
+					SEXP_free(val_lst);
+				}
 			}
+
+			if (ret != 0) {
+				SEXP_t s_flag;
+				SEXP_number_newi_32_r(&s_flag, SYSCHAR_FLAG_DOES_NOT_EXIST);
+				probe_item_attr_add(obj_sexp, "skip_eval", &s_flag);
+				SEXP_free_r(&s_flag);
+
+				SEXP_free(elm);
+				SEXP_free(ent_lst);
+				if (varrefs != NULL)
+					SEXP_free(varrefs);
+				oval_object_content_iterator_free(cit);
+
+				*out_sexp = obj_sexp;
+
+				return (0);
+			}
+
 			break;
 
 		case OVAL_OBJECTCONTENT_SET:
@@ -511,7 +533,7 @@ int oval_object_to_sexp(void *sess, const char *typestr, struct oval_syschar *sy
 				SEXP_free(varrefs);
 			oval_object_content_iterator_free(cit);
 
-			return (ret);
+			return -1;
 		}
 
 		SEXP_list_add(lst, elm);
@@ -528,6 +550,7 @@ int oval_object_to_sexp(void *sess, const char *typestr, struct oval_syschar *sy
 		SEXP_list_add(obj_sexp, r0);
 		SEXP_vfree(stmp, varrefs, r0, NULL);
 	}
+
 	stmp = SEXP_list_join(obj_sexp, ent_lst);
 	SEXP_free(obj_sexp);
 	SEXP_free(ent_lst);
@@ -593,14 +616,14 @@ static SEXP_t *oval_record_field_STATE_to_sexp(struct oval_record_field *rf)
 	return rf_sexp;
 }
 
-static struct oval_sysent *oval_sexp_to_sysent(struct oval_syschar_model *model, struct oval_sysitem *item, SEXP_t * sexp);
+static struct oval_sysent *oval_sexp_to_sysent(struct oval_syschar_model *model, struct oval_sysitem *item, SEXP_t * sexp, struct oval_string_map *mask_map);
 
 static struct oval_record_field *oval_record_field_ITEM_from_sexp(SEXP_t *sexp)
 {
 	struct oval_sysent *sysent;
 	struct oval_record_field *rf;
 
-	sysent = oval_sexp_to_sysent(NULL, NULL, sexp);
+	sysent = oval_sexp_to_sysent(NULL, NULL, sexp, NULL);
 	if (sysent == NULL)
 		return NULL;
 
@@ -628,7 +651,7 @@ int oval_state_to_sexp(void *sess, struct oval_state *state, SEXP_t **out_sexp)
         subtype_name = oval_subtype_to_str(oval_state_get_subtype(state));
 
 	if (subtype_name == NULL) {
-		_D("FAIL: unknown subtype: %d\n", oval_state_get_subtype(state));
+		dI("FAIL: unknown subtype: %d\n", oval_state_get_subtype(state));
 		return (-1);
 	}
 
@@ -736,7 +759,7 @@ static struct oval_message *oval_sexp_to_msg(const SEXP_t *msg)
 	return message;
 }
 
-static struct oval_sysent *oval_sexp_to_sysent(struct oval_syschar_model *model, struct oval_sysitem *item, SEXP_t * sexp)
+static struct oval_sysent *oval_sexp_to_sysent(struct oval_syschar_model *model, struct oval_sysitem *item, SEXP_t * sexp, struct oval_string_map *mask_map)
 {
 	char *key;
 	oval_syschar_status_t status;
@@ -779,7 +802,7 @@ static struct oval_sysent *oval_sexp_to_sysent(struct oval_syschar_model *model,
 	oval_sysent_set_name(ent, key);
 	oval_sysent_set_status(ent, status);
 	oval_sysent_set_datatype(ent, dt);
-	oval_sysent_set_mask(ent, probe_ent_getmask(sexp));
+	oval_sysent_set_mask(ent, oval_string_map_get_value(mask_map, key) == NULL ? 0 : 1);
 
 	if (status != SYSCHAR_STATUS_EXISTS)
 		return ent;
@@ -855,7 +878,7 @@ static struct oval_sysent *oval_sexp_to_sysent(struct oval_syschar_model *model,
 	return ent;
 }
 
-static struct oval_sysitem *oval_sexp_to_sysitem(struct oval_syschar_model *model, SEXP_t * sexp)
+static struct oval_sysitem *oval_sexp_to_sysitem(struct oval_syschar_model *model, SEXP_t * sexp, struct oval_string_map *mask_map)
 {
 	_A(sexp);
 
@@ -890,9 +913,10 @@ static struct oval_sysitem *oval_sexp_to_sysitem(struct oval_syschar_model *mode
 
 	int type = oval_str_to_subtype(name);
 
-	_D("Syschar entry type: %d '%s' => %s\n", type, name, (type ? "OK" : "FAILED to decode"));
+	dI("Syschar entry type: %d '%s' => %s\n", type, name,
+	   ((type != OVAL_SUBTYPE_UNKNOWN) ? "OK" : "FAILED to decode"));
 #ifndef NDEBUG
-	if (!type)
+	if (type == OVAL_SUBTYPE_UNKNOWN)
 		abort();
 #endif
 	SEXP_t *sub;
@@ -905,7 +929,7 @@ static struct oval_sysitem *oval_sexp_to_sysitem(struct oval_syschar_model *mode
 	oval_sysitem_set_subtype(sysitem, type);
 
 	for (int i = 2; (sub = SEXP_list_nth(sexp, i)) != NULL; ++i) {
-	    if ((sysent = oval_sexp_to_sysent(model, sysitem, sub)) != NULL)
+	    if ((sysent = oval_sexp_to_sysent(model, sysitem, sub, mask_map)) != NULL)
 		    oval_sysitem_add_sysent(sysitem, sysent);
 		SEXP_free(sub);
 	}
@@ -919,8 +943,10 @@ static struct oval_sysitem *oval_sexp_to_sysitem(struct oval_syschar_model *mode
 int oval_sexp_to_sysch(const SEXP_t *cobj, struct oval_syschar *syschar)
 {
 	oval_syschar_collection_flag_t flag;
-	SEXP_t *messages, *msg, *items, *item;
+	SEXP_t *messages, *msg, *items, *item, *mask;
 	struct oval_syschar_model *model;
+	struct oval_string_map *itm_id_map;
+        struct oval_string_map *item_mask_map;
 
 	_A(cobj != NULL);
 
@@ -937,16 +963,43 @@ int oval_sexp_to_sysch(const SEXP_t *cobj, struct oval_syschar *syschar)
 	}
 	SEXP_free(messages);
 
+	itm_id_map = oval_string_map_new();
 	model = oval_syschar_get_model(syschar);
 	items = probe_cobj_get_items(cobj);
+
+        mask = probe_cobj_get_mask(cobj);
+        if (mask != NULL) {
+            SEXP_t *mask_entname;
+            char mask_entname_cstr[128];
+            item_mask_map = oval_string_map_new();
+            SEXP_list_foreach(mask_entname, mask) {
+                SEXP_string_cstr_r(mask_entname, mask_entname_cstr, sizeof mask_entname_cstr);
+                oval_string_map_put_string(item_mask_map, mask_entname_cstr, mask_entname_cstr);
+            }
+            SEXP_free(mask);
+        } else
+            item_mask_map = NULL;
+
 	SEXP_list_foreach(item, items) {
 		struct oval_sysitem *sysitem;
 
-		sysitem = oval_sexp_to_sysitem(model, item);
-		if (sysitem != NULL)
-			oval_syschar_add_sysitem(syschar, sysitem);
+		sysitem = oval_sexp_to_sysitem(model, item, item_mask_map);
+		if (sysitem != NULL) {
+			char *itm_id;
+
+			itm_id = oval_sysitem_get_id(sysitem);
+			if (oval_string_map_get_value(itm_id_map, itm_id) == NULL) {
+				oval_string_map_put(itm_id_map, itm_id, itm_id);
+				oval_syschar_add_sysitem(syschar, sysitem);
+			}
+		}
 	}
 	SEXP_free(items);
+	oval_string_map_free(itm_id_map, NULL);
+        if (item_mask_map != NULL)
+            oval_string_map_free_string(item_mask_map);
 
 	return 0;
 }
+
+/// @}
