@@ -38,9 +38,8 @@
 #include "oval_definitions_impl.h"
 #include "oval_collection_impl.h"
 #include "oval_agent_api_impl.h"
-#include "../common/util.h"
+#include "common/util.h"
 #include "common/debug_priv.h"
-#include "../common/public/debug.h"
 
 typedef struct oval_state {
 	struct oval_definition_model *model;
