@@ -27,9 +27,14 @@
  *      "David Niemoller" <David.Niemoller@g2-inc.com>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+
 #include "oval_system_characteristics_impl.h"
 #include "oval_collection_impl.h"
 #include "common/util.h"
@@ -68,12 +73,10 @@ char *oval_sysint_get_name(struct oval_sysint *sysint)
 
 void oval_sysint_set_name(struct oval_sysint *sysint, char *name)
 {
-	if (sysint && !oval_sysint_is_locked(sysint)) {
-		if (sysint->name != NULL)
-			oscap_free(sysint->name);
-		sysint->name = oscap_strdup(name);
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(sysint);
+	if (sysint->name != NULL)
+		oscap_free(sysint->name);
+	sysint->name = oscap_strdup(name);
 }
 
 char *oval_sysint_get_ip_address(struct oval_sysint *sysint)
@@ -85,12 +88,10 @@ char *oval_sysint_get_ip_address(struct oval_sysint *sysint)
 
 void oval_sysint_set_ip_address(struct oval_sysint *sysint, char *ip_address)
 {
-	if (sysint && !oval_sysint_is_locked(sysint)) {
-		if (sysint->ipAddress != NULL)
-			oscap_free(sysint->ipAddress);
-		sysint->ipAddress = oscap_strdup(ip_address);
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(sysint);
+	if (sysint->ipAddress != NULL)
+		oscap_free(sysint->ipAddress);
+	sysint->ipAddress = oscap_strdup(ip_address);
 }
 
 char *oval_sysint_get_mac_address(struct oval_sysint *sysint)
@@ -100,12 +101,10 @@ char *oval_sysint_get_mac_address(struct oval_sysint *sysint)
 
 void oval_sysint_set_mac_address(struct oval_sysint *sysint, char *mac_address)
 {
-	if (sysint && !oval_sysint_is_locked(sysint)) {
-		if (sysint->macAddress != NULL)
-			oscap_free(sysint->macAddress);
-		sysint->macAddress = oscap_strdup(mac_address);
-	} else
-		oscap_dlprintf(DBG_W, "Attempt to update locked content.\n");
+	__attribute__nonnull__(sysint);
+	if (sysint->macAddress != NULL)
+		oscap_free(sysint->macAddress);
+	sysint->macAddress = oscap_strdup(mac_address);
 }
 
 struct oval_sysint *oval_sysint_new(struct oval_syschar_model *model)
@@ -121,19 +120,6 @@ struct oval_sysint *oval_sysint_new(struct oval_syschar_model *model)
 	return sysint;
 }
 
-bool oval_sysint_is_valid(struct oval_sysint * sysint)
-{
-        oscap_dlprintf(DBG_W, "NOOP.\n");
-	return true;		//TODO
-}
-
-bool oval_sysint_is_locked(struct oval_sysint * sysint)
-{
-	__attribute__nonnull__(sysint);
-
-	return oval_syschar_model_is_locked(sysint->model);
-}
-
 struct oval_sysint *oval_sysint_clone(struct oval_syschar_model *new_model, struct oval_sysint *old_sysint)
 {
 	struct oval_sysint *new_sysint = oval_sysint_new(new_model);
@@ -191,8 +177,9 @@ static int _oval_sysint_parse_tag(xmlTextReaderPtr reader, struct oval_parser_co
 	struct oval_sysint *sysint = (struct oval_sysint *)user;
 	char *tagname = (char *)xmlTextReaderLocalName(reader);
 	char *namespace = (char *)xmlTextReaderNamespaceUri(reader);
-	int is_ovalsys = strcmp(namespace, NAMESPACE_OVALSYS) == 0;
-	int return_code;
+	int return_code = 0;
+
+	int is_ovalsys = strcmp((const char *)OVAL_SYSCHAR_NAMESPACE, namespace) == 0;
 	if (is_ovalsys && (strcmp(tagname, "interface_name") == 0)) {
 		return_code = oval_parser_text_value(reader, context, &oval_consume_interface_name, sysint);
 	} else if (is_ovalsys && (strcmp(tagname, "ip_address") == 0)) {
@@ -200,67 +187,42 @@ static int _oval_sysint_parse_tag(xmlTextReaderPtr reader, struct oval_parser_co
 	} else if (is_ovalsys && (strcmp(tagname, "mac_address") == 0)) {
 		return_code = oval_parser_text_value(reader, context, &oval_consume_mac_address, sysint);
 	} else {
-		oscap_dlprintf(DBG_W, "Skipping tag: <%s:%s>.\n", namespace, tagname);
-		return_code = oval_parser_skip_tag(reader, context);
+		dW("Skipping tag: <%s:%s>.\n", namespace, tagname);
+		oval_parser_skip_tag(reader, context);
 	}
+
 	oscap_free(tagname);
 	oscap_free(namespace);
+
 	return return_code;
 }
 
 int oval_sysint_parse_tag(xmlTextReaderPtr reader,
 			  struct oval_parser_context *context, oval_sysint_consumer consumer, void *user)
 {
-
 	__attribute__nonnull__(context);
 
 	struct oval_sysint *sysint = oval_sysint_new(context->syschar_model);
 	char *tagname = (char *)xmlTextReaderLocalName(reader);
 	char *namespace = (char *)xmlTextReaderNamespaceUri(reader);
-	int is_ovalsys = strcmp(namespace, NAMESPACE_OVALSYS) == 0;
-	int return_code;
+	int return_code = 0;
+
+	int is_ovalsys = strcmp((const char *)OVAL_SYSCHAR_NAMESPACE, namespace) == 0;
 	if (is_ovalsys && (strcmp(tagname, "interface") == 0)) {
 		return_code = oval_parser_parse_tag(reader, context, &_oval_sysint_parse_tag, sysint);
 	} else {
-		oscap_dlprintf(DBG_W, "Expected <interface>, skipping <%s:%s>.\n",
-			      namespace, tagname);
-		return_code = oval_parser_skip_tag(reader, context);
+		dW("Skipping tag: <%s:%s>.\n", namespace, tagname);
+		oval_parser_skip_tag(reader, context);
 	}
+
+	(*consumer) (sysint, user);
+
 	oscap_free(tagname);
 	oscap_free(namespace);
-	(*consumer) (sysint, user);
+
 	return return_code;
 }
 
-void oval_sysint_to_print(struct oval_sysint *sysint, char *indent, int idx)
-{
-	char nxtindent[100];
-
-	if (strlen(indent) > 80)
-		indent = "....";
-
-	if (idx == 0)
-		snprintf(nxtindent, sizeof(nxtindent), "%sINTERFACE.", indent);
-	else
-		snprintf(nxtindent, sizeof(nxtindent), "%sINTERFACE[%d].", indent, idx);
-	/*
-	   char *name;
-	   char *ipAddress;
-	   char *macAddress;
-	 */
-	{			//name
-		oscap_dprintf("%sNAME           = %s\n", nxtindent, oval_sysint_get_name(sysint));
-	}
-	char *ipadd = oval_sysint_get_ip_address(sysint);
-	if (ipadd != NULL) {	//ipaddress
-		oscap_dprintf("%sIP_ADDRESS      = %s\n", nxtindent, ipadd);
-	}
-	char *macadd = oval_sysint_get_mac_address(sysint);
-	if (macadd != NULL) {	//mac address
-		oscap_dprintf("%sMAC_ADDRESS     = %s\n", nxtindent, macadd);
-	}
-}
-
 void oval_sysint_to_dom(struct oval_sysint *sysint, xmlDoc * doc, xmlNode * tag_parent)
 {
 	if (sysint) {
