@@ -44,7 +44,7 @@
 #include "common/util.h"
 #include "common/debug_priv.h"
 #include "common/_error.h"
-#include "common/reporter_priv.h"
+#include "common/elements.h"
 
 
 typedef struct oval_syschar_model {
@@ -67,6 +67,10 @@ struct oval_syschar_model *oval_syschar_model_new(struct oval_definition_model *
 		return NULL;
 
 	newmodel->generator = oval_generator_new();
+        struct oval_generator *generator = oval_definition_model_get_generator(definition_model);
+        char * schema_version = oval_generator_get_schema_version(generator);
+        oval_generator_set_schema_version(newmodel->generator, schema_version);
+
 	newmodel->sysinfo = NULL;
 	newmodel->definition_model = definition_model;
 	newmodel->syschar_map = oval_string_map_new();
@@ -237,9 +241,7 @@ int oval_syschar_model_import(struct oval_syschar_model *model, const char *file
 
 	xmlTextReader *reader = xmlNewTextReaderFilename(file);
 	if (reader == NULL) {
-                if(errno)
-                        oscap_seterr(OSCAP_EFAMILY_GLIBC, errno, strerror(errno));
-                oscap_dlprintf(DBG_E, "Unable to open file.\n");
+		oscap_seterr(OSCAP_EFAMILY_GLIBC, "%s '%s'", strerror(errno), file);
                 return -1;
 	}
 
@@ -257,10 +259,10 @@ int oval_syschar_model_import(struct oval_syschar_model *model, const char *file
 	char *namespace = (char *)xmlTextReaderNamespaceUri(reader);
 	int is_ovalsys = strcmp((const char *)OVAL_SYSCHAR_NAMESPACE, namespace) == 0;
 	/* start parsing */
-	if (is_ovalsys && (strcmp(tagname, "oval_system_characteristics") == 0)) {
+	if (is_ovalsys && (strcmp(tagname, OVAL_ROOT_ELM_SYSCHARS) == 0)) {
 		ret = oval_syschar_model_parse(reader, &context);
 	} else {
-		oscap_seterr(OSCAP_EFAMILY_OSCAP, OSCAP_EXMLELEM, "Missing \"oval_system_characteristics\" element");
+		oscap_seterr(OSCAP_EFAMILY_OSCAP, "Missing \"oval_system_characteristics\" element");
 		dE("Unprocessed tag: <%s:%s>.\n", namespace, tagname);
 		ret = -1;
 	}
@@ -321,9 +323,9 @@ xmlNode *oval_syschar_model_to_dom(struct oval_syschar_model * syschar_model, xm
 	xmlNodePtr root_node = NULL;
 
 	if (parent) { /* result file */
-		root_node = xmlNewTextChild(parent, NULL, BAD_CAST "oval_system_characteristics", NULL);
+		root_node = xmlNewTextChild(parent, NULL, BAD_CAST OVAL_ROOT_ELM_SYSCHARS, NULL);
 	} else { /* system characteristics file, we are the root */
-		root_node = xmlNewNode(NULL, BAD_CAST "oval_system_characteristics");
+		root_node = xmlNewNode(NULL, BAD_CAST OVAL_ROOT_ELM_SYSCHARS);
 		xmlDocSetRootElement(doc, root_node);
 	}
 	xmlNewProp(root_node, BAD_CAST "xsi:schemaLocation", BAD_CAST syschar_model->schema);
@@ -404,8 +406,6 @@ int oval_syschar_model_export(struct oval_syschar_model *model, const char *file
 
 	__attribute__nonnull__(model);
 
-	int retcode = 0;
-
 	LIBXML_TEST_VERSION;
 
 	xmlDocPtr doc = xmlNewDoc(BAD_CAST "1.0");
@@ -415,15 +415,6 @@ int oval_syschar_model_export(struct oval_syschar_model *model, const char *file
 	}
 
 	oval_syschar_model_to_dom(model, doc, NULL, NULL, NULL);
-	/*
-	 * Dumping document to stdio or file
-	 */
-	retcode = xmlSaveFormatFileEnc(file, doc, "UTF-8", 1);
-
-	if (retcode < 1)
-		oscap_setxmlerr(xmlGetLastError());
-
-	xmlFreeDoc(doc);
-	return retcode;
+	return oscap_xml_save_filename(file, doc);
 }
 
