@@ -43,6 +43,7 @@
 #include "common/util.h"
 #include "common/debug_priv.h"
 #include "common/_error.h"
+#include "common/elements.h"
 
 typedef struct _oval_variable_model_frame {
 	char *id;
@@ -285,7 +286,7 @@ static int _oval_variable_model_parse(struct oval_variable_model *model, xmlText
 	xmlTextReaderSetErrorHandler(reader, &libxml_error_handler, &context);
 	char *tagname = (char *)xmlTextReaderLocalName(reader);
 	char *namespace = (char *)xmlTextReaderNamespaceUri(reader);
-	bool is_variables = (strcmp(NAMESPACE_VARIABLES, namespace) == 0) && (strcmp("oval_variables", tagname) == 0);
+	bool is_variables = (strcmp(NAMESPACE_VARIABLES, namespace) == 0) && (strcmp(OVAL_ROOT_ELM_VARIABLES, tagname) == 0);
 	if (is_variables) {
 		return_code =
 		    oval_parser_parse_tag(reader, &context, (oval_xml_tag_parser) _oval_variable_model_parse_tag, model);
@@ -305,9 +306,7 @@ struct oval_variable_model * oval_variable_model_import(const char *file)
 
 	xmlTextReader *reader = xmlNewTextReaderFilename(file);
 	if (reader == NULL) {
-                if(errno)
-                        oscap_seterr(OSCAP_EFAMILY_GLIBC, errno, strerror(errno));
-                oscap_dlprintf(DBG_E, "Unable to open file.\n");
+		oscap_seterr(OSCAP_EFAMILY_GLIBC, "%s '%s'", strerror(errno), file);
                 return NULL;
 	}
 
@@ -317,7 +316,7 @@ struct oval_variable_model * oval_variable_model_import(const char *file)
 	if (ret != 1) {
 		oval_variable_model_free(model);
 		model = NULL;
-	}		
+	}
 	xmlFreeTextReader(reader);
 
 	return model;
@@ -331,9 +330,9 @@ static xmlNode *oval_variable_model_to_dom(struct oval_variable_model * variable
 	xmlNodePtr root_node;
 
 	if (parent) {
-		root_node = xmlNewTextChild(parent, NULL, BAD_CAST "oval_variables", NULL);
+		root_node = xmlNewTextChild(parent, NULL, BAD_CAST OVAL_ROOT_ELM_VARIABLES, NULL);
 	} else {
-		root_node = xmlNewNode(NULL, BAD_CAST "oval_variables");
+		root_node = xmlNewNode(NULL, BAD_CAST OVAL_ROOT_ELM_VARIABLES);
 		xmlDocSetRootElement(doc, root_node);
 	}
 
@@ -384,8 +383,6 @@ int oval_variable_model_export(struct oval_variable_model *model, const char *fi
 
 	__attribute__nonnull__(model);
 
-	int retcode = 0;
-
 	LIBXML_TEST_VERSION;
 
 	xmlDocPtr doc = xmlNewDoc(BAD_CAST "1.0");
@@ -395,16 +392,7 @@ int oval_variable_model_export(struct oval_variable_model *model, const char *fi
 	}
 
 	oval_variable_model_to_dom(model, doc, NULL, NULL);
-	/*
-	 * Dumping document to stdio or file
-	 */
-	retcode = xmlSaveFormatFileEnc(file, doc, "UTF-8", 1);
-
-	if (retcode < 1)
-		oscap_setxmlerr(xmlGetLastError());
-
-	xmlFreeDoc(doc);
-	return retcode;
+	return oscap_xml_save_filename(file, doc);
 }
 
 bool oval_variable_model_has_variable(struct oval_variable_model *model, const char * id)
