@@ -45,8 +45,10 @@
 #include <pthread.h>
 #include <errno.h>
 #include <crapi/crapi.h>
+#include <probe/option.h>
 
 #include "oval_fts.h"
+#include <common/debug_priv.h>
 
 #define FILE_SEPARATOR '/'
 
@@ -72,12 +74,12 @@ static int mem2hex (uint8_t *mem, size_t mlen, char *str, size_t slen)
         return (0);
 }
 
-static int filehash_cb (const char *p, const char *f, probe_ctx *ctx)
+static int filehash_cb (const char *p, const char *f, probe_ctx *ctx, oval_version_t over)
 {
         SEXP_t *itm;
         char   pbuf[PATH_MAX+1];
         size_t plen, flen;
-
+	bool include_filepath;
         int fd;
 
         if (f == NULL)
@@ -101,6 +103,7 @@ static int filehash_cb (const char *p, const char *f, probe_ctx *ctx)
 
         memcpy (pbuf + plen, f, sizeof (char) * flen);
         pbuf[plen+flen] = '\0';
+	include_filepath = oval_version_cmp(over, OVAL_VERSION(5.6)) >= 0;
 
         /*
          * Open the file
@@ -112,7 +115,7 @@ static int filehash_cb (const char *p, const char *f, probe_ctx *ctx)
                 pbuf[PATH_MAX] = '\0';
 
 		itm = probe_item_create(OVAL_INDEPENDENT_FILE_HASH, NULL,
-				"filepath", OVAL_DATATYPE_STRING, pbuf,
+				"filepath", OVAL_DATATYPE_STRING, include_filepath ? pbuf : NULL,
 				"path",     OVAL_DATATYPE_STRING, p,
 				"filename", OVAL_DATATYPE_STRING, f,
 				NULL
@@ -143,19 +146,30 @@ static int filehash_cb (const char *p, const char *f, probe_ctx *ctx)
 
                 close (fd);
 
-                mem2hex (md5_dst,  sizeof md5_dst,  md5_str,  sizeof md5_str);
-                mem2hex (sha1_dst, sizeof sha1_dst, sha1_str, sizeof sha1_str);
+		md5_str[0] = '\0';
+		sha1_str[0] = '\0';
+                mem2hex (md5_dst,  md5_dstlen,  md5_str,  sizeof md5_str);
+                mem2hex (sha1_dst, sha1_dstlen, sha1_str, sizeof sha1_str);
 
                 /*
                  * Create and add the item
                  */
                 itm = probe_item_create(OVAL_INDEPENDENT_FILE_HASH, NULL,
-                                        "filepath", OVAL_DATATYPE_STRING, pbuf,
+                                        "filepath", OVAL_DATATYPE_STRING, include_filepath ? pbuf : NULL,
                                         "path",     OVAL_DATATYPE_STRING, p,
                                         "filename", OVAL_DATATYPE_STRING, f,
                                         "md5",      OVAL_DATATYPE_STRING, md5_str,
                                         "sha1",     OVAL_DATATYPE_STRING, sha1_str,
                                         NULL);
+
+		if (md5_dstlen == 0 || sha1_dstlen == 0)
+			probe_item_setstatus(itm, SYSCHAR_STATUS_ERROR);
+		if (md5_dstlen == 0)
+			probe_item_add_msg(itm, OVAL_MESSAGE_LEVEL_ERROR,
+					   "Unable to compute md5 hash value of \"%s\".", pbuf);
+		if (sha1_dstlen == 0)
+			probe_item_add_msg(itm, OVAL_MESSAGE_LEVEL_ERROR,
+					   "Unable to compute sha1 hash value of \"%s\".", pbuf);
         }
 
         probe_item_collect(ctx, itm);
@@ -165,8 +179,6 @@ static int filehash_cb (const char *p, const char *f, probe_ctx *ctx)
 
 void *probe_init (void)
 {
-        _LOGCALL_;
-
         /*
          * Initialize crypto API
          */
@@ -180,9 +192,11 @@ void *probe_init (void)
         case 0:
                 return ((void *)&__filehash_probe_mutex);
         default:
-                _D("Can't initialize mutex: errno=%u, %s.\n", errno, strerror (errno));
+                dI("Can't initialize mutex: errno=%u, %s.\n", errno, strerror (errno));
         }
 
+        probe_setoption(PROBEOPT_OFFLINE_MODE_SUPPORTED, true);
+
         return (NULL);
 }
 
@@ -204,6 +218,7 @@ int probe_main (probe_ctx *ctx, void *mutex)
 
 	OVAL_FTS    *ofts;
 	OVAL_FTSENT *ofts_ent;
+	oval_version_t over;
 
         if (mutex == NULL) {
 		return (PROBE_EINIT);
@@ -217,6 +232,7 @@ int probe_main (probe_ctx *ctx, void *mutex)
         filename  = probe_obj_getent (probe_in, "filename",  1);
         behaviors = probe_obj_getent (probe_in, "behaviors", 1);
         filepath = probe_obj_getent (probe_in, "filepath", 1);
+	over = probe_obj_get_schema_version(probe_in);
 
         /* we want either path+filename or filepath */
         if ( (path == NULL || filename == NULL) && filepath==NULL ) {
@@ -228,19 +244,13 @@ int probe_main (probe_ctx *ctx, void *mutex)
 		return (PROBE_ENOELM);
         }
 
-        /* behaviours are not important if filepath is used */
-        if(filepath != NULL && behaviors != NULL) {
-                SEXP_free (behaviors);
-                behaviors = NULL;
-        }
-
 	probe_filebehaviors_canonicalize(&behaviors);
 
         switch (pthread_mutex_lock (&__filehash_probe_mutex)) {
         case 0:
                 break;
         default:
-                _D("Can't lock mutex(%p): %u, %s.\n", &__filehash_probe_mutex, errno, strerror (errno));
+                dI("Can't lock mutex(%p): %u, %s.\n", &__filehash_probe_mutex, errno, strerror (errno));
 
 		SEXP_free (behaviors);
 		SEXP_free (path);
@@ -252,7 +262,7 @@ int probe_main (probe_ctx *ctx, void *mutex)
 
 	if ((ofts = oval_fts_open(path, filename, filepath, behaviors)) != NULL) {
 		while ((ofts_ent = oval_fts_read(ofts)) != NULL) {
-			filehash_cb(ofts_ent->path, ofts_ent->file, ctx);
+			filehash_cb(ofts_ent->path, ofts_ent->file, ctx, over);
 			oval_ftsent_free(ofts_ent);
 		}
 
@@ -268,7 +278,7 @@ int probe_main (probe_ctx *ctx, void *mutex)
         case 0:
                 break;
         default:
-                _D("Can't unlock mutex(%p): %u, %s.\n", &__filehash_probe_mutex, errno, strerror (errno));
+                dI("Can't unlock mutex(%p): %u, %s.\n", &__filehash_probe_mutex, errno, strerror (errno));
 
 		return (PROBE_EFATAL);
         }
