@@ -30,38 +30,6 @@
  *      "Tomas Heinrich" <theinric@redhat.com>
  */
 
-
-/*
- * textfilecontent54 probe
- *
- *   reference:
- *     http://oval.mitre.org/language/version5.6/ovaldefinition/documentation/independent-definitions-schema.html#textfilecontent54_object
- *     http://oval.mitre.org/language/version5.6/ovalsc/documentation/independent-system-characteristics-schema.html#textfilecontent_item
- *
- *   object:
- *     behaviors ind-def:Textfilecontent54Behaviors 0 1
- *       max_depth         xsd:integer       (optional -- default='1')
- *       recurse_direction xsd:string (enum) (optional -- default='none')
- *       ignore_case       xsd:boolean       (optional -- default='false')
- *       multiline         xsd:boolean       (optional -- default='true')
- *       singleline        xsd:boolean       (optional -- default='false')
- *     filepath  oval-def:EntityObjectStringType    1 1
- *     path      oval-def:EntityObjectStringType    1 1
- *     filename  oval-def:EntityObjectStringType    1 1
- *     pattern   oval-def:EntityObjectStringType    1 1
- *     instance  oval-def:EntityObjectIntType       1 1
- *
- *   item:
- *     filepath      oval-sc:EntityItemStringType 0 1
- *     path          oval-sc:EntityItemStringType 0 1
- *     filename      oval-sc:EntityItemStringType 0 1
- *     pattern       oval-sc:EntityItemStringType 0 1
- *     instance      oval-sc:EntityItemIntType    0 1
- *     line          oval-sc:EntityItemStringType 0 1
- *     text          oval-sc:EntityItemStringType 0 1
- *     subexpression oval-sc:EntityItemAnyType    0 unbounded
- */
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -84,6 +52,7 @@
 #include <seap.h>
 #include <probe-api.h>
 #include <probe/entcmp.h>
+#include <probe/option.h>
 #include <oval_fts.h>
 #include <alloc.h>
 #include "common/assume.h"
@@ -91,6 +60,8 @@
 
 #define FILE_SEPARATOR '/'
 
+oval_version_t over;
+
 #if defined USE_REGEX_PCRE
 static int get_substrings(char *str, int *ofs, pcre *re, int want_substrs, char ***substrings) {
 	int i, ret, rc;
@@ -121,6 +92,7 @@ static int get_substrings(char *str, int *ofs, pcre *re, int want_substrs, char
 	ret = 0;
 	if (rc == 0) {
 		/* vector too small */
+		// todo: report partial results
 		rc = ovector_len / 3;
 	}
 
@@ -191,21 +163,33 @@ static SEXP_t *create_item(const char *path, const char *filename, char *pattern
 	int i;
 	SEXP_t *item;
 	SEXP_t *r0;
-        char filepath[PATH_MAX+1];
+	SEXP_t *se_instance, *se_filepath;
+	char *text;
 
         if (strlen(path) + strlen(filename) + 1 > PATH_MAX) {
                 dE("path+filename too long\n");
                 return (NULL);
         }
 
-        snprintf(filepath, PATH_MAX, "%s%c%s", path, FILE_SEPARATOR, filename);
+	if (oval_version_cmp(over, OVAL_VERSION(5.4)) < 0) {
+		pattern = text = NULL;
+		se_instance = NULL;
+	} else {
+		text = substrs[0];
+		se_instance = SEXP_number_newu_64((int64_t) instance);
+	}
+	if (oval_version_cmp(over, OVAL_VERSION(5.6)) < 0) {
+		se_filepath = NULL;
+	} else {
+		se_filepath = SEXP_string_newf("%s%c%s", path, FILE_SEPARATOR, filename);
+	}
 
         item = probe_item_create(OVAL_INDEPENDENT_TEXT_FILE_CONTENT, NULL,
-                                 "filepath", OVAL_DATATYPE_STRING, filepath,
+                                 "filepath", OVAL_DATATYPE_SEXP, se_filepath,
                                  "path",     OVAL_DATATYPE_STRING, path,
                                  "filename", OVAL_DATATYPE_STRING, filename,
                                  "pattern",  OVAL_DATATYPE_STRING, pattern,
-                                 "instance", OVAL_DATATYPE_INTEGER, (int64_t)instance,
+                                 "instance", OVAL_DATATYPE_SEXP, se_instance,
                                  "line",     OVAL_DATATYPE_STRING, pattern,
                                  "text",     OVAL_DATATYPE_STRING, substrs[0],
                                  NULL);
@@ -223,6 +207,11 @@ struct pfdata {
 	int re_opts;
 	SEXP_t *instance_ent;
         probe_ctx *ctx;
+#if defined USE_REGEX_PCRE
+	pcre *compiled_regex;
+#elif defined USE_REGEX_POSIX
+	regex_t *compiled_regex;
+#endif
 };
 
 static int process_file(const char *path, const char *file, void *arg)
@@ -232,37 +221,8 @@ static int process_file(const char *path, const char *file, void *arg)
 		buf_size = 0, buf_used = 0, ofs = 0, buf_inc = 4096;
 	char *whole_path = NULL, *buf = NULL;
 	SEXP_t *next_inst = NULL;
+	struct stat st;
 
-// todo: move to probe_main()?
-#if defined USE_REGEX_PCRE
-	int erroffset = -1;
-	pcre *re = NULL;
-	const char *error;
-
-	re = pcre_compile(pfd->pattern, pfd->re_opts, &error, &erroffset, NULL);
-	if (re == NULL) {
-		SEXP_t *msg;
-
-		msg = probe_msg_creatf(OVAL_MESSAGE_LEVEL_ERROR, "pcre_compile() '%s' %s.", pfd->pattern, error);
-		probe_cobj_add_msg(probe_ctx_getresult(pfd->ctx), msg);
-		SEXP_free(msg);
-		probe_cobj_set_flag(probe_ctx_getresult(pfd->ctx), SYSCHAR_FLAG_ERROR);
-		return -1;
-	}
-#elif defined USE_REGEX_POSIX
-	regex_t _re, *re = &_re;
-
-	int err;
-	if ( (err=regcomp(re, pfd->pattern, pfd->re_opts)) != 0) {
-		SEXP_t *msg;
-
-		msg = probe_msg_creatf(OVAL_MESSAGE_LEVEL_ERROR, "regcomp() '%s' returned %d.", pfd->pattern, err);
-		probe_cobj_add_msg(probe_ctx_getresult(pfd->ctx), msg);
-		SEXP_free(msg);
-		probe_cobj_set_flag(probe_ctx_getresult(pfd->ctx), SYSCHAR_FLAG_ERROR);
-		return -1;
-	}
-#endif
 	if (file == NULL)
 		goto cleanup;
 
@@ -279,6 +239,20 @@ static int process_file(const char *path, const char *file, void *arg)
 
 	memcpy(whole_path + path_len, file, file_len + 1);
 
+	/*
+	 * If stat() fails, don't report an error and just skip the file.
+	 * This is an expected situation, because the fts_*() functions
+	 * are called with the 'FTS_PHYSICAL' option. Normally, stumbling
+	 * upon a symlink without a target would cause fts_read() to return
+	 * the 'FTS_SLNONE' flag, but the 'FTS_PHYSICAL' option causes it
+	 * to return 'FTS_SL' and the presence of a valid target has to
+	 * be determined with stat().
+	 */
+	if (stat(whole_path, &st) == -1)
+		goto cleanup;
+	if (!S_ISREG(st.st_mode))
+		goto cleanup;
+
 	fd = open(whole_path, O_RDONLY);
 	if (fd == -1) {
 		SEXP_t *msg;
@@ -287,7 +261,6 @@ static int process_file(const char *path, const char *file, void *arg)
 		probe_cobj_add_msg(probe_ctx_getresult(pfd->ctx), msg);
 		SEXP_free(msg);
 		probe_cobj_set_flag(probe_ctx_getresult(pfd->ctx), SYSCHAR_FLAG_ERROR);
-
 		ret = -1;
 		goto cleanup;
 	}
@@ -299,11 +272,10 @@ static int process_file(const char *path, const char *file, void *arg)
 		if (ret == -1) {
 			SEXP_t *msg;
 
-			msg = probe_msg_creatf(OVAL_MESSAGE_LEVEL_ERROR, "read(): %s.", strerror(errno));
+			msg = probe_msg_creatf(OVAL_MESSAGE_LEVEL_ERROR, "read(): '%s' %s.", whole_path, strerror(errno));
 			probe_cobj_add_msg(probe_ctx_getresult(pfd->ctx), msg);
 			SEXP_free(msg);
 			probe_cobj_set_flag(probe_ctx_getresult(pfd->ctx), SYSCHAR_FLAG_ERROR);
-
 			ret = -2;
 			goto cleanup;
 		}
@@ -326,7 +298,7 @@ static int process_file(const char *path, const char *file, void *arg)
 			want_instance = 0;
 
 		SEXP_free(next_inst);
-		substr_cnt = get_substrings(buf, &ofs, re, want_instance, &substrs);
+		substr_cnt = get_substrings(buf, &ofs, pfd->compiled_regex, want_instance, &substrs);
 
 		if (substr_cnt > 0) {
 			++cur_inst;
@@ -353,31 +325,42 @@ static int process_file(const char *path, const char *file, void *arg)
 	oscap_free(buf);
 	if (whole_path != NULL)
 		oscap_free(whole_path);
-#if defined USE_REGEX_PCRE
-	if (re != NULL)
-		pcre_free(re);
-#elif defined USE_REGEX_POSIX
-	regfree(re);
-#endif
+
 	return ret;
 }
 
+void *probe_init(void)
+{
+  probe_setoption(PROBEOPT_OFFLINE_MODE_SUPPORTED, true);
+  return NULL;
+}
+
 int probe_main(probe_ctx *ctx, void *arg)
 {
 	SEXP_t *path_ent, *file_ent, *inst_ent, *bh_ent, *patt_ent, *filepath_ent, *probe_in;
         SEXP_t *r0;
-	char *pattern;
 	/* char *i_val, *m_val, *s_val; */
 	bool val;
 	struct pfdata pfd;
-
+	int ret = 0;
+#if defined USE_REGEX_PCRE
+	int errorffset = -1;
+	const char *error;
+#elif defined USE_REGEX_POSIX
+	regex_t _re;
+	pfd.compiled_regex = &_re;
+	int err;
+#endif
 	OVAL_FTS    *ofts;
 	OVAL_FTSENT *ofts_ent;
 
         (void)arg;
 
+	memset(&pfd, 0, sizeof(pfd));
+
         probe_in = probe_ctx_getobject(ctx);
 
+	over = probe_obj_get_schema_version(probe_in);
         path_ent = probe_obj_getent(probe_in, "path",     1);
         file_ent = probe_obj_getent(probe_in, "filename", 1);
         inst_ent = probe_obj_getent(probe_in, "instance", 1);
@@ -389,21 +372,16 @@ int probe_main(probe_ctx *ctx, void *arg)
         if ( ((path_ent == NULL || file_ent == NULL) && filepath_ent==NULL) || 
              inst_ent==NULL || 
              patt_ent==NULL) {
-                SEXP_free (path_ent);
-                SEXP_free (file_ent);
-                SEXP_free (inst_ent);
                 SEXP_free (patt_ent);
-                SEXP_free (filepath_ent);
-		SEXP_free (bh_ent);
-
-                return PROBE_ENOELM;
+		ret = PROBE_ENOELM;
+		goto cleanup;
         }
 
 	/* get pattern from SEXP */
         SEXP_t *ent_val;
         ent_val = probe_ent_getval(patt_ent);
-        pattern = SEXP_string_cstr(ent_val);
-        assume_d(pattern != NULL, -1);
+	pfd.pattern = SEXP_string_cstr(ent_val);
+	assume_d(pfd.pattern != NULL, -1);
         SEXP_free(patt_ent);
         SEXP_free(ent_val);
 
@@ -430,7 +408,6 @@ int probe_main(probe_ctx *ctx, void *arg)
 
 	probe_tfc54behaviors_canonicalize(&bh_ent);
 
-	pfd.pattern      = pattern;
 	pfd.instance_ent = inst_ent;
         pfd.ctx          = ctx;
 #if defined USE_REGEX_PCRE
@@ -456,6 +433,18 @@ int probe_main(probe_ctx *ctx, void *arg)
 		if (val)
 			pfd.re_opts |= PCRE_DOTALL;
 	}
+
+	pfd.compiled_regex = pcre_compile(pfd.pattern, pfd.re_opts, &error,
+					  &errorffset, NULL);
+	if (pfd.compiled_regex == NULL) {
+		SEXP_t *msg;
+
+		msg = probe_msg_creatf(OVAL_MESSAGE_LEVEL_ERROR, "pcre_compile() '%s' %s.", pfd.pattern, error);
+		probe_cobj_add_msg(probe_ctx_getresult(pfd.ctx), msg);
+		SEXP_free(msg);
+		probe_cobj_set_flag(probe_ctx_getresult(pfd.ctx), SYSCHAR_FLAG_ERROR);
+		goto cleanup;
+	}
 #elif defined USE_REGEX_POSIX
 	pfd.re_opts = REG_EXTENDED | REG_NEWLINE;
 	r0 = probe_ent_getattrval(bh_ent, "ignore_case");
@@ -465,23 +454,43 @@ int probe_main(probe_ctx *ctx, void *arg)
 		if (val)
 			pfd.re_opts |= REG_ICASE;
 	}
-#endif
 
+	if ((err = regcomp(pfd.compiled_regex, pfd.pattern, pfd.re_opts)) != 0) {
+		SEXP_t *msg;
+
+		msg = probe_msg_creatf(OVAL_MESSAGE_LEVEL_ERROR, "regcomp() '%s' returned %d.", pfd.pattern, err);
+		probe_cobj_add_msg(probe_ctx_getresult(pfd.ctx), msg);
+		SEXP_free(msg);
+		probe_cobj_set_flag(probe_ctx_getresult(pfd.ctx), SYSCHAR_FLAG_ERROR);
+		goto cleanup;
+	}
+#endif
 	if ((ofts = oval_fts_open(path_ent, file_ent, filepath_ent, bh_ent)) != NULL) {
 		while ((ofts_ent = oval_fts_read(ofts)) != NULL) {
-			process_file(ofts_ent->path, ofts_ent->file, &pfd);
+			if (ofts_ent->fts_info == FTS_F
+			    || ofts_ent->fts_info == FTS_SL) {
+				// todo: handle return code
+				process_file(ofts_ent->path, ofts_ent->file, &pfd);
+			}
 			oval_ftsent_free(ofts_ent);
 		}
 
 		oval_fts_close(ofts);
 	}
 
+ cleanup:
         SEXP_free(file_ent);
         SEXP_free(path_ent);
         SEXP_free(inst_ent);
         SEXP_free(bh_ent);
         SEXP_free(filepath_ent);
-        oscap_free(pattern);
-
-	return 0;
+	if (pfd.pattern != NULL)
+		oscap_free(pfd.pattern);
+#if defined USE_REGEX_PCRE
+	if (pfd.compiled_regex != NULL)
+		pcre_free(pfd.compiled_regex);
+#elif defined USE_REGEX_POSIX
+	regfree(re);
+#endif
+	return ret;
 }
