@@ -19,6 +19,11 @@
  * Authors:
  *      Daniel Kopecek <dkopecek@redhat.com>
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <errno.h>
 #include <pthread.h>
 #include <string.h>
@@ -43,7 +48,7 @@ void *probe_input_handler(void *arg)
         pthread_attr_t pth_attr;
         probe_t       *probe = (probe_t *)arg;
 
-        int probe_ret, ret, cstate; /* XXX */
+        int probe_ret, cstate; /* XXX */
         SEAP_msg_t *seap_request, *seap_reply;
         SEXP_t *probe_in, *probe_out, *oid;
 
@@ -61,13 +66,22 @@ void *probe_input_handler(void *arg)
         }
 
         pthread_cleanup_push((void(*)(void *))pthread_attr_destroy, (void *)&pth_attr);
+        
+        switch (errno = pthread_barrier_wait(&OSCAP_GSYM(th_barrier)))
+        {
+        case 0:
+        case PTHREAD_BARRIER_SERIAL_THREAD:
+	        break;
+        default:
+	        dE("pthread_barrier_wait: %d, %s.\n",
+	           errno, strerror(errno));
+	        return (NULL);
+        }
 
 	while(1) {
                 TH_CANCEL_ON;
 
 		if (SEAP_recvmsg(probe->SEAP_ctx, probe->sd, &seap_request) == -1) {
-			ret = errno;
-
 			dE("An error ocured while receiving SEAP message. errno=%u, %s.\n", errno, strerror(errno));
 
                         /*
@@ -91,59 +105,95 @@ void *probe_input_handler(void *arg)
                  * result cache.
                  */
 		oid = probe_obj_getattrval(probe_in, "id");
-                SEXP_free(probe_in);
 
 		if (oid != NULL) {
 			SEXP_VALIDATE(oid);
-
 			probe_out = probe_rcache_sexp_get(probe->rcache, oid);
-                        SEXP_free(oid);
 
-			if (probe_out == NULL) {
-				/* cache miss */
-                                probe_pwpair_t *pair;
+			if (OSCAP_GSYM(offline_mode) && !OSCAP_GSYM(offline_mode_supported)) {
+				probe_out = probe_cobj_new(OSCAP_GSYM(offline_mode_cobjflag), NULL, NULL, NULL);
+				if (probe_rcache_sexp_add(probe->rcache, oid, probe_out) != 0) {
+					abort();
+				}
+			}
 
-                                pair = oscap_talloc(probe_pwpair_t);
-                                pair->probe = probe;
-                                pair->pth   = probe_worker_new();
-                                pair->pth->sid = SEAP_msg_id(seap_request);
-                                pair->pth->msg = seap_request;
-                                pair->pth->msg_handler = &probe_worker;
+			if (probe_out == NULL) { /* cache miss */
+				SEXP_t *skip_flag, *obj_mask;
 
-                                if (rbt_i32_add(probe->workers, pair->pth->sid, pair->pth, NULL) != 0) {
-                                        /*
-                                         * Getting here means that there is already a
-                                         * thread handling the message with the given
-                                         * ID.
-                                         */
-                                        dW("Attempt to evaluate an object "
-                                           "(ID=%u) " // TODO: 64b IDs
-                                           "which is already being evaluated by an other thread.\n", pair->pth->sid);
+				skip_flag = probe_obj_getattrval(probe_in, "skip_eval");
+                                obj_mask  = probe_obj_getmask(probe_in);
+				SEXP_free(probe_in);
 
-                                        oscap_free(pair->pth);
-                                        oscap_free(pair);
-                                        SEAP_msg_free(seap_request);
-                                } else {
-                                        /* OK */
+				if (skip_flag != NULL) {
+					oval_syschar_collection_flag_t cobj_flag;
 
-                                        if (pthread_create(&pair->pth->tid, &pth_attr, &probe_worker_runfn, pair))
-                                        {
-                                                dE("Cannot start a new worker thread: %d, %s.\n", errno, strerror(errno));
+					cobj_flag = SEXP_number_geti_32(skip_flag);
+					probe_out = probe_cobj_new(cobj_flag, NULL, NULL, obj_mask);
 
-                                                if (rbt_i32_del(probe->workers, pair->pth->sid, NULL) != 0) {
-                                                        /* ... do something ... */
-                                                }
+					if (probe_rcache_sexp_add(probe->rcache, oid, probe_out) != 0) {
+						/* TODO */
+						abort();
+					}
 
-                                                SEAP_msg_free(pair->pth->msg);
-                                                oscap_free(pair->pth);
-                                                oscap_free(pair);
-                                        }
-                                }
+					probe_ret = 0;
+					SEXP_free(oid);
+                                        SEXP_free(skip_flag);
+                                        SEXP_free(obj_mask);
+				} else {
+					probe_pwpair_t *pair;
 
-				seap_request = NULL;
-				continue;
+                                        SEXP_free(oid);
+					SEXP_free(skip_flag);
+					SEXP_free(obj_mask);
+
+                                        pair = oscap_talloc(probe_pwpair_t);
+					pair->probe = probe;
+					pair->pth   = probe_worker_new();
+					pair->pth->sid = SEAP_msg_id(seap_request);
+					pair->pth->msg = seap_request;
+					pair->pth->msg_handler = &probe_worker;
+
+					if (rbt_i32_add(probe->workers, pair->pth->sid, pair->pth, NULL) != 0) {
+						/*
+						 * Getting here means that there is already a
+						 * thread handling the message with the given
+						 * ID.
+						 */
+						dW("Attempt to evaluate an object "
+						   "(ID=%u) " // TODO: 64b IDs
+						   "which is already being evaluated by an other thread.\n", pair->pth->sid);
+
+						oscap_free(pair->pth);
+						oscap_free(pair);
+						SEAP_msg_free(seap_request);
+					} else {
+						/* OK */
+
+						if (pthread_create(&pair->pth->tid, &pth_attr, &probe_worker_runfn, pair))
+						{
+							dE("Cannot start a new worker thread: %d, %s.\n", errno, strerror(errno));
+
+							if (rbt_i32_del(probe->workers, pair->pth->sid, NULL) != 0)
+								dE("rbt_i32_del: failed to remove worker thread (ID=%u)\n", pair->pth->sid);
+
+							SEAP_msg_free(pair->pth->msg);
+							oscap_free(pair->pth);
+							oscap_free(pair);
+
+							probe_ret = PROBE_EUNKNOWN;
+							probe_out = NULL;
+
+							goto __error_reply;
+						}
+					}
+
+					seap_request = NULL;
+					continue;
+				}
 			} else {
 				/* cache hit */
+				SEXP_free(oid);
+				SEXP_free(probe_in);
 				probe_ret = 0;
 			}
 		} else {
@@ -154,6 +204,7 @@ void *probe_input_handler(void *arg)
                 }
 
 		if (probe_out == NULL || probe_ret != 0) {
+		__error_reply:
 			if (SEAP_replyerr(probe->SEAP_ctx, probe->sd, seap_request, probe_ret) == -1)
                         {
 				dE("An error ocured while sending error status. errno=%u, %s.\n",
@@ -171,8 +222,6 @@ void *probe_input_handler(void *arg)
                         SEXP_free(probe_out);
 
 			if (SEAP_reply(probe->SEAP_ctx, probe->sd, seap_reply, seap_request) == -1) {
-				ret = errno;
-
 				dE("An error ocured while sending SEAP message. errno=%u, %s.\n",
 				   errno, strerror(errno));
 
