@@ -20,6 +20,12 @@
  *      Daniel Kopecek <dkopecek@redhat.com>
  *      Tomas Heinrich <theinric@redhat.com>
  */
+#include <config.h>
+#if !defined(_GNU_SOURCE)
+# if defined(HAVE_PTHREAD_TIMEDJOIN_NP) && defined(HAVE_CLOCK_GETTIME)
+#  define _GNU_SOURCE
+# endif
+#endif
 #include <stdio.h>
 #include <string.h>
 #include <signal.h>
@@ -36,6 +42,7 @@
 #include "signal_handler.h"
 #include "input_handler.h"
 #include "probe-api.h"
+#include "option.h"
 
 static int fail(int err, const char *who, int line)
 {
@@ -47,6 +54,11 @@ void  *OSCAP_GSYM(probe_arg)          = NULL;
 bool   OSCAP_GSYM(varref_handling)    = true;
 char **OSCAP_GSYM(no_varref_ents)     = NULL;
 size_t OSCAP_GSYM(no_varref_ents_cnt) = 0;
+bool OSCAP_GSYM(offline_mode) = false;
+bool OSCAP_GSYM(offline_mode_supported) = false;
+int OSCAP_GSYM(offline_mode_cobjflag) = SYSCHAR_FLAG_NOT_APPLICABLE;
+
+pthread_barrier_t OSCAP_GSYM(th_barrier);
 
 extern probe_ncache_t *OSCAP_GSYM(ncache);
 
@@ -70,12 +82,15 @@ static SEXP_t *probe_reset(SEXP_t *arg0, void *arg1)
         return(NULL);
 }
 
-static int probe_opthandler_varref(int option, va_list args)
+static int probe_opthandler_varref(int option, int op, va_list args)
 {
 	bool  o_switch;
 	char *o_name;
 	char *o_temp;
 
+	if (op == PROBE_OPTION_GET)
+		return -1;
+
 	o_switch = va_arg(args, int);
 	o_name   = va_arg(args, char *);
 
@@ -101,16 +116,57 @@ static int probe_opthandler_varref(int option, va_list args)
 	return (0);
 }
 
-static int probe_opthandler_rcache(int option, va_list args)
+static int probe_opthandler_rcache(int option, int op, va_list args)
 {
 	return (0);
 }
 
+static int probe_opthandler_offlinemode(int option, int op, va_list args)
+{
+	if (op == PROBE_OPTION_SET) {
+		bool o_offline_mode = OSCAP_GSYM(offline_mode_supported);
+		int o_cobjflag = OSCAP_GSYM(offline_mode_cobjflag);
+
+		o_offline_mode = va_arg(args, int);
+		if (!o_offline_mode) {
+			/*
+			 * If the probe doesn't support offline mode, then probe_main()
+			 * won't be called in offline modee and a collected object with
+			 * the following flag will be generated for all queries.
+			 */
+			o_cobjflag = va_arg(args, int);
+		}
+		OSCAP_GSYM(offline_mode_supported) = o_offline_mode;
+		OSCAP_GSYM(offline_mode_cobjflag) = o_cobjflag;
+	} else if (op == PROBE_OPTION_GET) {
+		int *offline_mode_supported = va_arg(args, int *);
+		int *offline_mode = va_arg(args, int *);
+
+		if (offline_mode_supported != NULL) {
+		  *offline_mode_supported = OSCAP_GSYM(offline_mode_supported);
+		}
+		if (offline_mode != NULL) {
+		  *offline_mode = OSCAP_GSYM(offline_mode);
+		}
+	}
+	return 0;
+}
+
 int main(int argc, char *argv[])
 {
 	pthread_attr_t th_attr;
 	sigset_t       sigmask;
 	probe_t        probe;
+	char *rootdir = NULL;
+
+	if ((errno = pthread_barrier_init(&OSCAP_GSYM(th_barrier), NULL,
+	                                  1 + // signal thread
+	                                  1 + // input thread
+	                                  1 + // icache thread
+	                                  0)) != 0)
+	{
+		fail(errno, "pthread_barrier_init", __LINE__ - 6);
+	}
 
 	/*
 	 * Block signals, any signals received will be
@@ -157,15 +213,20 @@ int main(int argc, char *argv[])
 	/*
 	 * Initialize probe option handlers
 	 */
-#define PROBE_OPTION_INITCOUNT 2
+#define PROBE_OPTION_INITCOUNT 3
 
 	probe.option = oscap_alloc(sizeof(probe_option_t) * PROBE_OPTION_INITCOUNT);
 	probe.optcnt = PROBE_OPTION_INITCOUNT;
 
-        probe.option[0].option  = PROBE_VARREF_HANDLING;
-        probe.option[0].handler = &probe_opthandler_varref;
-        probe.option[1].option  = PROBE_RESULT_CACHING;
-        probe.option[1].handler = &probe_opthandler_rcache;
+	probe.option[0].option  = PROBEOPT_VARREF_HANDLING;
+	probe.option[0].handler = &probe_opthandler_varref;
+	probe.option[1].option  = PROBEOPT_RESULT_CACHING;
+	probe.option[1].handler = &probe_opthandler_rcache;
+	probe.option[2].option  = PROBEOPT_OFFLINE_MODE_SUPPORTED;
+	probe.option[2].handler = &probe_opthandler_offlinemode;
+
+	OSCAP_GSYM(probe_optdef) = probe.option;
+	OSCAP_GSYM(probe_optdef_count) = probe.optcnt;
 
 	/*
 	 * Create signal handler
@@ -180,6 +241,28 @@ int main(int argc, char *argv[])
 
 	pthread_attr_destroy(&th_attr);
 
+	/*
+	 * Change root directory for probes, if requested
+	 */
+	if ((rootdir = getenv("OSCAP_PROBE_ROOT")) != NULL) {
+		if(strlen(rootdir) > 0) {
+			if (chdir(rootdir) != 0) {
+				fail(errno, "chdir", __LINE__ -1);
+			}
+			if (chroot(rootdir) != 0) {
+				fail(errno, "chroot", __LINE__ - 1);
+			}
+			/* NOTE: We're running in a different root directory.
+			 * Unless /proc, /sys are somehow emulated for the new
+			 * environment, they are not relevant and so are other
+			 * runtime only things (e.g. getenv, uname, ...).
+			 * Switch to offline mode. We may add a separate
+			 * mechanism to control this behaviour in the future.
+			 */
+			OSCAP_GSYM(offline_mode) = true;
+		}
+	}
+
 	/*
 	 * Create input handler (detached)
 	 */
@@ -202,9 +285,22 @@ int main(int argc, char *argv[])
 	/*
 	 * Wait for the input_handler thread
 	 */
+#if defined(HAVE_PTHREAD_TIMEDJOIN_NP) && defined(HAVE_CLOCK_GETTIME)
+	{
+		struct timespec j_tm;
+
+		if (clock_gettime(CLOCK_REALTIME, &j_tm) == -1)
+			fail(errno, "clock_gettime", __LINE__ - 1);
+
+		j_tm.tv_sec += 3;
+
+		if (pthread_timedjoin_np(probe.th_input, NULL, &j_tm) != 0)
+			fail(errno, "pthread_timedjoin_np", __LINE__ - 1);
+	}
+#else
 	if (pthread_join(probe.th_input, NULL))
 		fail(errno, "pthread_join", __LINE__ - 1);
-
+#endif
 	/*
 	 * Cleanup
 	 */
