@@ -19,6 +19,11 @@
  * Authors:
  *      Daniel Kopecek <dkopecek@redhat.com>
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <seap.h>
 #include <stdlib.h>
 #include <string.h>
@@ -151,6 +156,8 @@ struct probe_varref_ctx_ent {
 	unsigned int next_val_idx;
 };
 
+static void probe_varref_destroy_ctx(struct probe_varref_ctx *ctx);
+
 static int probe_varref_create_ctx(const SEXP_t *probe_in, SEXP_t *varrefs, struct probe_varref_ctx **octx)
 {
 	unsigned int i, ent_cnt, val_cnt;
@@ -200,6 +207,7 @@ static int probe_varref_create_ctx(const SEXP_t *probe_in, SEXP_t *varrefs, stru
 		if (varref == NULL) {
 			dE("Unexpected error: variable id \"%s\" not found in varrefs.", SEXP_string_cstr(vid));
 			SEXP_vfree(vid, ent_name, vidx_name, vidx_val, NULL);
+			probe_varref_destroy_ctx(ctx);
 			return -1;
 		}
 
@@ -302,8 +310,6 @@ static SEXP_t *probe_ste_fetch(probe_t *probe, SEXP_t *id_list)
 	SEXP_t *res, *ste, *id;
 	uint32_t i_len, r_len;
 
-	_LOGCALL_;
-
 	i_len = SEXP_list_length(id_list);
 
 	if (i_len == 0)
@@ -357,8 +363,6 @@ static SEXP_t *probe_obj_eval(probe_t *probe, SEXP_t *id)
 {
 	SEXP_t *res, *rid;
 
-	_LOGCALL_;
-
 	res = SEAP_cmd_exec(probe->SEAP_ctx, probe->sd, 0, PROBECMD_OBJ_EVAL, id, SEAP_CMDTYPE_SYNC, NULL, NULL);
 
 	rid = SEXP_list_first(res);
@@ -415,7 +419,7 @@ static SEXP_t *probe_prepare_filters(probe_t *probe, SEXP_t *obj)
  */
 static SEXP_t *probe_set_combine(SEXP_t *cobj0, SEXP_t *cobj1, oval_setobject_operation_t op)
 {
-        SEXP_t *set0, *set1, *res_cobj;
+        SEXP_t *set0, *set1, *res_cobj, *cobj0_mask, *cobj1_mask, *res_mask;
         register int cmp;
         register SEXP_t *item0, *item1, *res;
         register SEXP_list_it *sit0, *sit1;
@@ -428,11 +432,14 @@ static SEXP_t *probe_set_combine(SEXP_t *cobj0, SEXP_t *cobj1, oval_setobject_op
 
         set0 = probe_cobj_get_items(cobj0);
         set1 = probe_cobj_get_items(cobj1);
+        cobj0_mask = probe_cobj_get_mask(cobj0);
+        cobj1_mask = probe_cobj_get_mask(cobj1);
 
         /* prepare storage for results */
         res = SEXP_list_new(NULL);
         res_flag = probe_cobj_combine_flags(probe_cobj_get_flag(cobj0),
                                             probe_cobj_get_flag(cobj1), op);
+        res_mask = SEXP_list_join(cobj0_mask, cobj1_mask);
 
         /* prepare iterators & first items */
         sit0  = SEXP_list_it_new(set0);
@@ -523,11 +530,10 @@ static SEXP_t *probe_set_combine(SEXP_t *cobj0, SEXP_t *cobj1, oval_setobject_op
 	if (res_flag == SYSCHAR_FLAG_COMPLETE && SEXP_list_length(res) == 0)
 		res_flag = SYSCHAR_FLAG_DOES_NOT_EXIST;
 
-	res_cobj = probe_cobj_new(res_flag, NULL, res);
+	res_cobj = probe_cobj_new(res_flag, NULL, res, res_mask);
 
-	SEXP_free(set0);
-	SEXP_free(set1);
-	SEXP_free(res);
+        SEXP_vfree(set0, set1, res, res_mask);
+        SEXP_vfree(cobj0_mask, cobj1_mask);
 
 	// todo: variables
 
@@ -542,15 +548,14 @@ static SEXP_t *probe_set_combine(SEXP_t *cobj0, SEXP_t *cobj1, oval_setobject_op
  */
 static SEXP_t *probe_set_apply_filters(SEXP_t *cobj, SEXP_t *filters)
 {
-	SEXP_t *result_items, *items, *item;
+	SEXP_t *result_items, *items, *item, *mask;
 	oval_syschar_status_t item_status;
 	oval_syschar_collection_flag_t flag;
 
-	_LOGCALL_;
-
 	result_items = SEXP_list_new(NULL);
 	flag = probe_cobj_get_flag(cobj);
 	items = probe_cobj_get_items(cobj);
+        mask = probe_cobj_get_mask(cobj);
 
 	SEXP_list_foreach(item, items) {
 		item_status = probe_ent_getstatus(item);
@@ -567,7 +572,7 @@ static SEXP_t *probe_set_apply_filters(SEXP_t *cobj, SEXP_t *filters)
                                 r0 = probe_msg_creatf(OVAL_MESSAGE_LEVEL_ERROR,
                                                       "Supplied item has an invalid status: %d.", item_status);
 				r1 = SEXP_list_new(r0, NULL);
-				cobj = probe_cobj_new(SYSCHAR_FLAG_ERROR, r1, NULL);
+				cobj = probe_cobj_new(SYSCHAR_FLAG_ERROR, r1, NULL, NULL);
 				SEXP_vfree(items, item, result_items, r0, r1, NULL);
 				return cobj;
 			}
@@ -588,8 +593,8 @@ static SEXP_t *probe_set_apply_filters(SEXP_t *cobj, SEXP_t *filters)
 	    && SEXP_list_length(result_items) == 0)
 		flag = SYSCHAR_FLAG_DOES_NOT_EXIST;
 
-	cobj = probe_cobj_new(flag, NULL, result_items);
-	SEXP_vfree(items, result_items, NULL);
+	cobj = probe_cobj_new(flag, NULL, result_items, mask);
+	SEXP_vfree(items, result_items, mask);
 
 	return cobj;
 }
@@ -619,17 +624,15 @@ static SEXP_t *probe_set_eval(probe_t *probe, SEXP_t *set, size_t depth)
 
 	SEXP_t *r0, *r1, *result, *Omsg = NULL;
 
-	_LOGCALL_;
-
 	if (depth > MAX_EVAL_DEPTH) {
 		char *fmt = "probe_set_eval: Too many levels: max=%zu.";
 #ifndef NDEBUG
-                _D(fmt, (size_t) MAX_EVAL_DEPTH);
+                dI(fmt, (size_t) MAX_EVAL_DEPTH);
 		abort();
 #endif
 		r0 = probe_msg_creatf(OVAL_MESSAGE_LEVEL_ERROR, fmt, (size_t) MAX_EVAL_DEPTH);
 		r1 = SEXP_list_new(r0, NULL);
-		result = probe_cobj_new(SYSCHAR_FLAG_ERROR, r1, NULL);
+		result = probe_cobj_new(SYSCHAR_FLAG_ERROR, r1, NULL, NULL);
 		SEXP_vfree(r0, r1, NULL);
 		return result;
 	}
@@ -830,22 +833,22 @@ static SEXP_t *probe_set_eval(probe_t *probe, SEXP_t *set, size_t depth)
 
                 for (i = 0; i < s_subset_i; ++i) {
                         if (s_subset[i] != NULL) {
-                                _I("=== s_subset[%d] ===\n", i);
-                                _SE(s_subset[i]);
-                                _I("\n");
+                                dI("=== s_subset[%d] ===\n", i);
+                                dO(OSCAP_DEBUGOBJ_SEXP, s_subset[i]);
+                                dI("\n");
                         }
                 }
 
                 for (i = 0; i < o_subset_i; ++i) {
                         if (o_subset[i] != NULL) {
-                                _I("=== o_subset[%d] ===\n", i);
-                                _SE(o_subset[i]);
-                                _I("\n");
+                                dI("=== o_subset[%d] ===\n", i);
+                                dO(OSCAP_DEBUGOBJ_SEXP, o_subset[i]);
+                                dI("\n");
                         }
                 }
         }
 
-        _I("OP= %d\n", op_num);
+        dI("OP= %d\n", op_num);
 #endif
 
 	SEXP_free(filters_a);
@@ -856,9 +859,9 @@ static SEXP_t *probe_set_eval(probe_t *probe, SEXP_t *set, size_t depth)
 	SEXP_free(s_subset[0]);
 	SEXP_free(s_subset[1]);
 
-        _I("=== RESULT ===\n");
-        _SE(result);
-        _I("\n");
+        dI("=== RESULT ===\n");
+        dO(OSCAP_DEBUGOBJ_SEXP, result);
+        dI("\n");
 
 	return (result);
  eval_fail:
@@ -876,7 +879,7 @@ static SEXP_t *probe_set_eval(probe_t *probe, SEXP_t *set, size_t depth)
 	SEXP_free(result);
 
         r1 = SEXP_list_new(Omsg, NULL);
-	result = probe_cobj_new(SYSCHAR_FLAG_ERROR, r1, NULL);
+	result = probe_cobj_new(SYSCHAR_FLAG_ERROR, r1, NULL, NULL);
 	SEXP_vfree(Omsg, r1, NULL);
 	return result;
 }
@@ -913,11 +916,12 @@ SEXP_t *probe_worker(probe_t *probe, SEAP_msg_t *msg_in, int *ret)
 		*ret = 0;
 	} else {
                 struct probe_ctx pctx;
-		SEXP_t *varrefs;
+		SEXP_t *varrefs, *mask;
 
 		/* simple object */
                 pctx.icache  = probe->icache;
 		pctx.filters = probe_prepare_filters(probe, probe_in);
+                mask = probe_obj_getmask(probe_in);
 
 		if (OSCAP_GSYM(varref_handling))
 			varrefs = probe_obj_getent(probe_in, "varrefs", 1);
@@ -928,8 +932,9 @@ SEXP_t *probe_worker(probe_t *probe, SEAP_msg_t *msg_in, int *ret)
                         /*
                          * Prepare the collected object
                          */
-			probe_out = probe_cobj_new(SYSCHAR_FLAG_UNKNOWN, NULL, NULL);
-
+			probe_out = probe_cobj_new(SYSCHAR_FLAG_UNKNOWN, NULL, NULL, mask);
+			SEXP_free(mask);
+			
                         pctx.probe_in  = probe_in;
                         pctx.probe_out = probe_out;
                         /*
@@ -953,7 +958,7 @@ SEXP_t *probe_worker(probe_t *probe, SEAP_msg_t *msg_in, int *ret)
 			dI("handling varrefs in object\n");
 
 			if (probe_varref_create_ctx(probe_in, varrefs, &ctx) != 0) {
-				SEXP_vfree(varrefs, pctx.filters, probe_in, NULL);
+				SEXP_vfree(varrefs, pctx.filters, probe_in, mask, NULL);
 				*ret = PROBE_EUNKNOWN;
 				return (NULL);
 			}
@@ -965,7 +970,7 @@ SEXP_t *probe_worker(probe_t *probe, SEAP_msg_t *msg_in, int *ret)
                                 /*
                                  * Prepare the collected object
                                  */
-				cobj = probe_cobj_new(SYSCHAR_FLAG_UNKNOWN, NULL, NULL);
+				cobj = probe_cobj_new(SYSCHAR_FLAG_UNKNOWN, NULL, NULL, mask);
 
                                 pctx.probe_in  = ctx->pi2;
                                 pctx.probe_out = cobj;
@@ -986,6 +991,7 @@ SEXP_t *probe_worker(probe_t *probe, SEAP_msg_t *msg_in, int *ret)
 			} while (*ret == 0
 				 && probe_varref_iterate_ctx(ctx));
 
+			SEXP_free(mask);
 			probe_varref_destroy_ctx(ctx);
 		}
 
