@@ -275,9 +275,10 @@ size_t probe_obj_getname_r(const SEXP_t * obj, char *buffer, size_t buflen);
  * collected objects
  */
 
-SEXP_t *probe_cobj_new(oval_syschar_collection_flag_t flag, SEXP_t *msg_list, SEXP_t *item_list);
+SEXP_t *probe_cobj_new(oval_syschar_collection_flag_t flag, SEXP_t *msg_list, SEXP_t *item_list, SEXP_t *mask_list);
 int probe_cobj_add_msg(SEXP_t *cobj, const SEXP_t *msg);
 SEXP_t *probe_cobj_get_msgs(const SEXP_t *cobj);
+SEXP_t *probe_cobj_get_mask(const SEXP_t *cobj);
 int probe_cobj_add_item(SEXP_t *cobj, const SEXP_t *item);
 SEXP_t *probe_cobj_get_items(const SEXP_t *cobj);
 void probe_cobj_set_flag(SEXP_t *cobj, oval_syschar_collection_flag_t flag);
@@ -474,11 +475,6 @@ typedef struct probe_ctx probe_ctx;
 
 int probe_main(probe_ctx *, void *) __attribute__ ((nonnull(1)));
 
-#define PROBE_VARREF_HANDLING 0
-#define PROBE_RESULT_CACHING  1
-
-int probe_setoption(int option, ...);
-
 bool probe_item_filtered(const SEXP_t *item, const SEXP_t *filters);
 
 int probe_result_additem(SEXP_t *result, SEXP_t *item);
@@ -517,7 +513,7 @@ SEXP_t *probe_item_create(oval_subtype_t item_subtype, probe_elmatr_t *item_attr
 #define PROBE_ENT_AREF(ent, dst, attr_name, invalid_exp)		\
 	do {								\
 		if (((dst) = probe_ent_getattrval(ent, attr_name)) == NULL) { \
-			_F("Attribute `%s' is missing!\n", attr_name);	\
+			dE("Attribute `%s' is missing!\n", attr_name);	\
 			invalid_exp					\
 		}               					\
 	} while(0)
@@ -527,11 +523,11 @@ SEXP_t *probe_item_create(oval_subtype_t item_subtype, probe_elmatr_t *item_attr
 		SEXP_t *___r;					\
 								\
 		if ((___r = probe_ent_getval(ent)) == NULL) {	\
-			_W("Entity has no value!\n");		\
+			dW("Entity has no value!\n");		\
 			invalid_exp				\
 		} else {					\
 			if (!SEXP_stringp(___r)) {		\
-				_F("Invalid type\n");		\
+				dE("Invalid type\n");		\
 				SEXP_free(___r);		\
 				invalid_exp			\
 			}					\
@@ -550,11 +546,11 @@ SEXP_t *probe_item_create(oval_subtype_t item_subtype, probe_elmatr_t *item_attr
 		SEXP_t *___r;					\
 								\
 		if ((___r = probe_ent_getval(ent)) == NULL) {	\
-			_W("Entity has no value!\n");		\
+			dW("Entity has no value!\n");		\
 			nil_exp;				\
 		} else {					\
 			if (!SEXP_numberp(___r)) {		\
-				_F("Invalid type\n");		\
+				dE("Invalid type\n");		\
 				SEXP_free(___r);		\
 				invalid_exp;			\
 			} else {				\
@@ -569,3 +565,16 @@ SEXP_t *probe_item_create(oval_subtype_t item_subtype, probe_elmatr_t *item_attr
 oval_operation_t probe_ent_getoperation(SEXP_t *entity, oval_operation_t op);
 
 int probe_item_add_msg(SEXP_t *item, oval_message_level_t msglvl, char *msgfmt, ...);
+
+SEXP_t *probe_entval_from_cstr(oval_datatype_t type, const char *value, size_t vallen);
+SEXP_t *probe_ent_from_cstr(const char *name, oval_datatype_t type, const char *value, size_t vallen);
+
+oval_version_t probe_obj_get_schema_version(const SEXP_t *obj);
+
+/**
+ * Get object entity mask
+ * @returns a list of masked entities
+ */
+SEXP_t *probe_obj_getmask(SEXP_t *obj);
+
+/// @}
