@@ -48,12 +48,10 @@
 # include <acl/libacl.h>
 #endif
 
+#include <probe/option.h>
 #include "oval_fts.h"
 #include "SEAP/generic/rbt/rbt.h"
-
-#ifndef _A
-#define _A(x) assert(x)
-#endif
+#include "common/debug_priv.h"
 
 #ifndef PATH_MAX
 #define PATH_MAX 4096
@@ -81,6 +79,7 @@
 # error "Sorry, your OS isn't supported."
 #endif
 
+oval_version_t over;
 
 static SEXP_t *gr_true   = NULL, *gr_false  = NULL, *gr_t_reg  = NULL;
 static SEXP_t *gr_t_dir  = NULL, *gr_t_lnk  = NULL, *gr_t_blk  = NULL;
@@ -119,7 +118,11 @@ static SEXP_t *ID_cache_get(int32_t id)
 	if (rbt_i32_get(g_ID_cache, id, (void *)&s_id) == 0)
 		return SEXP_ref(s_id); /* cache hit (first attempt) */
 
-	s_id = SEXP_number_newu_32(id);
+	if (oval_version_cmp(over, OVAL_VERSION(5.8)) < 0) {
+		s_id = SEXP_string_newf("%u", id);
+	} else {
+		s_id = SEXP_number_newu_32(id);
+	}
 
 	if (g_ID_cache_max == 0 || rbt_i32_size(g_ID_cache) < g_ID_cache_max) {
 		if (rbt_i32_add(g_ID_cache, id, (void *)s_id, NULL) == 0)
@@ -155,7 +158,7 @@ static void ID_cache_free(void)
 
 static SEXP_t *get_atime(struct stat *st, SEXP_t *sexp)
 {
-	return SEXP_number_newu_64_r (sexp,
+	uint64_t t = (
 #if defined(OS_FREEBSD)
 # if (__STDC_VERSION__ >= 199901L)
 		(uint64_t) st->st_atimespec.tv_sec
@@ -166,11 +169,17 @@ static SEXP_t *get_atime(struct stat *st, SEXP_t *sexp)
 		(uint64_t) st->st_atim.tv_sec
 #endif
 		);
+
+	if (oval_version_cmp(over, OVAL_VERSION(5.8)) < 0) {
+		return SEXP_string_newf_r(sexp, "%llu", (long long unsigned) t);
+	} else {
+		return SEXP_number_newu_64_r(sexp, t);
+	}
 }
 
 static SEXP_t *get_ctime(struct stat *st, SEXP_t *sexp)
 {
-	return SEXP_number_newu_64_r (sexp,
+	uint64_t t = (
 #if defined(OS_FREEBSD)
 # if (__STDC_VERSION__ >= 199901L)
 		(uint64_t) st->st_ctimespec.tv_sec
@@ -181,11 +190,17 @@ static SEXP_t *get_ctime(struct stat *st, SEXP_t *sexp)
 		(uint64_t) st->st_ctim.tv_sec
 #endif
 		);
+
+	if (oval_version_cmp(over, OVAL_VERSION(5.8)) < 0) {
+		return SEXP_string_newf_r(sexp, "%llu", (long long unsigned) t);
+	} else {
+		return SEXP_number_newu_64_r(sexp, t);
+	}
 }
 
 static SEXP_t *get_mtime(struct stat *st, SEXP_t *sexp)
 {
-	return SEXP_number_newu_64_r (sexp,
+	uint64_t t = (
 #if defined(OS_FREEBSD)
 # if (__STDC_VERSION__ >= 199901L)
 		(uint64_t) st->st_mtimespec.tv_sec
@@ -196,6 +211,12 @@ static SEXP_t *get_mtime(struct stat *st, SEXP_t *sexp)
 		(uint64_t) st->st_mtim.tv_sec
 #endif
 		);
+
+	if (oval_version_cmp(over, OVAL_VERSION(5.8)) < 0) {
+		return SEXP_string_newf_r(sexp, "%llu", (long long unsigned) t);
+	} else {
+		return SEXP_number_newu_64_r(sexp, t);
+	}
 }
 
 static SEXP_t *get_size(struct stat *st, SEXP_t *sexp)
@@ -234,11 +255,19 @@ static int file_cb (const char *p, const char *f, void *ptr)
 	}
 
         if (lstat (st_path, &st) == -1) {
-                _D("FAIL: errno=%u, %s.\n", errno, strerror (errno));
+                dI("FAIL: errno=%u, %s.\n", errno, strerror (errno));
                 return (-1);
         } else {
                 SEXP_t *se_usr_id, *se_grp_id;
                 SEXP_t  se_atime_mem, se_ctime_mem, se_mtime_mem, se_size_mem;
+		SEXP_t *se_filepath, *se_acl;
+
+		if (oval_version_cmp(over, OVAL_VERSION(5.6)) < 0
+		    || f == NULL) {
+			se_filepath = NULL;
+		} else {
+			se_filepath = SEXP_string_newf("%s", st_path);
+		}
 
 		se_usr_id = ID_cache_get(st.st_uid);
 		se_grp_id = st.st_gid != st.st_uid ? ID_cache_get(st.st_gid) : SEXP_ref(se_usr_id);
@@ -251,8 +280,18 @@ static int file_cb (const char *p, const char *f, void *ptr)
 		} else
 			SEXP_string_new_r(&gr_lastpath, p, strlen(p));
 
+#if defined(HAVE_LIBACL)
+		if (oval_version_cmp(over, OVAL_VERSION(5.7)) < 0) {
+			se_acl = NULL;
+		} else {
+			se_acl = acl_extended_file(st_path) ? gr_true : gr_false;
+		}
+#else
+		se_acl = NULL;
+#endif
+
                 item = probe_item_create(OVAL_UNIX_FILE, NULL,
-                                         "filepath", OVAL_DATATYPE_STRING, f == NULL ? NULL : st_path,
+                                         "filepath", OVAL_DATATYPE_SEXP, se_filepath,
                                          "path",     OVAL_DATATYPE_SEXP,  &gr_lastpath,
                                          "filename", OVAL_DATATYPE_STRING, f == NULL ? "" : f,
                                          "type",     OVAL_DATATYPE_SEXP, se_filetype(st.st_mode),
@@ -274,19 +313,22 @@ static int file_cb (const char *p, const char *f, void *ptr)
                                          "oread",    OVAL_DATATYPE_SEXP, MODEP(&st, S_IROTH),
                                          "owrite",   OVAL_DATATYPE_SEXP, MODEP(&st, S_IWOTH),
                                          "oexec",    OVAL_DATATYPE_SEXP, MODEP(&st, S_IXOTH),
-#if defined(HAVE_LIBACL)
-                                         "has_extended_acl", OVAL_DATATYPE_SEXP, acl_extended_file(st_path) ? gr_true : gr_false,
-#endif
+					 "has_extended_acl", OVAL_DATATYPE_SEXP, se_acl,
                                          NULL);
 
                 SEXP_free(se_grp_id);
                 SEXP_free(se_usr_id);
+		SEXP_free(se_filepath);
                 SEXP_free_r(&se_atime_mem);
                 SEXP_free_r(&se_ctime_mem);
                 SEXP_free_r(&se_mtime_mem);
                 SEXP_free_r(&se_size_mem);
 
-                probe_item_collect(args->ctx, item); /* XXX: handle ENOMEM */
+		/*
+		 * Stop collecting if we hit the memory usage limit
+		 * (return code == 2)
+		 */
+                return probe_item_collect(args->ctx, item) == 2 ? 1 : 0;
         }
 
         return (0);
@@ -296,8 +338,6 @@ static pthread_mutex_t __file_probe_mutex;
 
 void *probe_init (void)
 {
-        _LOGCALL_;
-
         /*
          * Initialize true/false global reference.
          */
@@ -339,12 +379,13 @@ void *probe_init (void)
         case 0:
                 return ((void *)&__file_probe_mutex);
         default:
-                _D("Can't initialize mutex: errno=%u, %s.\n", errno, strerror (errno));
+                dI("Can't initialize mutex: errno=%u, %s.\n", errno, strerror (errno));
         }
 #if 0
-	probe_setoption(PROBE_VARREF_HANDLING, false, "path");
-	probe_setoption(PROBE_VARREF_HANDLING, false, "filename");
+	probe_setoption(PROBEOPT_VARREF_HANDLING, false, "path");
+	probe_setoption(PROBEOPT_VARREF_HANDLING, false, "filename");
 #endif
+		probe_setoption(PROBEOPT_OFFLINE_MODE_SUPPORTED, true);
         return (NULL);
 }
 
@@ -392,6 +433,7 @@ int probe_main (probe_ctx *ctx, void *mutex)
 
         probe_in  = probe_ctx_getobject(ctx);
 
+	over = probe_obj_get_schema_version(probe_in);
         path      = probe_obj_getent (probe_in, "path",      1);
         filename  = probe_obj_getent (probe_in, "filename",  1);
         behaviors = probe_obj_getent (probe_in, "behaviors", 1);
@@ -407,19 +449,13 @@ int probe_main (probe_ctx *ctx, void *mutex)
                 return PROBE_ENOELM;
         }
 
-	/* behaviours are not important if filepath is used */
-	if(filepath != NULL && behaviors != NULL) {
-		SEXP_free (behaviors);
-		behaviors = NULL;
-	}
-
 	probe_filebehaviors_canonicalize(&behaviors);
 
         switch (pthread_mutex_lock (&__file_probe_mutex)) {
         case 0:
                 break;
         default:
-                _D("Can't lock mutex(%p): %u, %s.\n", &__file_probe_mutex, errno, strerror (errno));
+                dI("Can't lock mutex(%p): %u, %s.\n", &__file_probe_mutex, errno, strerror (errno));
 
 		SEXP_free(path);
 		SEXP_free(filename);
@@ -434,7 +470,10 @@ int probe_main (probe_ctx *ctx, void *mutex)
 
 	if ((ofts = oval_fts_open(path, filename, filepath, behaviors)) != NULL) {
 		while ((ofts_ent = oval_fts_read(ofts)) != NULL) {
-			file_cb(ofts_ent->path, ofts_ent->file, &cbargs);
+			if (file_cb(ofts_ent->path, ofts_ent->file, &cbargs) != 0) {
+				oval_ftsent_free(ofts_ent);
+				break;
+			}
 			oval_ftsent_free(ofts_ent);
 		}
 		oval_fts_close(ofts);
@@ -451,7 +490,7 @@ int probe_main (probe_ctx *ctx, void *mutex)
         case 0:
                 break;
         default:
-                _D("Can't unlock mutex(%p): %u, %s.\n", &__file_probe_mutex, errno, strerror (errno));
+                dI("Can't unlock mutex(%p): %u, %s.\n", &__file_probe_mutex, errno, strerror (errno));
 
                 return PROBE_EFATAL;
         }
