@@ -41,7 +41,9 @@
  *                string release,
  *                string version,
  *                string evr,
- *                string signature_keyid)
+ *                string signature_keyid
+ *                string extended_status OVAL >= 5.10
+ *              )
  */
 
 #ifdef HAVE_CONFIG_H
@@ -55,6 +57,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+#include <regex.h>
 
 /* RPM headers */
 #include <rpm/rpmdb.h>
@@ -80,15 +83,13 @@
 /* SEAP */
 #include <seap.h>
 #include <probe-api.h>
+#include <probe/option.h>
 #include "probe/entcmp.h"
 #include <alloc.h>
 #include <common/assume.h>
+#include "common/debug_priv.h"
 
 
-#ifndef _A
-#define _A(x) assert(x)
-#endif
-
 struct rpminfo_req {
         char *name;
         oval_operation_t op;
@@ -102,6 +103,7 @@ struct rpminfo_rep {
         char *version;
         char *evr;
         char *signature_keyid;
+	char extended_name[1024];
 };
 
 struct rpminfo_global {
@@ -110,6 +112,8 @@ struct rpminfo_global {
 };
 
 static struct rpminfo_global g_rpm;
+static const char g_keyid_regex_string[] = "Key ID [a-fA-F0-9]{16}";
+static regex_t g_keyid_regex;
 
 static void __rpminfo_rep_free (struct rpminfo_rep *ptr)
 {
@@ -127,6 +131,7 @@ static void pkgh2rep (Header h, struct rpminfo_rep *r)
         errmsg_t rpmerr;
         char *str, *sid;
         size_t len;
+	regmatch_t keyid_match[1];
 
         assume_d (h != NULL, /* void */);
         assume_d (r != NULL, /* void */);
@@ -144,6 +149,7 @@ static void pkgh2rep (Header h, struct rpminfo_rep *r)
         r->epoch   = str;
         r->release = headerFormat (h, "%{RELEASE}", &rpmerr);
         r->version = headerFormat (h, "%{VERSION}", &rpmerr);
+	snprintf(r->extended_name, 1024, "%s-%s:%s-%s.%s", r->name, r->epoch, r->version, r->release, r->arch);
 
         len = (strlen (r->epoch)   +
                strlen (r->release) +
@@ -158,8 +164,25 @@ static void pkgh2rep (Header h, struct rpminfo_rep *r)
         r->evr = str;
 
         str = headerFormat (h, "%|SIGGPG?{%{SIGGPG:pgpsig}}:{%{SIGPGP:pgpsig}}|", &rpmerr);
-        sid = strrchr (str, ' ');
-        r->signature_keyid = (sid != NULL ? strdup (sid+1) : strdup ("0"));
+
+	if (regexec(&g_keyid_regex, str, 1, keyid_match, 0) != 0) {
+		sid = NULL;
+		dW("Failed to extract the Key ID value: regex=\"%s\", string=\"%s\"\n",
+		   g_keyid_regex_string, str);
+	} else {
+		size_t keyid_start, keyid_length;
+
+		if (keyid_match[0].rm_so < 0 || keyid_match[0].rm_eo < 0)
+			sid = NULL;
+		else {
+			keyid_start = keyid_match[0].rm_so + strlen("Key ID ");
+			keyid_length = keyid_match[0].rm_eo - keyid_start;
+			sid = str + keyid_start;
+			sid[keyid_length] = '\0';
+		}
+	}
+
+        r->signature_keyid = strdup(sid != NULL ? sid : "0");
         oscap_free (str);
 }
 
@@ -266,14 +289,20 @@ ret:
 void *probe_init (void)
 {
         if (rpmReadConfigFiles ((const char *)NULL, (const char *)NULL) != 0) {
-                _D("rpmReadConfigFiles failed: %u, %s.\n", errno, strerror (errno));
+                dI("rpmReadConfigFiles failed: %u, %s.\n", errno, strerror (errno));
                 return (NULL);
         }
 
         g_rpm.rpmts = rpmtsCreate();
-
         pthread_mutex_init (&(g_rpm.mutex), NULL);
 
+	if (regcomp(&g_keyid_regex, g_keyid_regex_string, REG_EXTENDED) != 0) {
+		dE("regcomp(%s) failed.\n");
+		return NULL;
+	}
+
+	probe_setoption(PROBEOPT_OFFLINE_MODE_SUPPORTED, true);
+
         return ((void *)&g_rpm);
 }
 
@@ -287,19 +316,90 @@ void probe_fini (void *ptr)
         rpmFreeMacros(NULL);
         rpmlogClose();
         pthread_mutex_destroy (&(r->mutex));
+	regfree(&g_keyid_regex);
 
         return;
 }
 
+static int collect_rpm_files(SEXP_t *item, const struct rpminfo_rep *rep) {
+	SEXP_t *value;
+	rpmdbMatchIterator ts;
+	Header pkgh;
+	rpmfi fi;
+	rpmTag tag[2] = { RPMTAG_BASENAMES, RPMTAG_DIRNAMES };
+	int i, ret = 0;
+
+	ts = rpmtsInitIterator(g_rpm.rpmts, RPMDBI_PACKAGES, NULL, 0);
+	if (ts == NULL) {
+		return -1;
+	}
+
+	if (rpmdbSetIteratorRE(ts, RPMTAG_NAME, RPMMIRE_STRCMP, rep->name) != 0) {
+		ret = -1;
+		goto cleanup;
+	}
+	if (rpmdbSetIteratorRE(ts, RPMTAG_EPOCH, RPMMIRE_STRCMP, rep->epoch) != 0) {
+		ret = -1;
+		goto cleanup;
+	}
+	if (rpmdbSetIteratorRE(ts, RPMTAG_VERSION, RPMMIRE_STRCMP, rep->version) != 0) {
+		ret = -1;
+		goto cleanup;
+	}
+	if (rpmdbSetIteratorRE(ts, RPMTAG_RELEASE, RPMMIRE_STRCMP, rep->release) != 0) {
+		ret = -1;
+		goto cleanup;
+	}
+	if (rpmdbSetIteratorRE(ts, RPMTAG_ARCH, RPMMIRE_STRCMP, rep->arch) != 0) {
+		ret = -1;
+		goto cleanup;
+	}
+
+	while ((pkgh = rpmdbNextIterator(ts)) != NULL) {
+		/*
+		 * Inspect package files & directories
+		 */
+		for (i = 0; i < 2; ++i) {
+			fi = rpmfiNew(g_rpm.rpmts, pkgh, tag[i], 1);
+
+			while (rpmfiNext(fi) != -1) {
+				const char *filepath;
+				filepath = rpmfiFN(fi);
+				value = probe_entval_from_cstr(
+						OVAL_DATATYPE_STRING,
+						filepath,
+						strlen(filepath)
+						);
+				if (value != NULL) {
+					probe_item_ent_add(item, "filepath", NULL, value);
+					SEXP_free(value);
+				}
+			}
+			rpmfiFree(fi);
+		}
+
+	}
+cleanup:
+	ts = rpmdbFreeIterator(ts);
+	return ret;
+}
+
 int probe_main (probe_ctx *ctx, void *arg)
 {
-        SEXP_t *val, *item, *ent;
+	SEXP_t *val, *item, *ent, *probe_in;
+	oval_version_t over;
 	int rpmret, i;
 
         struct rpminfo_req request_st;
         struct rpminfo_rep *reply_st;
 
-        ent = probe_obj_getent (probe_ctx_getobject(ctx), "name", 1);
+	probe_in = probe_ctx_getobject(ctx);
+	if (probe_in == NULL)
+		return PROBE_ENOOBJ;
+
+	over = probe_obj_get_schema_version(probe_in);
+
+        ent = probe_obj_getent (probe_in, "name", 1);
 
         if (ent == NULL) {
                 return (PROBE_ENOENT);
@@ -308,7 +408,7 @@ int probe_main (probe_ctx *ctx, void *arg)
         val = probe_ent_getval (ent);
 
         if (val == NULL) {
-                _D("%s: no value\n", "name");
+                dI("%s: no value\n", "name");
                 SEXP_free (ent);
                 return (PROBE_ENOVAL);
         }
@@ -342,11 +442,11 @@ int probe_main (probe_ctx *ctx, void *arg)
 		SEXP_free (ent);
                 switch (errno) {
                 case EINVAL:
-                        _D("%s: invalid value type\n", "name");
+                        dI("%s: invalid value type\n", "name");
 			return PROBE_EINVAL;
                         break;
                 case EFAULT:
-                        _D("%s: element not found\n", "name");
+                        dI("%s: element not found\n", "name");
 			return PROBE_ENOELM;
                         break;
 		default:
@@ -359,10 +459,10 @@ int probe_main (probe_ctx *ctx, void *arg)
         /* get info from RPM db */
         switch (rpmret = get_rpminfo (&request_st, &reply_st)) {
         case 0: /* Not found */
-                _D("Package \"%s\" not found.\n", request_st.name);
+                dI("Package \"%s\" not found.\n", request_st.name);
                 break;
         case -1: /* Error */
-                _D("get_rpminfo failed\n");
+                dI("get_rpminfo failed\n");
 
                 item = probe_item_create(OVAL_LINUX_RPM_INFO, NULL,
                                          "name", OVAL_DATATYPE_STRING, request_st.name,
@@ -395,10 +495,42 @@ int probe_main (probe_ctx *ctx, void *arg)
                                                          "signature_keyid", OVAL_DATATYPE_STRING, reply_st[i].signature_keyid,
                                                          NULL);
 
-                                probe_item_collect(ctx, item);
+				/* OVAL 5.10 added extended_name and filepaths behavior */
+				if (oval_version_cmp(over, OVAL_VERSION(5.10)) >= 0) {
+					SEXP_t *value, *bh_value;
+					value = probe_entval_from_cstr(
+							OVAL_DATATYPE_STRING,
+							reply_st[i].extended_name,
+							strlen(reply_st[i].extended_name)
+					);
+					probe_item_ent_add(item, "extended_name", NULL, value);
+					SEXP_free(value);
+
+					/*
+					 * Parse behaviors
+					 */
+					value = probe_obj_getent(probe_in, "behaviors", 1);
+					if (value != NULL) {
+						bh_value = probe_ent_getattrval(value, "filepaths");
+						if (bh_value != NULL) {
+							if (SEXP_strcmp(bh_value, "true") == 0) {
+								/* collect package files */
+								collect_rpm_files(item, &reply_st[i]);
+
+							}
+							SEXP_free(bh_value);
+						}
+						SEXP_free(value);
+					}
+
+				}
+
 
 				SEXP_free(name);
                                 __rpminfo_rep_free (&(reply_st[i]));
+
+				if (probe_item_collect(ctx, item))
+					return 1;
                         }
 
                         oscap_free (reply_st);
