@@ -33,6 +33,7 @@
 
 #include <probe-api.h>
 #include <probe/entcmp.h>
+#include <probe/option.h>
 #include <errno.h>
 #include <unistd.h>
 #include <stdlib.h>
@@ -54,10 +55,6 @@
 #include <netdb.h>
 #include "../SEAP/generic/rbt/rbt.h"
 
-#define dI(...) oscap_dlprintf(DBG_I, __VA_ARGS__)
-#define dW(...) oscap_dlprintf(DBG_W, __VA_ARGS__)
-#define dE(...) oscap_dlprintf(DBG_E, __VA_ARGS__)
-
 #define PATH_SEPARATOR '/'
 
 /*
@@ -458,8 +455,11 @@ void xiconf_free(xiconf_t *xiconf)
         rbt_str_free_cb(xiconf->stree, xiconf_stree_free_cb);
         rbt_str_free_cb(xiconf->ttree, xiconf_ttree_free_cb);
 
-	oscap_free(xiconf->defaults->name);
-	oscap_free(xiconf->defaults);
+        if (xiconf->defaults != NULL) {
+	        oscap_free(xiconf->defaults->name);
+	        oscap_free(xiconf->defaults);
+        }
+
 	oscap_free(xiconf);
 
 	return;
@@ -557,7 +557,7 @@ static int xiconf_add_cfile(xiconf_t *xiconf, const char *path, int depth)
 	}
 
 	xifile->depth = depth;
-	xiconf->cfile = oscap_realloc(xiconf->cfile, sizeof(xiconf_file_t) * ++xiconf->count);
+	xiconf->cfile = oscap_realloc(xiconf->cfile, sizeof(xiconf_file_t *) * ++xiconf->count);
 	xiconf->cfile[xiconf->count - 1] = xifile;
 
 	dI("Added new file to the cfile queue: %s; fi=%zu\n", path, xiconf->count - 1);
@@ -642,6 +642,7 @@ xiconf_t *xiconf_parse(const char *path, unsigned int max_depth)
 
 					if (bufidx >= l_size) {
 						tmpbuf_free(buffer);
+						xiconf_free(xiconf);
 						return (NULL);
 					}
 
@@ -650,6 +651,7 @@ xiconf_t *xiconf_parse(const char *path, unsigned int max_depth)
 
 					if (bufidx >= l_size) {
 						tmpbuf_free(buffer);
+						xiconf_free(xiconf);
 						return (NULL);
 					}
 
@@ -657,6 +659,7 @@ xiconf_t *xiconf_parse(const char *path, unsigned int max_depth)
 
 					if (xiconf_parse_section (xiconf, xifile, XICONF_SECTION_SERVICE, buffer + bufidx) != 0) {
 						tmpbuf_free(buffer);
+						xiconf_free(xiconf);
 						return (NULL);
 					}
 				}
@@ -665,6 +668,7 @@ xiconf_t *xiconf_parse(const char *path, unsigned int max_depth)
 				if (strcmp("efaults", buffer + bufidx + 1) == 0) {
 					if (xiconf_parse_section (xiconf, xifile, XICONF_SECTION_DEFAULTS, buffer + bufidx) != 0) {
 						tmpbuf_free(buffer);
+						xiconf_free(xiconf);
 						return (NULL);
 					}
 				}
@@ -758,7 +762,6 @@ xiconf_t *xiconf_parse(const char *path, unsigned int max_depth)
 					for (;;) {
 						if (readdir_r (dirfp, &dent, &dentp) != 0) {
 							dW("Can't read directory: %s; %d, %s.\n", inclarg, errno, strerror (errno));
-							closedir (dirfp);
 							break;
 						}
 
@@ -1132,7 +1135,7 @@ finish_section:
 
 			st = oscap_talloc (xiconf_strans_t);
 			st->cnt = 1;
-			st->srv = oscap_alloc (sizeof (xiconf_service_t) * 1);
+			st->srv = oscap_alloc (sizeof (xiconf_service_t *));
 			st->srv[0] = scur;
 
 			if (rbt_str_add (xiconf->ttree, strdup(st_key), st) != 0) {
@@ -1144,7 +1147,7 @@ finish_section:
 			dI("adding new strans record to an exiting one: k=%s, cnt=%u+1\n",
 			   st_key, st->cnt);
 
-			st->srv = oscap_realloc(st->srv, sizeof (xiconf_service_t) * ++(st->cnt));
+			st->srv = oscap_realloc(st->srv, sizeof (xiconf_service_t *) * ++(st->cnt));
 			st->srv[st->cnt - 1] = scur;
 		}
 
@@ -1523,6 +1526,7 @@ int op_assign_enabled(void *var, char *val)
 #ifndef XINETD_TEST
 void *probe_init(void)
 {
+	probe_setoption(PROBEOPT_OFFLINE_MODE_SUPPORTED, true);
 	return xiconf_parse(XINETD_CONFPATH, XINETD_CONFDEPTH);
 }
 
@@ -1587,7 +1591,7 @@ int probe_main(probe_ctx *ctx, void *arg)
 
 	SEXP_free (eval);
 
-	_D("Updating xinetd configuration cache");
+	dI("Updating xinetd configuration cache");
 
 	if (xiconf_update(arg) != 0) {
 		err = PROBE_EUNKNOWN;
