@@ -74,13 +74,6 @@ int oval_variable_model_export (struct oval_variable_model *, const char *file);
  * @{
  */
 void oval_variable_model_set_generator(struct oval_variable_model *model, struct oval_generator *generator);
-/**
- * Lock the variable_model instance.
- * The state of a locked instance cannot be changed.
- * This operation has no effect if the model is already locked.
- * @memberof oval_variable_model
- */ 
-void oval_variable_model_lock(struct oval_variable_model *variable_model);
 /**
  * Get the values bound to a specified external variable.
  * If the varid does not resolve to a managed external variable, this method returns NULL.
@@ -133,29 +126,41 @@ const char *oval_variable_model_get_comment (struct oval_variable_model *, char
  * @param varid the identifier of the required oval_variable.
  * @memberof oval_variable_model
  */ 
-struct oval_string_iterator *oval_variable_model_get_values (struct oval_variable_model *, char *);
-/**
- * Return <b>true</b> if the variable_model instance is locked.
- * The state of a locked instance cannot be changed.
- * @memberof oval_variable_model
- */ 
-bool oval_variable_model_is_locked(struct oval_variable_model *variable_model);
+struct oval_value_iterator *oval_variable_model_get_values(struct oval_variable_model *, char *);
 /** @} */
 
+/**
+ * @name Iterators
+ * @{
+ */
+/**
+ * @struct oval_variable_model_iterator
+ * @see oval_definition_model_get_bound_variable_models
+ */
+struct oval_variable_model_iterator;
+/**
+ * Returns <b>true</b> if iterator not exhausted.
+ * @memberof oval_variable_model_iterator
+ */
+bool oval_variable_model_iterator_has_more(struct oval_variable_model_iterator *);
+/**
+ * Returns next instance of @ref oval_variable_model.
+ * @memberof oval_variable_model_iterator
+ */
+struct oval_variable_model *oval_variable_model_iterator_next(struct oval_variable_model_iterator *);
+/**
+ * Free iterator.
+ * @memberof oval_variable_model_iterator
+ */
+void oval_variable_model_iterator_free(struct oval_variable_model_iterator *);
+/** @} */
 
 /**
  * @name Evaluators
  * @{
  */
-/**
- * Return <b>true</b> if the variable_model instance is valid
- * @memberof oval_variable_model
- */ 
-bool oval_variable_model_is_valid(struct oval_variable_model *variable_model);
 /** @} */
 
-
-
 /**
  * @}END OVALVAR
  * @}END OVAL
