@@ -57,7 +57,7 @@ const struct oscap_string_map XCCDF_LEVEL_MAP[] = {
 	{XCCDF_LOW, "low"},
 	{XCCDF_MEDIUM, "medium"},
 	{XCCDF_HIGH, "high"},
-	{XCCDF_UNKNOWN, NULL}
+	{XCCDF_LEVEL_NOT_DEFINED, NULL}
 };
 
 static const struct oscap_string_map XCCDF_STATUS_MAP[] = {
@@ -115,12 +115,15 @@ struct xccdf_item *xccdf_item_new(xccdf_type_t type, struct xccdf_item *parent)
 	item->item.question = oscap_list_new();
 	item->item.rationale = oscap_list_new();
 	item->item.statuses = oscap_list_new();
+	item->item.dc_statuses = oscap_list_new();
 	item->item.platforms = oscap_list_new();
 	item->item.warnings = oscap_list_new();
 	item->item.references = oscap_list_new();
 	item->item.weight = 1.0;
 	item->item.flags.selected = true;
 	item->item.parent = parent;
+	item->item.metadata = (struct oscap_list*)oscap_stringlist_new();
+
 	return item;
 }
 
@@ -128,13 +131,11 @@ struct xccdf_item *xccdf_item_clone(const struct xccdf_item *old_item)
 {
 	struct xccdf_item *new_item = oscap_calloc(1, sizeof(struct xccdf_item));
 
-	//new_item->item = *(xccdf_item_base_clone(&(old_item->item)));
     xccdf_item_base_clone(&new_item->item, &(old_item->item));
 	new_item->type = old_item->type;
 
 	switch (new_item->type) {
 	case XCCDF_BENCHMARK:
-		 //new_item->sub.benchmark = *(xccdf_benchmark_item_clone(&(old_item->sub.benchmark), new_item));
         xccdf_benchmark_item_clone(new_item, XBENCHMARK(old_item));
 		break;
 	case XCCDF_RULE:
@@ -165,11 +166,13 @@ void xccdf_item_base_clone(struct xccdf_item_base *new_base, const struct xccdf_
 {
 	new_base->id = oscap_strdup(old_base->id);
 	new_base->cluster_id = oscap_strdup(old_base->cluster_id);
-	new_base->version = oscap_strdup(old_base->version);
-	new_base->version_update = oscap_strdup(old_base->version_update);
 	new_base->extends = oscap_strdup(old_base->extends);
 
-	new_base->version_time = old_base->version_time;
+	new_base->version = oscap_strdup(old_base->version);
+	new_base->version_update = oscap_strdup(old_base->version_update);
+	new_base->version_time = oscap_strdup(old_base->version_time);
+
+
 	new_base->weight = old_base->weight;
 	new_base->parent = NULL;
 
@@ -180,12 +183,15 @@ void xccdf_item_base_clone(struct xccdf_item_base *new_base, const struct xccdf_
 
 	new_base->warnings = oscap_list_clone(old_base->warnings, (oscap_clone_func) xccdf_warning_clone);
 	new_base->statuses = oscap_list_clone(old_base->statuses, (oscap_clone_func) xccdf_status_clone);
+	new_base->dc_statuses = oscap_list_clone(old_base->dc_statuses, (oscap_clone_func) oscap_reference_clone);
 	new_base->references = oscap_list_clone(old_base->references, (oscap_clone_func) oscap_reference_clone);
 	new_base->platforms = oscap_list_clone(old_base->platforms, (oscap_clone_func) oscap_strdup);
 
 	/* Handling flags */
 	new_base->flags = old_base->flags;
 	new_base->defined_flags = old_base->defined_flags;
+
+	new_base->metadata = (struct oscap_list*)oscap_stringlist_clone((struct oscap_stringlist*)(old_base->metadata));
 }
 
 /* Performs a deep copy of xccdf_status and returns a pointer to that copy */
@@ -197,7 +203,6 @@ struct xccdf_status *xccdf_status_clone(const struct xccdf_status *old_status)
 	return new_status;
 }
 
-
 /* Performs a deep copy of xccdf_warning and returns a pointer to that copy */
 struct xccdf_warning *xccdf_warning_clone(const struct xccdf_warning *old_warning)
 {
@@ -212,6 +217,7 @@ void xccdf_item_release(struct xccdf_item *item)
 {
 	if (item) {
 		oscap_list_free(item->item.statuses, (oscap_destruct_func) xccdf_status_free);
+		oscap_list_free(item->item.dc_statuses, (oscap_destruct_func) oscap_reference_free);
 		oscap_list_free(item->item.platforms, oscap_free);
 		oscap_list_free(item->item.title, (oscap_destruct_func) oscap_text_free);
 		oscap_list_free(item->item.description, (oscap_destruct_func) oscap_text_free);
@@ -222,9 +228,12 @@ void xccdf_item_release(struct xccdf_item *item)
 		if (item->type != XCCDF_BENCHMARK) xccdf_benchmark_unregister_item(item);
 		oscap_free(item->item.id);
 		oscap_free(item->item.cluster_id);
+		oscap_free(item->item.version_time);
 		oscap_free(item->item.version_update);
 		oscap_free(item->item.version);
 		oscap_free(item->item.extends);
+		oscap_stringlist_free((struct oscap_stringlist*)(item->item.metadata));
+
 		oscap_free(item);
 	}
 }
@@ -307,7 +316,12 @@ void xccdf_item_print(struct xccdf_item *item, int depth)
 		printf("platforms ");
 		oscap_list_dump(item->item.platforms, xccdf_cstring_dump, depth + 1);
 		xccdf_print_depth(depth);
-		printf("status (cur = %d)", xccdf_item_get_current_status(item));
+		printf("status (cur = ");
+		const struct xccdf_status *status = xccdf_item_get_current_status(item);
+		if (status != NULL)
+			printf("%d)", xccdf_status_get_status(status));
+		else
+			printf("(NULL))");
 		oscap_list_dump(item->item.statuses, xccdf_status_dump, depth + 1);
 	}
 }
@@ -320,7 +334,10 @@ void xccdf_texts_to_dom(struct oscap_text_iterator *texts, xmlNode *parent, cons
 
 xmlNode *xccdf_item_to_dom(struct xccdf_item *item, xmlDoc *doc, xmlNode *parent)
 {
-	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent, XCCDF_BASE_NAMESPACE);
+	const struct xccdf_version_info* version_info = xccdf_item_get_schema_version(item);
+
+	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent,
+			(const xmlChar*)xccdf_version_info_get_namespace_uri(version_info));
 	xmlNode *item_node = NULL;
 	if (parent == NULL)
 		item_node = xmlNewNode(NULL, BAD_CAST "Item");
@@ -335,8 +352,12 @@ xmlNode *xccdf_item_to_dom(struct xccdf_item *item, xmlDoc *doc, xmlNode *parent
 	if (cluster_id)
 		xmlNewProp(item_node, BAD_CAST "cluster-id", BAD_CAST cluster_id);
 
-	if (xccdf_item_get_hidden(item))
-		xmlNewProp(item_node, BAD_CAST "hidden", BAD_CAST "true");
+	if (item->item.defined_flags.hidden) {
+		if (xccdf_item_get_hidden(item))
+			xmlNewProp(item_node, BAD_CAST "hidden", BAD_CAST "true");
+		else
+			xmlNewProp(item_node, BAD_CAST "hidden", BAD_CAST "false");
+	}
 
 	if (xccdf_item_get_prohibit_changes(item))
 		xmlNewProp(item_node, BAD_CAST "prohibitChanges", BAD_CAST "true");
@@ -347,24 +368,30 @@ xmlNode *xccdf_item_to_dom(struct xccdf_item *item, xmlDoc *doc, xmlNode *parent
 	struct xccdf_status_iterator *statuses = xccdf_item_get_statuses(item);
 	while (xccdf_status_iterator_has_more(statuses)) {
 		struct xccdf_status *status = xccdf_status_iterator_next(statuses);
-		xccdf_status_to_dom(status, doc, item_node);
+		xccdf_status_to_dom(status, doc, item_node, version_info);
 	}
 	xccdf_status_iterator_free(statuses);
 
-    time_t vt = xccdf_item_get_version_time(item);
+	struct oscap_reference_iterator *dc_statuses = xccdf_item_get_dc_statuses(item);
+	while (oscap_reference_iterator_has_more(dc_statuses)) {
+		struct oscap_reference *ref = oscap_reference_iterator_next(dc_statuses);
+		oscap_reference_to_dom(ref, item_node, doc, "dc-status");
+	}
+	oscap_reference_iterator_free(dc_statuses);
+
+	/* version and attributes */
 	const char *version = xccdf_item_get_version(item);
-	if (xccdf_item_get_type(item) != XCCDF_BENCHMARK && (version || vt || xccdf_item_get_version_update(item))) {
+	if ((xccdf_item_get_type(item) != XCCDF_BENCHMARK) && version) {
 		xmlNode* version_node = xmlNewTextChild(item_node, ns_xccdf, BAD_CAST "version", BAD_CAST version);
-        if (xccdf_item_get_version_update(item))
-		    xmlNewProp(version_node, BAD_CAST "update", BAD_CAST xccdf_item_get_version_update(item));
-        if (vt) {
-            struct tm *lt = localtime(&vt);
-            char timestamp[] = "yyyy-mm-ddThh:mm:ss";
-            snprintf(timestamp, sizeof(timestamp), "%4d-%02d-%02dT%02d:%02d:%02d",
-                 1900 + lt->tm_year, 1 + lt->tm_mon, lt->tm_mday, lt->tm_hour, lt->tm_min, lt->tm_sec);
-            xmlNewProp(version_node, BAD_CAST "time", BAD_CAST timestamp);
-        }
-    }
+
+		const char *version_update = xccdf_item_get_version_update(item);
+		if (version_update)
+			xmlNewProp(version_node, BAD_CAST "update", BAD_CAST version_update);
+
+		const char *version_time = xccdf_item_get_version_time(item);
+		if (version_time)
+			xmlNewProp(version_node, BAD_CAST "time", BAD_CAST version_time);
+	}
 
 	/* Handle generic item child nodes */
 	xccdf_texts_to_dom(xccdf_item_get_title(item), item_node, "title");
@@ -406,6 +433,16 @@ xmlNode *xccdf_item_to_dom(struct xccdf_item *item, xmlDoc *doc, xmlNode *parent
 	}
     oscap_reference_iterator_free(references);
 
+	if (xccdf_item_get_type(item)!=XCCDF_BENCHMARK) {
+		struct oscap_string_iterator* metadata = xccdf_item_get_metadata(item);
+		while (oscap_string_iterator_has_more(metadata))
+		{
+			const char* meta = oscap_string_iterator_next(metadata);
+			oscap_xmlstr_to_dom(item_node, "metadata", meta);
+		}
+		oscap_string_iterator_free(metadata);
+    }
+
 	/* Handle type specific attributes and children */
 	switch (xccdf_item_get_type(item)) {
 		case XCCDF_RULE:
@@ -464,9 +501,10 @@ xmlNode *xccdf_warning_to_dom(struct xccdf_warning *warning, xmlDoc *doc, xmlNod
 	return warning_node;
 }
 
-xmlNode *xccdf_status_to_dom(struct xccdf_status *status, xmlDoc *doc, xmlNode *parent)
+xmlNode *xccdf_status_to_dom(struct xccdf_status *status, xmlDoc *doc, xmlNode *parent, const struct xccdf_version_info* version_info)
 {
-	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent, XCCDF_BASE_NAMESPACE);
+	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent,
+				(const xmlChar*)xccdf_version_info_get_namespace_uri(version_info));
 
 	xmlNode *status_node = NULL;
 	xccdf_status_type_t type = xccdf_status_get_status(status);
@@ -475,10 +513,12 @@ xmlNode *xccdf_status_to_dom(struct xccdf_status *status, xmlDoc *doc, xmlNode *
 							BAD_CAST XCCDF_STATUS_MAP[type - 1].string);
 
 	time_t date_time = xccdf_status_get_date(status);
-	struct tm *date = localtime(&date_time);
-	char date_str[] = "YYYY-DD-MM";
-	snprintf(date_str, sizeof(date_str), "%04d-%02d-%02d", date->tm_year + 1900, date->tm_mon + 1, date->tm_mday);
-	xmlNewProp(status_node, BAD_CAST "date", BAD_CAST date_str);
+	if (date_time) {
+		struct tm *date = localtime(&date_time);
+		char date_str[] = "YYYY-DD-MM";
+		snprintf(date_str, sizeof(date_str), "%04d-%02d-%02d", date->tm_year + 1900, date->tm_mon + 1, date->tm_mday);
+		xmlNewProp(status_node, BAD_CAST "date", BAD_CAST date_str);
+	}
 
 	return status_node;
 }
@@ -495,15 +535,15 @@ xmlNode *xccdf_fixtext_to_dom(struct xccdf_fixtext *fixtext, xmlDoc *doc, xmlNod
 	    xmlNewProp(fixtext_node, BAD_CAST "fixref", BAD_CAST fixref);
 
 	xccdf_level_t complexity = xccdf_fixtext_get_complexity(fixtext);
-        if (complexity != 0)
+        if (complexity != XCCDF_LEVEL_NOT_DEFINED)
 	        xmlNewProp(fixtext_node, BAD_CAST "complexity", BAD_CAST XCCDF_LEVEL_MAP[complexity-1].string);
 
 	xccdf_level_t disruption = xccdf_fixtext_get_disruption(fixtext);
-        if (disruption != 0)
+        if (disruption != XCCDF_LEVEL_NOT_DEFINED)
 	        xmlNewProp(fixtext_node, BAD_CAST "disruption", BAD_CAST XCCDF_LEVEL_MAP[disruption-1].string);
 
 	xccdf_strategy_t strategy = xccdf_fixtext_get_strategy(fixtext);
-        if (strategy != 0)
+        if (strategy != XCCDF_STRATEGY_UNKNOWN)
 	        xmlNewProp(fixtext_node, BAD_CAST "strategy", BAD_CAST XCCDF_STRATEGY_MAP[strategy].string);
 
 	return fixtext_node;
@@ -526,15 +566,15 @@ xmlNode *xccdf_fix_to_dom(struct xccdf_fix *fix, xmlDoc *doc, xmlNode *parent)
 		xmlNewProp(fix_node, BAD_CAST "reboot", BAD_CAST "true");
 
 	xccdf_level_t complexity = xccdf_fix_get_complexity(fix);
-        if (complexity != 0)
+        if (complexity != XCCDF_LEVEL_NOT_DEFINED)
 	    xmlNewProp(fix_node, BAD_CAST "complexity", BAD_CAST XCCDF_LEVEL_MAP[complexity-1].string);
 
 	xccdf_level_t disruption = xccdf_fix_get_disruption(fix);
-        if (disruption != 0)
+        if (disruption != XCCDF_LEVEL_NOT_DEFINED)
 	        xmlNewProp(fix_node, BAD_CAST "disruption", BAD_CAST XCCDF_LEVEL_MAP[disruption-1].string);
 
 	xccdf_strategy_t strategy = xccdf_fix_get_strategy(fix);
-        if (strategy != 0)
+        if (strategy != XCCDF_STRATEGY_UNKNOWN)
 	        xmlNewProp(fix_node, BAD_CAST "strategy", BAD_CAST XCCDF_STRATEGY_MAP[strategy-1].string);
 
         // Sub element is used to store XCCDF value substitutions, not a content
@@ -547,9 +587,11 @@ xmlNode *xccdf_fix_to_dom(struct xccdf_fix *fix, xmlDoc *doc, xmlNode *parent)
 	return fix_node;
 }
 
-xmlNode *xccdf_ident_to_dom(struct xccdf_ident *ident, xmlDoc *doc, xmlNode *parent)
+xmlNode *xccdf_ident_to_dom(struct xccdf_ident *ident, xmlDoc *doc, xmlNode *parent, const struct xccdf_version_info* version_info)
 {
-	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent, XCCDF_BASE_NAMESPACE);
+	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent,
+			(const xmlChar*)xccdf_version_info_get_namespace_uri(version_info));
+
 	const char *id = xccdf_ident_get_id(ident);
 	xmlNode *ident_node = xmlNewTextChild(parent, ns_xccdf, BAD_CAST "ident", BAD_CAST id);
 
@@ -559,31 +601,50 @@ xmlNode *xccdf_ident_to_dom(struct xccdf_ident *ident, xmlDoc *doc, xmlNode *par
 	return ident_node;
 }
 
-xmlNode *xccdf_check_to_dom(struct xccdf_check *check, xmlDoc *doc, xmlNode *parent)
+static const struct oscap_string_map XCCDF_BOOLOP_MAP[] = {
+	{XCCDF_OPERATOR_AND, "AND"},
+	{XCCDF_OPERATOR_OR, "OR"},
+	{0, NULL}
+};
+
+xmlNode *xccdf_check_to_dom(struct xccdf_check *check, xmlDoc *doc, xmlNode *parent, const struct xccdf_version_info* version_info)
 {
-	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent, XCCDF_BASE_NAMESPACE);
+	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent,
+			(const xmlChar*)xccdf_version_info_get_namespace_uri(version_info));
+
 	xmlNode *check_node = NULL;
 	if (xccdf_check_get_complex(check))
 		check_node = xmlNewTextChild(parent, ns_xccdf, BAD_CAST "complex-check", NULL);
-	else
+	else {
 		check_node = xmlNewTextChild(parent, ns_xccdf, BAD_CAST "check", NULL);
+		const char *sys = xccdf_check_get_system(check);
+		xmlNewProp(check_node, BAD_CAST "system", BAD_CAST sys);
+	}
+
 
 	const char *id = xccdf_check_get_id(check);
 	if (id)
 		xmlNewProp(check_node, BAD_CAST "id", BAD_CAST id);
 
-	const char *sys = xccdf_check_get_system(check);
-	xmlNewProp(check_node, BAD_CAST "system", BAD_CAST sys);
-
 	const char *selector = xccdf_check_get_selector(check);
 	if (selector)
 		xmlNewProp(check_node, BAD_CAST "selector", BAD_CAST selector);
+	if (!xccdf_check_get_complex(check) && (check->flags.def_multicheck || xccdf_check_get_multicheck(check))) {
+		xmlNewProp(check_node, BAD_CAST "multi-check",
+			BAD_CAST (xccdf_check_get_multicheck(check) ? "true" : "false"));
+	}
+	if (check->flags.def_negate || xccdf_check_get_negate(check))
+		xmlNewProp(check_node, BAD_CAST "negate",
+			BAD_CAST (xccdf_check_get_negate(check) ? "true" : "false"));
+	if (xccdf_check_get_complex(check))
+		xmlNewProp(check_node, BAD_CAST "operator",
+			BAD_CAST oscap_enum_to_string(XCCDF_BOOLOP_MAP, xccdf_check_get_oper(check)));
 
 	/* Handle complex checks */
 	struct xccdf_check_iterator *checks = xccdf_check_get_children(check);
 	while (xccdf_check_iterator_has_more(checks)) {
 		struct xccdf_check *new_check = xccdf_check_iterator_next(checks);
-		xccdf_check_to_dom(new_check, doc, check_node);
+		xccdf_check_to_dom(new_check, doc, check_node, version_info);
 	}
 	xccdf_check_iterator_free(checks);
 
@@ -591,18 +652,19 @@ xmlNode *xccdf_check_to_dom(struct xccdf_check *check, xmlDoc *doc, xmlNode *par
 	while (xccdf_check_import_iterator_has_more(imports)) {
 		struct xccdf_check_import *import = xccdf_check_import_iterator_next(imports);
 		const char *name = xccdf_check_import_get_name(import);
+		const char *xpath = xccdf_check_import_get_xpath(import);
 		const char *content = xccdf_check_import_get_content(import);
-		xmlNode *import_node = xmlNewTextChild(check_node, ns_xccdf, BAD_CAST "check-import", BAD_CAST content);
+		xmlNode *import_node = xmlNewChild(check_node, ns_xccdf, BAD_CAST "check-import", BAD_CAST content);
 		xmlNewProp(import_node, BAD_CAST "import-name", BAD_CAST name);
+		if (xpath)
+			xmlNewProp(import_node, BAD_CAST "import-xpath", BAD_CAST xpath);
 	}
 	xccdf_check_import_iterator_free(imports);
 
 	struct xccdf_check_export_iterator *exports = xccdf_check_get_exports(check);
 	while (xccdf_check_export_iterator_has_more(exports)) {
 		struct xccdf_check_export *export = xccdf_check_export_iterator_next(exports);
-		// This function seems like it should be in OpenSCAP, but isn't according to the docs
-		//const char *name = xccdf_check_export_get_name(export);
-		const char *name = export->name;
+		const char *name = xccdf_check_export_get_name(export);
 		const char *value= xccdf_check_export_get_value(export);
 		xmlNode *export_node = xmlNewTextChild(check_node, ns_xccdf, BAD_CAST "check-export", NULL);
 		xmlNewProp(export_node, BAD_CAST "export-name", BAD_CAST name);
@@ -610,23 +672,25 @@ xmlNode *xccdf_check_to_dom(struct xccdf_check *check, xmlDoc *doc, xmlNode *par
 	}
 	xccdf_check_export_iterator_free(exports);
 
-	const char *content = xccdf_check_get_content(check);
-	if (content)
-		xmlNewTextChild(check_node, ns_xccdf, BAD_CAST "check-content", BAD_CAST content);
-
 	struct xccdf_check_content_ref_iterator *refs = xccdf_check_get_content_refs(check);
 	while (xccdf_check_content_ref_iterator_has_more(refs)) {
 		struct xccdf_check_content_ref *ref = xccdf_check_content_ref_iterator_next(refs);
 		xmlNode *ref_node = xmlNewTextChild(check_node, ns_xccdf, BAD_CAST "check-content-ref", NULL);
 
 		const char *name = xccdf_check_content_ref_get_name(ref);
-		xmlNewProp(ref_node, BAD_CAST "name", BAD_CAST name);
+		if (name != NULL)
+			xmlNewProp(ref_node, BAD_CAST "name", BAD_CAST name);
 
 		const char *href = xccdf_check_content_ref_get_href(ref);
 		xmlNewProp(ref_node, BAD_CAST "href", BAD_CAST href);
 	}
 	xccdf_check_content_ref_iterator_free(refs);
 
+	const char *content = xccdf_check_get_content(check);
+	if (content) {
+		oscap_xmlstr_to_dom(check_node, "check-content", content);
+	}
+
 	return check_node;
 }
 
@@ -658,7 +722,7 @@ bool xccdf_item_process_attributes(struct xccdf_item *item, xmlTextReaderPtr rea
 	if (item->item.id) {
 		struct xccdf_item *bench = XITEM(xccdf_item_get_benchmark_internal(item));
 		if (bench != NULL && bench != item)
-			oscap_htable_add(bench->sub.benchmark.dict, item->item.id, item);
+			xccdf_benchmark_register_item(xccdf_item_get_benchmark(item), item);
 	}
 	return item->item.id != NULL;
 }
@@ -691,17 +755,20 @@ bool xccdf_item_process_element(struct xccdf_item * item, xmlTextReaderPtr reade
         }
         break;
     }
+	case XCCDFE_DC_STATUS:
+        oscap_list_add(item->item.dc_statuses, oscap_reference_new_parse(reader));
+		return true;
 	case XCCDFE_VERSION: {
-        xmlNode *ver = xmlTextReaderExpand(reader);
-        char *vt = (char*) xmlGetProp(ver, BAD_CAST "time");
-        if (vt) item->item.version_time = oscap_get_datetime(vt);
-        oscap_free(vt);
-        item->item.version_update = (char*) xmlGetProp(ver, BAD_CAST "update");
-        item->item.version = (char *) xmlNodeGetContent(ver);
-        if (oscap_streq(item->item.version, "")) {
-            oscap_free(item->item.version);
-            item->item.version = NULL;
-        }
+	        xmlNode *ver = xmlTextReaderExpand(reader);
+		/* optional attributes */
+		item->item.version_time = (char*) xmlGetProp(ver, BAD_CAST "time");
+		item->item.version_update = (char*) xmlGetProp(ver, BAD_CAST "update");
+		/* content */
+		item->item.version = (char *) xmlNodeGetContent(ver);
+		if (oscap_streq(item->item.version, "")) {
+			oscap_free(item->item.version);
+			item->item.version = NULL;
+		}
 		return true;
     }
 	case XCCDFE_RATIONALE:
@@ -713,6 +780,12 @@ bool xccdf_item_process_element(struct xccdf_item * item, xmlTextReaderPtr reade
 	case XCCDFE_QUESTION:
         oscap_list_add(item->item.question, oscap_text_new_parse(XCCDF_TEXT_PLAIN, reader));
 		return true;
+	case XCCDFE_METADATA: {
+		char* xml = oscap_get_xml(reader);
+		xccdf_item_add_metadata(item, xml);
+		oscap_free(xml);
+		return true;
+	}
 	default:
 		break;
 	}
@@ -734,6 +807,15 @@ XCCDF_BENCHGETTER(item)  XCCDF_BENCHGETTER(profile) XCCDF_BENCHGETTER(rule)
 XCCDF_BENCHGETTER(group) XCCDF_BENCHGETTER(value)   XCCDF_BENCHGETTER(result)
 #undef XCCDF_BENCHGETTER
 
+const struct xccdf_version_info* xccdf_item_get_schema_version(struct xccdf_item* item)
+{
+	struct xccdf_benchmark* top_benchmark = xccdf_item_get_benchmark(item);
+	if (top_benchmark == NULL)
+		return NULL;
+
+	return xccdf_benchmark_get_schema_version(top_benchmark);
+}
+
 static void *xccdf_item_convert(struct xccdf_item *item, xccdf_type_t type)
 {
 	return ((item != NULL && (item->type & type)) ? item : NULL);
@@ -767,12 +849,22 @@ XCCDF_ITEM_ADDER(struct oscap_text *, rationale, rationale)
 
 XCCDF_ITEM_GETTER(const char *, version)
 XCCDF_ITEM_GETTER(const char *, cluster_id)
-XCCDF_ITEM_GETTER(const char *, version_update) XCCDF_ITEM_GETTER(time_t, version_time) XCCDF_ITEM_GETTER(float, weight)
+XCCDF_ITEM_GETTER(const char *, version_update)
+XCCDF_ITEM_GETTER(const char *, version_time)
+XCCDF_ITEM_GETTER(float, weight)
 XCCDF_ITEM_GETTER(struct xccdf_item *, parent)
 XCCDF_ITEM_GETTER(const char *, extends)
 XCCDF_FLAG_GETTER(resolved)
 XCCDF_FLAG_GETTER(hidden)
 XCCDF_FLAG_GETTER(selected)
+XCCDF_ITEM_SIGETTER(metadata);
+XCCDF_ITEM_ADDER_ONE(item, metadata, metadata, const char*, oscap_strdup);
+XCCDF_ITEM_ADDER_ONE(benchmark, metadata, metadata, const char*, oscap_strdup);
+XCCDF_ITEM_ADDER_ONE(profile, metadata, metadata, const char*, oscap_strdup);
+XCCDF_ITEM_ADDER_ONE(group, metadata, metadata, const char*, oscap_strdup);
+XCCDF_ITEM_ADDER_ONE(rule, metadata, metadata, const char*, oscap_strdup);
+XCCDF_ITEM_ADDER_ONE(value, metadata, metadata, const char*, oscap_strdup);
+XCCDF_ITEM_ADDER_ONE(result, metadata, metadata, const char*, oscap_strdup);
 XCCDF_FLAG_GETTER(multiple)
 XCCDF_FLAG_GETTER(prohibit_changes)
 XCCDF_FLAG_GETTER(abstract)
@@ -784,12 +876,13 @@ XCCDF_ITEM_IGETTER(status, statuses)
 XCCDF_ITEM_ADDER(struct oscap_reference *, reference, references)
 XCCDF_ITEM_ADDER(struct xccdf_warning *, warning, warnings)
 XCCDF_ITEM_ADDER(struct xccdf_status *, status, statuses)
+XCCDF_ITEM_ADDER(struct oscap_reference *, dc_status, dc_statuses)
 XCCDF_ITERATOR_GEN_S(item) XCCDF_ITERATOR_GEN_S(status)
 OSCAP_ITERATOR_GEN(xccdf_warning)
 OSCAP_ITERATOR_REMOVE_F(xccdf_warning)
 
 XCCDF_ITEM_SETTER_SIMPLE(xccdf_numeric, weight)
-XCCDF_ITEM_SETTER_SIMPLE(time_t, version_time)
+XCCDF_ITEM_SETTER_STRING(version_time)
 XCCDF_ITEM_SETTER_STRING(version)
 XCCDF_ITEM_SETTER_STRING(version_update)
 XCCDF_ITEM_SETTER_STRING(extends)
@@ -802,11 +895,17 @@ XCCDF_SETTER_ID(rule) XCCDF_SETTER_ID(group) XCCDF_SETTER_ID(value) XCCDF_SETTER
 #undef XCCDF_SETTER_ID
 
 struct oscap_reference_iterator *xccdf_item_get_references(const struct xccdf_item *item) { return oscap_iterator_new(item->item.references); }
+struct oscap_reference_iterator *xccdf_item_get_dc_statuses(const struct xccdf_item *item) { return oscap_iterator_new(item->item.dc_statuses); }
 struct oscap_reference_iterator *xccdf_benchmark_get_references(const struct xccdf_benchmark *item) { return oscap_iterator_new(XITEM(item)->item.references); }
+struct oscap_reference_iterator *xccdf_benchmark_get_dc_statuses(const struct xccdf_benchmark *item) { return oscap_iterator_new(XITEM(item)->item.dc_statuses); }
 struct oscap_reference_iterator *xccdf_value_get_references(const struct xccdf_value *item) { return oscap_iterator_new(XITEM(item)->item.references); }
+struct oscap_reference_iterator *xccdf_value_get_dc_statuses(const struct xccdf_value *item) { return oscap_iterator_new(XITEM(item)->item.dc_statuses); }
 struct oscap_reference_iterator *xccdf_group_get_references(const struct xccdf_group *item) { return oscap_iterator_new(XITEM(item)->item.references); }
+struct oscap_reference_iterator *xccdf_group_get_dc_statuses(const struct xccdf_group *item) { return oscap_iterator_new(XITEM(item)->item.dc_statuses); }
 struct oscap_reference_iterator *xccdf_rule_get_references(const struct xccdf_rule *item) { return oscap_iterator_new(XITEM(item)->item.references); }
+struct oscap_reference_iterator *xccdf_rule_get_dc_statuses(const struct xccdf_rule *item) { return oscap_iterator_new(XITEM(item)->item.dc_statuses); }
 struct oscap_reference_iterator *xccdf_profile_get_references(const struct xccdf_profile *item) { return oscap_iterator_new(XITEM(item)->item.references); }
+struct oscap_reference_iterator *xccdf_profile_get_dc_statuses(const struct xccdf_profile *item) { return oscap_iterator_new(XITEM(item)->item.dc_statuses); }
 
 struct xccdf_item_iterator *xccdf_item_get_content(const struct xccdf_item *item)
 {
@@ -837,6 +936,11 @@ struct xccdf_status *xccdf_status_new(void)
     return oscap_calloc(1, sizeof(struct xccdf_status));
 }
 
+const char *xccdf_status_type_to_text(xccdf_status_type_t id)
+{
+	return oscap_enum_to_string(XCCDF_STATUS_MAP, id);
+}
+
 void xccdf_status_dump(struct xccdf_status *status, int depth)
 {
 	xccdf_print_depth(depth);
@@ -853,21 +957,21 @@ void xccdf_status_free(struct xccdf_status *status)
 OSCAP_ACCESSOR_SIMPLE(time_t, xccdf_status, date)
 OSCAP_ACCESSOR_SIMPLE(xccdf_status_type_t, xccdf_status, status)
 
-xccdf_status_type_t xccdf_item_get_current_status(const struct xccdf_item *item)
+struct xccdf_status * xccdf_item_get_current_status(const struct xccdf_item *item)
 {
 	time_t maxtime = 0;
-	xccdf_status_type_t maxtype = XCCDF_STATUS_NOT_SPECIFIED;
-	const struct oscap_list_item *li = item->item.statuses->first;
+	struct xccdf_status *max_status = NULL;
 	struct xccdf_status *status;
+	const struct oscap_list_item *li = item->item.statuses->first;
 	while (li) {
 		status = li->data;
 		if (status->date == 0 || status->date >= maxtime) {
 			maxtime = status->date;
-			maxtype = status->status;
+			max_status = status;
 		}
 		li = li->next;
 	}
-	return maxtype;
+	return max_status;
 }
 
 struct xccdf_model *xccdf_model_clone(const struct xccdf_model *old_model)
@@ -963,6 +1067,7 @@ void xccdf_profile_item_clone(struct xccdf_profile_item *clone, const struct xcc
 	clone->setvalues = oscap_list_clone(item->setvalues, (oscap_clone_func) xccdf_setvalue_clone);
 	clone->refine_values = oscap_list_clone(item->refine_values, (oscap_clone_func) xccdf_refine_value_clone);
 	clone->refine_rules = oscap_list_clone(item->refine_rules, (oscap_clone_func) xccdf_refine_rule_clone);
+	clone->tailoring = item->tailoring;
 }
 
 struct xccdf_benchmark_item * xccdf_benchmark_item_clone(struct xccdf_item *parent, const struct xccdf_benchmark * bench)
@@ -970,7 +1075,11 @@ struct xccdf_benchmark_item * xccdf_benchmark_item_clone(struct xccdf_item *pare
     struct xccdf_benchmark_item *item = &XITEM(bench)->sub.benchmark;
     struct xccdf_benchmark_item *clone = &parent->sub.benchmark;
 
-	clone->dict = oscap_htable_new();
+    clone->schema_version = item->schema_version;
+
+	clone->items_dict = oscap_htable_new();
+	clone->profiles_dict = oscap_htable_new();
+	clone->results_dict = oscap_htable_new();
 	clone->notices = oscap_list_clone(item->notices, (oscap_clone_func) xccdf_notice_clone);
 	clone->plain_texts = oscap_list_clone(item->plain_texts, (oscap_clone_func) xccdf_plain_text_clone);
 	
@@ -1040,8 +1149,6 @@ struct xccdf_value_instance * xccdf_value_instance_clone(const struct xccdf_valu
 
 void xccdf_value_item_clone(struct xccdf_value_item *clone, const struct xccdf_value_item * item)
 {
-	//struct xccdf_value_item * clone = oscap_calloc(1, sizeof(struct xccdf_value_item));
-
 	//these three are values which can be directly assigned.
 	clone->type = item->type;
 	clone->interface_hint = item->interface_hint;
@@ -1103,7 +1210,7 @@ struct xccdf_rule_result * xccdf_rule_result_clone(const struct xccdf_rule_resul
 	struct xccdf_rule_result * clone = oscap_calloc(1, sizeof(struct xccdf_rule_result));
 	clone->idref = oscap_strdup(result->idref);
 	clone->role = result->role;
-	clone->time = result->time;
+	clone->time = oscap_strdup(result->time);
 	clone->weight = result->weight;
 	clone->severity = result->severity;
 	clone->result = result->result;
@@ -1128,8 +1235,8 @@ struct xccdf_score * xccdf_score_clone(const struct xccdf_score * score)
 
 void xccdf_result_item_clone(struct xccdf_result_item *clone, const struct xccdf_result_item * item)
 {
-	clone->start_time = item->start_time;
-	clone->end_time = item->end_time;
+	clone->start_time = oscap_strdup(item->start_time);
+	clone->end_time = oscap_strdup(item->end_time);
 	clone->test_system = oscap_strdup(item->test_system);
 	clone->benchmark_uri = oscap_strdup(item->benchmark_uri);
 	clone->profile = oscap_strdup(item->profile);
@@ -1140,6 +1247,7 @@ void xccdf_result_item_clone(struct xccdf_result_item *clone, const struct xccdf
 	clone->remarks = oscap_list_clone(item->remarks, (oscap_clone_func) oscap_text_clone);
 	clone->target_addresses = oscap_list_clone(item->target_addresses, (oscap_clone_func) oscap_strdup);
 	clone->target_facts = oscap_list_clone(item->target_facts, (oscap_clone_func) xccdf_target_fact_clone);
+	clone->target_id_refs = oscap_list_clone(item->target_id_refs, (oscap_clone_func) xccdf_target_identifier_clone);
 	clone->setvalues = oscap_list_clone(item->setvalues, (oscap_clone_func) xccdf_setvalue_clone);
 	clone->rule_results = oscap_list_clone(item->rule_results, (oscap_clone_func) xccdf_rule_result_clone);
 	clone->scores = oscap_list_clone(item->scores, (oscap_clone_func) xccdf_score_clone);
@@ -1199,7 +1307,8 @@ char* oscap_text_xccdf_substitute(const char *text, xccdf_substitution_func cb,
     xpathCtx = xmlXPathNewContext(doc);
     if (xpathCtx == NULL) goto cleanup;
 
-    if (xmlXPathRegisterNs(xpathCtx, BAD_CAST "cdf", BAD_CAST XCCDF_XMLNS) != 0) goto cleanup;
+    // FIXME: Remove hardcoded version
+    if (xmlXPathRegisterNs(xpathCtx, BAD_CAST "cdf", BAD_CAST "http://checklists.nist.gov/xccdf/1.1") != 0) goto cleanup;
 
     xpathObj = xmlXPathEvalExpression(BAD_CAST xpath, xpathCtx);
     if (xpathObj == NULL) goto cleanup;
@@ -1216,9 +1325,6 @@ char* oscap_text_xccdf_substitute(const char *text, xccdf_substitution_func cb,
                 const char *id = (const char *) xmlGetProp(cur, BAD_CAST "idref");
                 if (id) sub = cb(XCCDF_SUBST_SUB, id, arg);
             }
-            else if (oscap_streq((const char *) cur->name, "instance")) {
-                // TODO
-            }
         }
 
         // do the substitution
