@@ -23,13 +23,15 @@
 #ifndef XCCDF_ITEM_
 #define XCCDF_ITEM_
 
-#include "public/xccdf.h"
 #include <assert.h>
+
+#include "public/xccdf_benchmark.h"
 #include "elements.h"
-#include "../common/reference_priv.h"
-#include "../common/list.h"
-#include "../common/util.h"
-#include "../common/text_priv.h"
+
+#include "common/reference_priv.h"
+#include "common/list.h"
+#include "common/util.h"
+#include "common/text_priv.h"
 
 OSCAP_HIDDEN_START;
 
@@ -72,15 +74,18 @@ struct xccdf_item_base {
 
 	char *version;
 	char *version_update;
-	time_t version_time;
+	char *version_time;
 
 	struct xccdf_item *parent;
 	char *extends;
 	struct oscap_list *statuses;
+	struct oscap_list *dc_statuses;
 	struct oscap_list *references;
 	struct oscap_list *platforms;
 	struct xccdf_flags flags;
 	struct xccdf_defflags defined_flags;
+
+	struct oscap_list *metadata;
 };
 
 struct xccdf_rule_item {
@@ -134,8 +139,8 @@ struct xccdf_value_item {
 };
 
 struct xccdf_result_item {
-	time_t start_time;
-	time_t end_time;
+	char *start_time;
+	char *end_time;
 	char *test_system;
 	char *benchmark_uri;
 	char *profile;
@@ -146,6 +151,7 @@ struct xccdf_result_item {
 	struct oscap_list *remarks;
 	struct oscap_list *target_addresses;
 	struct oscap_list *target_facts;
+	struct oscap_list *target_id_refs;
 	struct oscap_list *setvalues;
 	struct oscap_list *rule_results;
 	struct oscap_list *scores;
@@ -157,17 +163,41 @@ struct xccdf_profile_item {
 	struct oscap_list *setvalues;
 	struct oscap_list *refine_values;
 	struct oscap_list *refine_rules;
+	bool tailoring;
+};
+
+struct xccdf_tailoring {
+	// The ref is "just informative" according to the spec, we never
+	// interpret it and just load and save it as a string.
+	char *benchmark_ref;
+
+	struct oscap_list *statuses;
+	struct oscap_list *dc_statuses;
+
+	char *version;
+	char *version_update;
+	char *version_time;
+
+	struct oscap_list *metadata;
+
+	struct oscap_list *profiles;
 };
 
 struct xccdf_benchmark_item {
+	const struct xccdf_version_info *schema_version;
 
-	struct oscap_htable *dict;
+	struct oscap_htable *items_dict;		/* Stores only xccdf:Item */
+	struct oscap_htable *profiles_dict;		/* xccdf:Profile */
+	struct oscap_htable *results_dict;		/* xccdf:TestResult */
+	struct oscap_htable *clusters_dict;		/* xccdf:items by cluster-id */
 	struct oscap_list *notices;
 	struct oscap_list *plain_texts;
 
+	struct cpe_dict_model *cpe_list;
+	struct cpe_lang_model *cpe_lang_model;
+
 	char *style;
 	char *style_href;
-	char *metadata;
 	char *lang;
 
 	struct oscap_list *front_matter;
@@ -252,6 +282,12 @@ struct xccdf_check {
 	char *system;
 	char *selector;
 	char *content;
+	struct _xccdf_check_flags {
+		bool multicheck:1;
+		bool def_multicheck:1;
+		bool negate:1;
+		bool def_negate:1;
+	} flags;
 	struct oscap_list *imports;
 	struct oscap_list *exports;
 	struct oscap_list *content_refs;
@@ -264,6 +300,7 @@ struct xccdf_check_content_ref {
 
 struct xccdf_check_import {
 	char *name;
+	char *xpath;
 	char *content;
 };
 
@@ -297,17 +334,10 @@ struct xccdf_fixtext {
 	char *fixref;
 };
 
-/*
-struct xccdf_reference {
-	struct oscap_text *text;
-    char *href;
-};
-*/
-
 struct xccdf_rule_result {
 	char *idref;
 	xccdf_role_t role;
-	time_t time;
+	char *time;
 	float weight;
 	xccdf_level_t severity;
 	xccdf_test_result_type_t result;
@@ -336,7 +366,7 @@ struct xccdf_score {
 };
 
 struct xccdf_override {
-	time_t time;
+	char *time;
 	char *authority;
 	xccdf_test_result_type_t old_result;
 	xccdf_test_result_type_t new_result;
@@ -354,6 +384,16 @@ struct xccdf_target_fact {
 	char *value;
 };
 
+struct xccdf_target_identifier {
+	bool any_element; // if false, this represents <target-id-ref>, else it's any element
+
+	xmlNodePtr element;
+
+	const char* system;
+	const char* href;
+	const char* name;
+};
+
 struct xccdf_instance {
     char *context;
     char *parent_context;
@@ -387,15 +427,22 @@ void xccdf_item_release(struct xccdf_item *item);
 void xccdf_item_print(struct xccdf_item *item, int depth);
 void xccdf_item_dump(struct xccdf_item *item, int depth);
 struct xccdf_item* xccdf_item_get_benchmark_internal(struct xccdf_item* item);
-
 bool xccdf_benchmark_parse(struct xccdf_item *benchmark, xmlTextReaderPtr reader);
 void xccdf_benchmark_dump(struct xccdf_benchmark *benchmark);
+struct oscap_htable_iterator *xccdf_benchmark_get_cluster_items(struct xccdf_benchmark *benchmark, const char *cluster_id);
 bool xccdf_benchmark_register_item(struct xccdf_benchmark *benchmark, struct xccdf_item *item);
 bool xccdf_benchmark_unregister_item(struct xccdf_item *item);
 bool xccdf_benchmark_rename_item(struct xccdf_item *item, const char *newid);
-char *xccdf_benchmark_gen_id(struct xccdf_benchmark *benchmark, const char *prefix);
+char *xccdf_benchmark_gen_id(struct xccdf_benchmark *benchmark, xccdf_type_t type, const char *prefix);
+struct xccdf_profile *xccdf_benchmark_get_profile_by_id(struct xccdf_benchmark *benchmark, const char *profile_id);
+struct xccdf_result *xccdf_benchmark_get_result_by_id(struct xccdf_benchmark *benchmark, const char *testresult_id);
 bool xccdf_add_item(struct oscap_list *list, struct xccdf_item *parent, struct xccdf_item *item, const char *prefix);
 
+struct xccdf_tailoring *xccdf_tailoring_new(void);
+void xccdf_tailoring_free(struct xccdf_tailoring *tailoring);
+struct xccdf_tailoring *xccdf_tailoring_parse(xmlTextReaderPtr reader, struct xccdf_item* benchmark);
+xmlNodePtr xccdf_tailoring_to_dom(struct xccdf_tailoring *tailoring, xmlDocPtr doc, xmlNodePtr parent, const struct xccdf_version_info *version_info);
+
 struct xccdf_item *xccdf_profile_new_internal(struct xccdf_item *bench);
 struct xccdf_item *xccdf_profile_parse(xmlTextReaderPtr reader, struct xccdf_item *bench);
 void xccdf_profile_dump(struct xccdf_item *prof, int depth);
@@ -411,6 +458,8 @@ void xccdf_group_dump(struct xccdf_item *group, int depth);
 struct xccdf_item *xccdf_rule_new_internal(struct xccdf_item *parent);
 struct xccdf_item *xccdf_rule_parse(xmlTextReaderPtr reader, struct xccdf_item *parent);
 void xccdf_rule_dump(struct xccdf_item *rule, int depth);
+struct xccdf_check_iterator *xccdf_rule_get_checks_filtered(struct xccdf_item *rule, char *selector);
+struct xccdf_check_iterator *xccdf_rule_get_complex_checks(struct xccdf_item *rule);
 
 struct xccdf_item *xccdf_value_parse(xmlTextReaderPtr reader, struct xccdf_item *parent);
 struct xccdf_item *xccdf_value_new_internal(struct xccdf_item *parent, xccdf_value_type_t type);
@@ -426,10 +475,14 @@ struct xccdf_model *xccdf_model_new_xml(xmlTextReaderPtr reader);
 void xccdf_cstring_dump(const char *data, int depth);
 void xccdf_result_dump(struct xccdf_result *res, int depth);
 struct xccdf_result *xccdf_result_new_parse(xmlTextReaderPtr reader);
+int xccdf_rule_result_set_time_current(struct xccdf_rule_result *item);
+int xccdf_result_set_start_time_current(struct xccdf_result *item);
+int xccdf_result_set_end_time_current(struct xccdf_result *item);
 
 
 struct xccdf_check *xccdf_check_parse(xmlTextReaderPtr reader);
 void xccdf_check_dump(struct xccdf_check *check, int depth);
+bool xccdf_check_inject_content_ref(struct xccdf_check *check, const struct xccdf_check_content_ref *content, const char *name);
 void xccdf_check_content_ref_dump(struct xccdf_check_content_ref *ref, int depth);
 struct xccdf_ident *xccdf_ident_parse(xmlTextReaderPtr reader);
 void xccdf_ident_dump(struct xccdf_ident *ident, int depth);
@@ -440,7 +493,6 @@ struct xccdf_setvalue *xccdf_setvalue_new_parse(xmlTextReaderPtr reader);
 void xccdf_setvalue_dump(struct xccdf_setvalue *sv, int depth);
 
 struct xccdf_warning *xccdf_warning_new_parse(xmlTextReaderPtr reader);
-//struct xccdf_reference *xccdf_reference_new_parse(xmlTextReaderPtr reader);
 
 //private methods for cloning items
 //Will clone the item member of a xccdf_item object
