@@ -24,9 +24,12 @@
 #include <config.h>
 #endif
 
+#include <math.h>
+
 #include "item.h"
 #include "helpers.h"
 #include "xccdf_impl.h"
+#include "common/debug_priv.h"
 
 struct xccdf_setvalue *xccdf_setvalue_new(void)
 {
@@ -52,9 +55,11 @@ struct xccdf_setvalue *xccdf_setvalue_new_parse(xmlTextReaderPtr reader)
 	return sv;
 }
 
-xmlNode *xccdf_setvalue_to_dom(struct xccdf_setvalue *setvalue, xmlDoc *doc, xmlNode *parent)
+xmlNode *xccdf_setvalue_to_dom(struct xccdf_setvalue *setvalue, xmlDoc *doc, xmlNode *parent, const struct xccdf_version_info* version_info)
 {
-	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent, XCCDF_BASE_NAMESPACE);
+	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent,
+			(const xmlChar*)xccdf_version_info_get_namespace_uri(version_info));
+
 	xmlNode *setvalue_node = xmlNewTextChild(parent, ns_xccdf, BAD_CAST "set-value",  BAD_CAST setvalue->value);
 
 	if (setvalue->item)
@@ -108,6 +113,11 @@ struct xccdf_refine_rule * xccdf_refine_rule_clone(const struct xccdf_refine_rul
 	return clone;
 }	
 
+bool xccdf_refine_rule_weight_defined(const struct xccdf_refine_rule *rule)
+{
+	return !isnan(rule->weight);
+}
+
 struct xccdf_select *xccdf_select_new(void)
 {
 	struct xccdf_select *foo = oscap_calloc(1, sizeof(struct xccdf_select));
@@ -183,6 +193,7 @@ struct xccdf_item *xccdf_profile_new_internal(struct xccdf_item *bench)
 	prof->sub.profile.setvalues = oscap_list_new();
 	prof->sub.profile.refine_values = oscap_list_new();
 	prof->sub.profile.refine_rules = oscap_list_new();
+	prof->sub.profile.tailoring = false;
 	return prof;
 }
 
@@ -245,9 +256,7 @@ struct xccdf_item *xccdf_profile_parse(xmlTextReaderPtr reader, struct xccdf_ite
 				struct xccdf_refine_rule *rr = xccdf_refine_rule_new();
 				rr->item = oscap_strdup(id);
 				rr->selector = xccdf_attribute_copy(reader, XCCDFA_SELECTOR);
-				if (xccdf_attribute_has(reader, XCCDFA_WEIGHT))
-				    rr->weight = xccdf_attribute_get_float(reader, XCCDFA_WEIGHT);
-                                else rr->weight = -1.0;
+				rr->weight = xccdf_attribute_get_float(reader, XCCDFA_WEIGHT);
 				if (xccdf_attribute_has(reader, XCCDFA_ROLE))
 					rr->role =
 					    oscap_string_to_enum(XCCDF_ROLE_MAP,
@@ -280,7 +289,9 @@ struct xccdf_item *xccdf_profile_parse(xmlTextReaderPtr reader, struct xccdf_ite
 				break;
 			}
 		default:
-			xccdf_item_process_element(prof, reader);
+			if (!xccdf_item_process_element(prof, reader))
+				dW("Encountered an unknown element '%s' while parsing XCCDF profile.",
+				   xmlTextReaderConstLocalName(reader));
 		}
 		xmlTextReaderRead(reader);
 	}
@@ -290,7 +301,10 @@ struct xccdf_item *xccdf_profile_parse(xmlTextReaderPtr reader, struct xccdf_ite
 
 void xccdf_profile_to_dom(struct xccdf_profile *profile, xmlNode *profile_node, xmlDoc *doc, xmlNode *parent)
 {
-	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent, XCCDF_BASE_NAMESPACE);
+	const struct xccdf_version_info* version_info = xccdf_item_get_schema_version(XITEM(profile));
+
+	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent,
+			(const xmlChar*)xccdf_version_info_get_namespace_uri(version_info));
 
 	/* Handle attributes */
 	if (xccdf_profile_get_abstract(profile))
@@ -333,7 +347,7 @@ void xccdf_profile_to_dom(struct xccdf_profile *profile, xmlNode *profile_node,
 	struct xccdf_setvalue_iterator *setvalues = xccdf_profile_get_setvalues(profile);
 	while (xccdf_setvalue_iterator_has_more(setvalues)) {
 		struct xccdf_setvalue *setvalue = xccdf_setvalue_iterator_next(setvalues);
-		xccdf_setvalue_to_dom(setvalue, doc, profile_node);
+		xccdf_setvalue_to_dom(setvalue, doc, profile_node, version_info);
 	}
 	xccdf_setvalue_iterator_free(setvalues);
 
@@ -376,13 +390,15 @@ void xccdf_profile_to_dom(struct xccdf_profile *profile, xmlNode *profile_node,
 			xmlNewProp(refrule_node, BAD_CAST "selector", BAD_CAST selector);
 
 		xccdf_level_t severity = xccdf_refine_rule_get_severity(refine_rule);
-		if (severity != 0)
+		if (severity != XCCDF_LEVEL_NOT_DEFINED)
 			xmlNewProp(refrule_node, BAD_CAST "severity", BAD_CAST XCCDF_LEVEL_MAP[severity - 1].string);
 
-		float weight = xccdf_refine_rule_get_weight(refine_rule);
-		char *weight_str = oscap_sprintf("%f", weight);
-		xmlNewProp(refrule_node, BAD_CAST "weight", BAD_CAST weight_str);
-        oscap_free(weight_str);
+		if (xccdf_refine_rule_weight_defined(refine_rule)) {
+			float weight = xccdf_refine_rule_get_weight(refine_rule);
+			char *weight_str = oscap_sprintf("%f", weight);
+			xmlNewProp(refrule_node, BAD_CAST "weight", BAD_CAST weight_str);
+			oscap_free(weight_str);
+		}
 		
 		xccdf_texts_to_dom(xccdf_refine_rule_get_remarks(refine_rule), refrule_node, "remark");
 	}
@@ -417,6 +433,7 @@ void xccdf_profile_free(struct xccdf_item *prof)
 
 XCCDF_STATUS_CURRENT(profile)
 XCCDF_ACCESSOR_STRING(profile, note_tag)
+XCCDF_ACCESSOR_SIMPLE(profile, bool, tailoring)
 XCCDF_LISTMANIP(profile, select, selects)
 XCCDF_LISTMANIP(profile, setvalue, setvalues)
 XCCDF_LISTMANIP(profile, refine_value, refine_values)
