@@ -25,13 +25,25 @@
 #endif
 
 #include <math.h>
-#include <text.h>
+#include <sys/utsname.h>
+
+#if defined(__linux__)
+#include <arpa/inet.h>
+#include <ifaddrs.h>
+#include <net/if.h>
+#include <netdb.h>
+#include <string.h>
+#include <sys/ioctl.h>
+#include <sys/socket.h>
+#include <unistd.h>
+#endif
 
 #include "item.h"
 #include "helpers.h"
 #include "xccdf_impl.h"
 #include "common/_error.h"
-
+#include "oscap_text.h"
+#include "common/debug_priv.h"
 
 // constants
 static const xccdf_numeric XCCDF_SCORE_MAX_DAFAULT = 100.0f;
@@ -53,7 +65,8 @@ struct xccdf_result *xccdf_result_new(void)
 	struct xccdf_item *result = xccdf_item_new(XCCDF_RESULT, NULL);
 	oscap_create_lists(&result->sub.result.identities, &result->sub.result.targets,
 		&result->sub.result.remarks, &result->sub.result.target_addresses,
-		&result->sub.result.target_facts, &result->sub.result.setvalues, &result->sub.result.organizations,
+		&result->sub.result.target_facts, &result->sub.result.target_id_refs,
+		&result->sub.result.setvalues, &result->sub.result.organizations,
 		&result->sub.result.rule_results, &result->sub.result.scores, NULL);
 	return XRESULT(result);
 }
@@ -71,12 +84,15 @@ struct xccdf_result * xccdf_result_clone(const struct xccdf_result * result)
 static inline void xccdf_result_free_impl(struct xccdf_item *result)
 {
 	if (result != NULL) {
+		oscap_free(result->sub.result.start_time);
+		oscap_free(result->sub.result.end_time);
 		oscap_free(result->sub.result.test_system);
 		oscap_free(result->sub.result.benchmark_uri);
 		oscap_free(result->sub.result.profile);
 
 		oscap_list_free(result->sub.result.identities, (oscap_destruct_func) xccdf_identity_free);
 		oscap_list_free(result->sub.result.target_facts, (oscap_destruct_func) xccdf_target_fact_free);
+		oscap_list_free(result->sub.result.target_id_refs, (oscap_destruct_func) xccdf_target_identifier_free);
 		oscap_list_free(result->sub.result.scores, (oscap_destruct_func) xccdf_score_free);
 		oscap_list_free(result->sub.result.targets, oscap_free);
 		oscap_list_free(result->sub.result.remarks, (oscap_destruct_func) oscap_text_free);
@@ -89,27 +105,10 @@ static inline void xccdf_result_free_impl(struct xccdf_item *result)
 	}
 }
 
-time_t xccdf_result_get_start_time(const struct xccdf_result* result)
-{
-	return ((struct xccdf_item *)result)->sub.result.start_time;
-}
-
-time_t xccdf_result_get_end_time(const struct xccdf_result* result)
-{
-	return ((struct xccdf_item *)result)->sub.result.end_time;
-}
-
-_Bool xccdf_result_set_start_time(struct xccdf_result* result, time_t start)
-{
-	return (((struct xccdf_item *)result)->sub.result.start_time = start);
-}
-
-_Bool xccdf_result_set_end_time(struct xccdf_result* result, time_t end)
-{
-	return (((struct xccdf_item *)result)->sub.result.end_time = end);
-}
 XCCDF_FREE_GEN(result)
 
+XCCDF_ACCESSOR_STRING(result, start_time)
+XCCDF_ACCESSOR_STRING(result, end_time)
 XCCDF_ACCESSOR_STRING(result, test_system)
 XCCDF_ACCESSOR_STRING(result, benchmark_uri)
 XCCDF_ACCESSOR_STRING(result, profile)
@@ -119,12 +118,128 @@ XCCDF_LISTMANIP_STRING(result, target_address, target_addresses)
 XCCDF_LISTMANIP_STRING(result, organization, organizations)
 XCCDF_LISTMANIP_TEXT(result, remark, remarks)
 XCCDF_LISTMANIP(result, target_fact, target_facts)
+XCCDF_LISTMANIP(result, target_identifier, target_id_refs)
 XCCDF_LISTMANIP(result, setvalue, setvalues)
 XCCDF_LISTMANIP(result, rule_result, rule_results)
 XCCDF_LISTMANIP(result, score, scores)
 OSCAP_ITERATOR_GEN(xccdf_result)
 OSCAP_ITERATOR_REMOVE_F(xccdf_result)
 
+static inline void _xccdf_result_fill_scanner(struct xccdf_result *result)
+{
+	struct xccdf_target_fact *fact = NULL;
+	fact = xccdf_target_fact_new();
+	xccdf_target_fact_set_name(fact, "urn:xccdf:fact:scanner:name");
+	xccdf_target_fact_set_string(fact, "OpenSCAP");
+	xccdf_result_add_target_fact(result, fact);
+
+	fact = xccdf_target_fact_new();
+	xccdf_target_fact_set_name(fact, "urn:xccdf:fact:scanner:version");
+	xccdf_target_fact_set_string(fact, oscap_get_version());
+	xccdf_result_add_target_fact(result, fact);
+}
+
+static inline void _xccdf_result_fill_identity(struct xccdf_result *result)
+{
+	struct xccdf_identity *id = xccdf_identity_new();
+	// Reasoning here is that OpenSCAP does not authenticate user
+	// nor it grants she any additional privileges
+	xccdf_identity_set_authenticated(id, 0);
+	xccdf_identity_set_privileged(id, 0);
+	xccdf_identity_set_name(id, getlogin());
+	xccdf_result_add_identity(result, id);
+}
+
+static inline void _xccdf_result_clear_metadata(struct xccdf_item *result)
+{
+	oscap_list_free(result->sub.result.targets, oscap_free);
+	oscap_list_free(result->sub.result.target_addresses, oscap_free);
+	oscap_list_free(result->sub.result.target_facts, (oscap_destruct_func) xccdf_target_fact_free);
+	oscap_list_free(result->sub.result.identities, (oscap_destruct_func) xccdf_identity_free);
+	oscap_create_lists(
+		&result->sub.result.targets,
+		&result->sub.result.target_addresses,
+		&result->sub.result.target_facts,
+		&result->sub.result.identities,
+		NULL);
+}
+
+void xccdf_result_fill_sysinfo(struct xccdf_result *result)
+{
+	struct utsname sname;
+	struct ifaddrs *ifaddr, *ifa;
+	int fd;
+
+	if (uname(&sname) == -1)
+		return;
+
+	_xccdf_result_clear_metadata(XITEM(result));
+	/* store target name */
+	xccdf_result_add_target(result, sname.nodename);
+	_xccdf_result_fill_scanner(result);
+	_xccdf_result_fill_identity(result);
+
+#if defined(__linux__)
+
+	/* get network interfaces */
+	if (getifaddrs(&ifaddr) == -1)
+		return;
+
+	fd = socket(PF_INET, SOCK_DGRAM, IPPROTO_IP);
+	if (fd == -1)
+		goto out1;
+
+	for (ifa = ifaddr; ifa != NULL; ifa = ifa->ifa_next) {
+		int family;
+		char hostip[NI_MAXHOST];
+		struct ifreq ifr;
+
+		if (!ifa->ifa_addr)
+			continue;
+		family = ifa->ifa_addr->sa_family;
+		if (family != AF_INET && family != AF_INET6)
+			continue;
+
+		if (family == AF_INET) {
+			if (getnameinfo(ifa->ifa_addr, sizeof(struct sockaddr_in),
+					hostip, sizeof(hostip), NULL, 0, NI_NUMERICHOST))
+				goto out2;
+		} else {
+			struct sockaddr_in6 *sin6;
+
+			sin6 = (struct sockaddr_in6 *) ifa->ifa_addr;
+			if (!inet_ntop(family, (const void *) &sin6->sin6_addr,
+				       hostip, sizeof(hostip)))
+				goto out2;
+		}
+		/* store ip address */
+		xccdf_result_add_target_address(result, hostip);
+
+		memset(&ifr, 0, sizeof(ifr));
+		strncpy(ifr.ifr_name, ifa->ifa_name, IFNAMSIZ);
+		ifr.ifr_name[IFNAMSIZ - 1] = 0;
+		if (ioctl(fd, SIOCGIFHWADDR, &ifr) >= 0) {
+			struct xccdf_target_fact *fact;
+			unsigned char mac[6];
+			char macbuf[20];
+
+			memcpy(mac, ifr.ifr_hwaddr.sa_data, sizeof(mac));
+			snprintf(macbuf, sizeof(macbuf), "%02X:%02X:%02X:%02X:%02X:%02X",
+				 mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
+			fact = xccdf_target_fact_new();
+			xccdf_target_fact_set_name(fact, "urn:xccdf:fact:ethernet:MAC");
+			xccdf_target_fact_set_string(fact, macbuf);
+			/* store mac address */
+			xccdf_result_add_target_fact(result, fact);
+		}
+	}
+
+ out2:
+	close(fd);
+ out1:
+	freeifaddrs(ifaddr);
+#endif
+}
 
 struct xccdf_rule_result *xccdf_rule_result_new(void)
 {
@@ -139,6 +254,7 @@ void xccdf_rule_result_free(struct xccdf_rule_result *rr)
 	if (rr != NULL) {
 		oscap_free(rr->idref);
 		oscap_free(rr->version);
+		oscap_free(rr->time);
 
 		oscap_list_free(rr->overrides, (oscap_destruct_func) xccdf_override_free);
 		oscap_list_free(rr->idents, (oscap_destruct_func) xccdf_ident_free);
@@ -151,11 +267,11 @@ void xccdf_rule_result_free(struct xccdf_rule_result *rr)
 	}
 }
 
-OSCAP_ACCESSOR_SIMPLE(time_t, xccdf_rule_result, time)
 OSCAP_ACCESSOR_SIMPLE(xccdf_role_t, xccdf_rule_result, role)
 OSCAP_ACCESSOR_SIMPLE(float, xccdf_rule_result, weight)
 OSCAP_ACCESSOR_SIMPLE(xccdf_level_t, xccdf_rule_result, severity)
 OSCAP_ACCESSOR_SIMPLE(xccdf_test_result_type_t, xccdf_rule_result, result)
+OSCAP_ACCESSOR_STRING(xccdf_rule_result, time)
 OSCAP_ACCESSOR_STRING(xccdf_rule_result, version)
 OSCAP_ACCESSOR_STRING(xccdf_rule_result, idref)
 OSCAP_IGETINS(xccdf_ident, xccdf_rule_result, idents, ident)
@@ -226,9 +342,9 @@ void xccdf_override_free(struct xccdf_override *oride)
 	}
 }
 
-OSCAP_ACCESSOR_SIMPLE(time_t, xccdf_override, time)
 OSCAP_ACCESSOR_SIMPLE(xccdf_test_result_type_t, xccdf_override, new_result)
 OSCAP_ACCESSOR_SIMPLE(xccdf_test_result_type_t, xccdf_override, old_result)
+OSCAP_ACCESSOR_STRING(xccdf_override, time)
 OSCAP_ACCESSOR_STRING(xccdf_override, authority)
 OSCAP_ACCESSOR_TEXT(xccdf_override, remark)
 
@@ -294,6 +410,136 @@ OSCAP_ACCESSOR_STRING(xccdf_target_fact, name)
 OSCAP_ITERATOR_GEN(xccdf_target_fact)
 OSCAP_ITERATOR_REMOVE_F(xccdf_target_fact)
 
+struct xccdf_target_identifier* xccdf_target_identifier_new(void)
+{
+    return oscap_calloc(1, sizeof(struct xccdf_target_identifier));
+}
+
+struct xccdf_target_identifier* xccdf_target_identifier_clone(const struct xccdf_target_identifier* ti)
+{
+	struct xccdf_target_identifier* ret = xccdf_target_identifier_new();
+
+	if (xccdf_target_identifier_get_xml_node(ti)) {
+		xccdf_target_identifier_set_xml_node(ret, xccdf_target_identifier_get_xml_node(ti));
+	}
+	else {
+		xccdf_target_identifier_set_system(ret, xccdf_target_identifier_get_system(ti));
+		xccdf_target_identifier_set_name(ret, xccdf_target_identifier_get_name(ti));
+		xccdf_target_identifier_set_href(ret, xccdf_target_identifier_get_href(ti));
+	}
+
+	return ret;
+}
+
+void xccdf_target_identifier_free(struct xccdf_target_identifier *ti)
+{
+    if (ti != NULL) {
+    	if (ti->any_element) {
+    		xmlFreeNode(ti->element);
+    	}
+		else {
+        	oscap_free(ti->system);
+        	oscap_free(ti->href);
+        	oscap_free(ti->name);
+        }
+
+    	oscap_free(ti);
+    }
+}
+
+bool xccdf_target_identifier_set_xml_node(struct xccdf_target_identifier *ti, void* node)
+{
+	if (!ti->any_element) {
+		oscap_free(ti->system);
+		oscap_free(ti->href);
+		oscap_free(ti->name);
+	}
+	else if (ti->element)
+		xmlFreeNode(ti->element);
+
+	ti->any_element = true;
+	ti->element = (xmlNodePtr)node;
+
+	return true;
+}
+
+void* xccdf_target_identifier_get_xml_node(const struct xccdf_target_identifier* ti)
+{
+	if (!ti->any_element)
+		return NULL;
+
+	return ti->element;
+}
+
+bool xccdf_target_identifier_set_system(struct xccdf_target_identifier *ti, const char *newval)
+{
+	if (ti->any_element) {
+		if (ti->element)
+			xmlFreeNode(ti->element);
+	}
+
+	ti->any_element = false;
+	oscap_free(ti->system);
+	ti->system = oscap_strdup(newval);
+
+	return true;
+}
+
+const char* xccdf_target_identifier_get_system(const struct xccdf_target_identifier* ti)
+{
+	if (ti->any_element)
+		return NULL;
+
+	return ti->system;
+}
+
+bool xccdf_target_identifier_set_href(struct xccdf_target_identifier *ti, const char *newval)
+{
+	if (ti->any_element) {
+		if (ti->element)
+			xmlFreeNode(ti->element);
+	}
+
+	ti->any_element = false;
+	oscap_free(ti->href);
+	ti->href = oscap_strdup(newval);
+
+	return true;
+}
+
+const char* xccdf_target_identifier_get_href(const struct xccdf_target_identifier* ti)
+{
+	if (ti->any_element)
+		return NULL;
+
+	return ti->href;
+}
+
+bool xccdf_target_identifier_set_name(struct xccdf_target_identifier *ti, const char *newval)
+{
+	if (ti->any_element) {
+		if (ti->element)
+			xmlFreeNode(ti->element);
+	}
+
+	ti->any_element = false;
+	oscap_free(ti->name);
+	ti->name = oscap_strdup(newval);
+
+	return true;
+}
+
+const char* xccdf_target_identifier_get_name(const struct xccdf_target_identifier* ti)
+{
+	if (ti->any_element)
+		return NULL;
+
+	return ti->name;
+}
+
+OSCAP_ITERATOR_GEN(xccdf_target_identifier)
+OSCAP_ITERATOR_REMOVE_F(xccdf_target_identifier)
+
 struct xccdf_instance *xccdf_instance_new(void)
 {
     struct xccdf_instance *inst = oscap_calloc(1, sizeof(struct xccdf_instance));
@@ -341,6 +587,7 @@ const struct oscap_string_map XCCDF_RESULT_MAP[] = {
 
 static struct xccdf_identity *xccdf_identity_new_parse(xmlTextReaderPtr reader);
 static struct xccdf_target_fact *xccdf_target_fact_new_parse(xmlTextReaderPtr reader);
+static struct xccdf_target_identifier *xccdf_target_identifier_new_parse(xmlTextReaderPtr reader);
 static struct xccdf_score *xccdf_score_new_parse(xmlTextReaderPtr reader);
 static struct xccdf_rule_result *xccdf_rule_result_new_parse(xmlTextReaderPtr reader);
 static struct xccdf_override *xccdf_override_new_parse(xmlTextReaderPtr reader);
@@ -354,12 +601,17 @@ struct xccdf_result *xccdf_result_new_parse(xmlTextReaderPtr reader)
 
 	struct xccdf_item *res = XITEM(xccdf_result_new());
 
-	if (!xccdf_item_process_attributes(res, reader)) goto fail;
+	if (!xccdf_item_process_attributes(res, reader))
+		goto fail;
+
+	/* end time attribute is mandatory */
+	if (!xccdf_attribute_has(reader, XCCDFA_END_TIME)) {
+		oscap_seterr(OSCAP_EFAMILY_XCCDF, "Missing end-time attribute in TestResult element: %s", xccdf_item_get_id(res));
+		goto fail;
+	}
+	res->sub.result.end_time = xccdf_attribute_copy(reader, XCCDFA_END_TIME);
+	res->sub.result.start_time = xccdf_attribute_copy(reader, XCCDFA_START_TIME);
 
-	if (xccdf_attribute_has(reader, XCCDFA_END_TIME))
-		res->sub.result.end_time = oscap_get_datetime(xccdf_attribute_get(reader, XCCDFA_END_TIME));
-	else goto fail;
-	res->sub.result.start_time = oscap_get_datetime(xccdf_attribute_get(reader, XCCDFA_START_TIME));
 	res->item.version = xccdf_attribute_copy(reader, XCCDFA_VERSION);
 	res->sub.result.test_system = xccdf_attribute_copy(reader, XCCDFA_TEST_SYSTEM);
 
@@ -384,12 +636,19 @@ struct xccdf_result *xccdf_result_new_parse(xmlTextReaderPtr reader)
 			oscap_list_add(res->sub.result.target_addresses, oscap_element_string_copy(reader));
 			break;
 		case XCCDFE_TARGET_FACTS:
+			// we have to read inside the target-facts element
+			xmlTextReaderRead(reader);
+
+			// iterate over all inner elements of the target-facts element
 			while (oscap_to_start_element(reader, depth + 1)) {
 				if (xccdf_element_get(reader) == XCCDFE_FACT)
 					oscap_list_add(res->sub.result.target_facts, xccdf_target_fact_new_parse(reader));
 				xmlTextReaderRead(reader);
 			}
 			break;
+		case XCCDFE_TARGET_IDENTIFIER:
+			oscap_list_add(res->sub.result.target_id_refs, xccdf_target_identifier_new_parse(reader));
+			break;
 		case XCCDFE_SET_VALUE:
 			oscap_list_add(res->sub.result.setvalues, xccdf_setvalue_new_parse(reader));
 			break;
@@ -403,7 +662,13 @@ struct xccdf_result *xccdf_result_new_parse(xmlTextReaderPtr reader)
 			if (res->sub.result.benchmark_uri == NULL)
 				res->sub.result.benchmark_uri = xccdf_attribute_copy(reader, XCCDFA_HREF);
 			break;
-		default: xccdf_item_process_element(res, reader);
+		default:
+			if (!xccdf_item_process_element(res, reader))
+				dW("Encountered an unknown element '%s' while parsing XCCDF result.",
+				   xmlTextReaderConstLocalName(reader));
+			break;
+
+		// TODO: any element from other namespace is supposed to go into xccdf_target_identifier
 		}
 		xmlTextReaderRead(reader);
 	}
@@ -419,8 +684,6 @@ int xccdf_result_export(struct xccdf_result *result, const char *file)
 {
 	__attribute__nonnull__(file);
 
-	int retcode = 0;
-
 	LIBXML_TEST_VERSION;
 
 	xmlDocPtr doc = xmlNewDoc(BAD_CAST "1.0");
@@ -431,44 +694,29 @@ int xccdf_result_export(struct xccdf_result *result, const char *file)
 
 	xccdf_result_to_dom(result, NULL, doc, NULL);
 
-	retcode = xmlSaveFormatFileEnc(file, doc, "UTF-8", 1);
-	if (retcode < 1)
-		oscap_setxmlerr(xmlGetLastError());
-
-	xmlFreeDoc(doc);
-
-	return retcode;
+	return oscap_xml_save_filename(file, doc);
 }
 
 void xccdf_result_to_dom(struct xccdf_result *result, xmlNode *result_node, xmlDoc *doc, xmlNode *parent)
 {
         xmlNs *ns_xccdf = NULL;
+	const struct xccdf_version_info* version_info = xccdf_item_get_schema_version(XITEM(result));
 	if (parent) {
-	        ns_xccdf = xmlSearchNsByHref(doc, parent, XCCDF_BASE_NAMESPACE);
+	        ns_xccdf = xmlSearchNsByHref(doc, parent,
+	        		(const xmlChar*)xccdf_version_info_get_namespace_uri(version_info));
 	} else {
 		if (!result_node) result_node = xccdf_item_to_dom(XITEM(result), doc, NULL);
-                ns_xccdf = xmlNewNs(result_node, XCCDF_BASE_NAMESPACE, NULL);
+                ns_xccdf = xmlNewNs(result_node,
+                		(const xmlChar*)xccdf_version_info_get_namespace_uri(version_info), NULL);
 		xmlDocSetRootElement(doc, result_node);
 	}
 
 	/* Handle attributes */
-	time_t start = xccdf_result_get_start_time(result);
+	const char * start = xccdf_result_get_start_time(result);
 	if (start) {
-		struct tm *lt = localtime(&start);
-		char timestamp[] = "yyyy-mm-ddThh:mm:ss";
-		snprintf(timestamp, sizeof(timestamp), "%4d-%02d-%02dT%02d:%02d:%02d",
-			 1900 + lt->tm_year, 1 + lt->tm_mon, lt->tm_mday, lt->tm_hour, lt->tm_min, lt->tm_sec);
-		xmlNewProp(result_node, BAD_CAST "start-time", BAD_CAST timestamp);
-	}
-
-	time_t end = xccdf_result_get_end_time(result);
-	if (end) {
-		struct tm *lt = localtime(&end);
-		char timestamp[] = "yyyy-mm-ddThh:mm:ss";
-		snprintf(timestamp, sizeof(timestamp), "%4d-%02d-%02dT%02d:%02d:%02d",
-			 1900 + lt->tm_year, 1 + lt->tm_mon, lt->tm_mday, lt->tm_hour, lt->tm_min, lt->tm_sec);
-		xmlNewProp(result_node, BAD_CAST "end-time", BAD_CAST timestamp);
+		xmlNewProp(result_node, BAD_CAST "start-time", BAD_CAST start);
 	}
+	xmlNewProp(result_node, BAD_CAST "end-time", BAD_CAST xccdf_result_get_end_time(result));
 
 	/* Handle children */
 	xccdf_texts_to_dom(xccdf_result_get_remarks(result), result_node, "remark");
@@ -506,7 +754,7 @@ void xccdf_result_to_dom(struct xccdf_result *result, xmlNode *result_node, xmlD
 	struct xccdf_setvalue_iterator *setvalues = xccdf_result_get_setvalues(result);
 	while (xccdf_setvalue_iterator_has_more(setvalues)) {
 		struct xccdf_setvalue *setvalue = xccdf_setvalue_iterator_next(setvalues);
-		xccdf_setvalue_to_dom(setvalue, doc, result_node);
+		xccdf_setvalue_to_dom(setvalue, doc, result_node, version_info);
 	}
 	xccdf_setvalue_iterator_free(setvalues);
 
@@ -543,10 +791,17 @@ void xccdf_result_to_dom(struct xccdf_result *result, xmlNode *result_node, xmlD
         }
         xccdf_target_fact_iterator_free(target_facts);
 
+	struct xccdf_target_identifier_iterator *target_id_refs = xccdf_result_get_target_id_refs(result);
+	while (xccdf_target_identifier_iterator_has_more(target_id_refs)) {
+		struct xccdf_target_identifier *target_identifier = xccdf_target_identifier_iterator_next(target_id_refs);
+		xccdf_target_identifier_to_dom(target_identifier, doc, result_node, version_info);
+	}
+	xccdf_target_identifier_iterator_free(target_id_refs);
+
 	struct xccdf_rule_result_iterator *rule_results = xccdf_result_get_rule_results(result);
 	while (xccdf_rule_result_iterator_has_more(rule_results)) {
 		struct xccdf_rule_result *rule_result = xccdf_rule_result_iterator_next(rule_results);
-		xccdf_rule_result_to_dom(rule_result, doc, result_node);
+		xccdf_rule_result_to_dom(rule_result, doc, result_node, version_info);
 	}
 	xccdf_rule_result_iterator_free(rule_results);
 
@@ -593,6 +848,23 @@ static struct xccdf_target_fact *xccdf_target_fact_new_parse(xmlTextReaderPtr re
 	return fact;
 }
 
+static struct xccdf_target_identifier *xccdf_target_identifier_new_parse(xmlTextReaderPtr reader)
+{
+	struct xccdf_target_identifier *ret = xccdf_target_identifier_new();
+
+	if (xccdf_element_get(reader) == XCCDFE_TARGET_IDENTIFIER) {
+		xccdf_target_identifier_set_system(ret, xccdf_attribute_get(reader, XCCDFA_SYSTEM));
+		xccdf_target_identifier_set_href(ret, xccdf_attribute_get(reader, XCCDFA_HREF));
+		xccdf_target_identifier_set_name(ret, xccdf_attribute_get(reader, XCCDFA_NAME));
+	}
+	else {
+		// this is OK because we clone the node in the setter
+		xccdf_target_identifier_set_xml_node(ret, xmlTextReaderCurrentNode(reader));
+	}
+
+	return ret;
+}
+
 static struct xccdf_score *xccdf_score_new_parse(xmlTextReaderPtr reader)
 {
 	XCCDF_ASSERT_ELEMENT(reader, XCCDFE_SCORE);
@@ -614,7 +886,7 @@ static struct xccdf_rule_result *xccdf_rule_result_new_parse(xmlTextReaderPtr re
 
 	rr->idref    = xccdf_attribute_copy(reader, XCCDFA_IDREF);
 	rr->role     = oscap_string_to_enum(XCCDF_ROLE_MAP, xccdf_attribute_get(reader, XCCDFA_ROLE));
-	rr->time     = oscap_get_datetime(xccdf_attribute_get(reader, XCCDFA_TIME));
+	rr->time     = xccdf_attribute_copy(reader, XCCDFA_TIME);
 	rr->version  = xccdf_attribute_copy(reader, XCCDFA_VERSION);
 	rr->weight   = xccdf_attribute_get_float(reader, XCCDFA_WEIGHT);
 	rr->severity = oscap_string_to_enum(XCCDF_LEVEL_MAP, xccdf_attribute_get(reader, XCCDFA_SEVERITY));
@@ -652,9 +924,43 @@ static struct xccdf_rule_result *xccdf_rule_result_new_parse(xmlTextReaderPtr re
 	return rr;
 }
 
-xmlNode *xccdf_rule_result_to_dom(struct xccdf_rule_result *result, xmlDoc *doc, xmlNode *parent)
+xmlNode *xccdf_target_identifier_to_dom(const struct xccdf_target_identifier *ti, xmlDoc *doc, xmlNode *parent, const struct xccdf_version_info* version_info)
 {
-	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent, XCCDF_BASE_NAMESPACE);
+	if (ti->any_element) {
+		// we have to copy since we are adding the node to an existing document
+		// and that document will take ownership
+		xmlNodePtr ret = xmlCopyNode(ti->element, 1);
+		xmlAddChild(parent, ret);
+		return ret;
+	}
+	else {
+		xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent,
+				(const xmlChar*)xccdf_version_info_get_namespace_uri(version_info));
+
+		xmlNode *target_idref_node = xmlNewTextChild(parent, ns_xccdf, BAD_CAST "target-id-ref", NULL);
+
+		/* Handle attributes */
+		const char *psystem = xccdf_target_identifier_get_system(ti);
+		if (psystem)
+			xmlNewProp(target_idref_node, BAD_CAST "system", BAD_CAST psystem);
+
+		const char *href = xccdf_target_identifier_get_href(ti);
+		if (href)
+			xmlNewProp(target_idref_node, BAD_CAST "href", BAD_CAST href);
+
+		const char *name = xccdf_target_identifier_get_name(ti);
+		if (name)
+			xmlNewProp(target_idref_node, BAD_CAST "name", BAD_CAST name);
+
+		return target_idref_node;
+	}
+}
+
+xmlNode *xccdf_rule_result_to_dom(struct xccdf_rule_result *result, xmlDoc *doc, xmlNode *parent, const struct xccdf_version_info* version_info)
+{
+	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent,
+			(const xmlChar*)xccdf_version_info_get_namespace_uri(version_info));
+
 	xmlNode *result_node = xmlNewTextChild(parent, ns_xccdf, BAD_CAST "rule-result", NULL);
 
 	/* Handle attributes */
@@ -663,20 +969,16 @@ xmlNode *xccdf_rule_result_to_dom(struct xccdf_rule_result *result, xmlDoc *doc,
 		xmlNewProp(result_node, BAD_CAST "idref", BAD_CAST idref);
 
 	xccdf_role_t role = xccdf_rule_result_get_role(result);
-	if (role != 0) 
+	if (role != 0)
             xmlNewProp(result_node, BAD_CAST "role", BAD_CAST XCCDF_ROLE_MAP[role - 1].string);
 
-	time_t date = xccdf_rule_result_get_time(result);
-	if (date) {
-		struct tm *lt = localtime(&date);
-		char timestamp[] = "yyyy-mm-ddThh:mm:ss";
-		snprintf(timestamp, sizeof(timestamp), "%4d-%02d-%02dT%02d:%02d:%02d",
-			 1900 + lt->tm_year, 1 + lt->tm_mon, lt->tm_mday, lt->tm_hour, lt->tm_min, lt->tm_sec);
-		xmlNewProp(result_node, BAD_CAST "time", BAD_CAST timestamp);
+	const char * tm = xccdf_rule_result_get_time(result);
+	if (tm) {
+		xmlNewProp(result_node, BAD_CAST "time", BAD_CAST tm);
 	}
 
 	xccdf_level_t severity = xccdf_rule_result_get_severity(result);
-	if (severity != 0)
+	if (severity != XCCDF_LEVEL_NOT_DEFINED)
             xmlNewProp(result_node, BAD_CAST "severity", BAD_CAST XCCDF_LEVEL_MAP[severity - 1].string);
 
 	const char *version = xccdf_rule_result_get_version(result);
@@ -696,14 +998,14 @@ xmlNode *xccdf_rule_result_to_dom(struct xccdf_rule_result *result, xmlDoc *doc,
 	struct xccdf_override_iterator *overrides = xccdf_rule_result_get_overrides(result);
 	while (xccdf_override_iterator_has_more(overrides)) {
 		struct xccdf_override *override = xccdf_override_iterator_next(overrides);
-		xccdf_override_to_dom(override, doc, result_node);
+		xccdf_override_to_dom(override, doc, result_node, version_info);
 	}
 	xccdf_override_iterator_free(overrides);
 
 	struct xccdf_ident_iterator *idents = xccdf_rule_result_get_idents(result);
 	while (xccdf_ident_iterator_has_more(idents)) {
 		struct xccdf_ident *ident = xccdf_ident_iterator_next(idents);
-		xccdf_ident_to_dom(ident, doc, result_node);
+		xccdf_ident_to_dom(ident, doc, result_node, version_info);
 	}
 	xccdf_ident_iterator_free(idents);
 
@@ -714,7 +1016,7 @@ xmlNode *xccdf_rule_result_to_dom(struct xccdf_rule_result *result, xmlDoc *doc,
 		xmlNode *message_node = xmlNewTextChild(result_node, ns_xccdf, BAD_CAST "message", BAD_CAST content);
 
                 xccdf_level_t message_severity = xccdf_message_get_severity(message);
-		if (message_severity != 0)
+		if (message_severity != XCCDF_LEVEL_NOT_DEFINED)
                     xmlNewProp(message_node, BAD_CAST "severity", BAD_CAST XCCDF_LEVEL_MAP[message_severity - 1].string);
 	}
 	xccdf_message_iterator_free(messages);
@@ -745,7 +1047,7 @@ xmlNode *xccdf_rule_result_to_dom(struct xccdf_rule_result *result, xmlDoc *doc,
 	struct xccdf_check_iterator *checks = xccdf_rule_result_get_checks(result);
 	while (xccdf_check_iterator_has_more(checks)) {
 		struct xccdf_check *check = xccdf_check_iterator_next(checks);
-		xccdf_check_to_dom(check, doc, result_node);
+		xccdf_check_to_dom(check, doc, result_node, version_info);
 	}
 	xccdf_check_iterator_free(checks);
 
@@ -758,7 +1060,7 @@ static struct xccdf_override *xccdf_override_new_parse(xmlTextReaderPtr reader)
 
 	struct xccdf_override *override = xccdf_override_new();
 
-	override->time      = oscap_get_datetime(xccdf_attribute_get(reader, XCCDFA_TIME));
+	override->time      = xccdf_attribute_copy(reader, XCCDFA_TIME);
 	override->authority = xccdf_attribute_copy(reader, XCCDFA_AUTHORITY);
 
 	int depth = oscap_element_depth(reader) + 1;
@@ -783,18 +1085,18 @@ static struct xccdf_override *xccdf_override_new_parse(xmlTextReaderPtr reader)
 	return override;
 }
 
-xmlNode *xccdf_override_to_dom(struct xccdf_override *override, xmlDoc *doc, xmlNode *parent)
+xmlNode *xccdf_override_to_dom(struct xccdf_override *override, xmlDoc *doc, xmlNode *parent, const struct xccdf_version_info* version_info)
 {
-	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent, XCCDF_BASE_NAMESPACE);
+	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent,
+			(const xmlChar*)xccdf_version_info_get_namespace_uri(version_info));
+
 	xmlNode *override_node = xmlNewTextChild(parent, ns_xccdf, BAD_CAST "override", NULL);
 
 	/* Handle attributes */
-	time_t date = xccdf_override_get_time(override);
-	struct tm *lt = localtime(&date);
-	char timestamp[] = "yyyy-mm-ddThh:mm:ss";
-	snprintf(timestamp, sizeof(timestamp), "%4d-%02d-%02dT%02d:%02d:%02d",
-		 1900 + lt->tm_year, 1 + lt->tm_mon, lt->tm_mday, lt->tm_hour, lt->tm_min, lt->tm_sec);
-	xmlNewProp(override_node, BAD_CAST "date", BAD_CAST timestamp);
+	const char *tm = xccdf_override_get_time(override);
+	if(tm) {
+		xmlNewProp(override_node, BAD_CAST "date", BAD_CAST tm);
+	}
 
 	const char *authority = xccdf_override_get_authority(override);
 	if (authority)
@@ -866,18 +1168,6 @@ static void xccdf_rule_result_dump(struct xccdf_rule_result *res, int depth)
 	xccdf_print_depth(depth); printf("Rule result: %s\n", xccdf_rule_result_get_idref(res)); ++depth;
 	xccdf_print_depth(depth); printf("version:   %s\n", xccdf_rule_result_get_version(res));
 	xccdf_print_depth(depth); printf("weight:    %f\n", xccdf_rule_result_get_weight(res));
-	/*
-	xccdf_role_t role;
-	time_t time;
-	xccdf_level_t severity;
-	xccdf_test_result_type_t result;
-	struct oscap_list *overrides;
-	struct oscap_list *idents;
-	struct oscap_list *messages;
-	struct oscap_list *instances;
-	struct oscap_list *fixes;
-	struct oscap_list *checks;
-	*/
 }
 
 void xccdf_result_dump(struct xccdf_result *res, int depth)
@@ -900,6 +1190,7 @@ void xccdf_result_dump(struct xccdf_result *res, int depth)
 		xccdf_print_depth(depth); printf("target addresses");
 		oscap_list_dump(XITEM(res)->sub.result.target_addresses, xccdf_cstring_dump, depth+1);
 		//xccdf_print_depth(depth); printf("target_facts");
+		//xccdf_print_depth(depth); printf("target_id_refs");
 		xccdf_print_depth(depth); printf("setvalues");
 		oscap_list_dump(XITEM(res)->sub.result.setvalues, xccdf_setvalue_dump, depth+1);
 		xccdf_print_depth(depth); printf("rule results");
@@ -908,4 +1199,29 @@ void xccdf_result_dump(struct xccdf_result *res, int depth)
 	}
 }
 
+/*
+ * @returns pointer to text representation of the current local time
+ * and NULL on failure. The string is statically allocated and might
+ * be overwritten on subsequent calls to this function
+ */
+static inline const char *_get_timestamp(void)
+{
+	static char timestamp[] = "yyyy-mm-ddThh:mm:ss";
+	time_t tm;
+	struct tm *lt;
 
+	tm = time(NULL);
+	lt = localtime(&tm);
+	snprintf(timestamp, sizeof(timestamp), "%4d-%02d-%02dT%02d:%02d:%02d",
+		1900 + lt->tm_year, 1 + lt->tm_mon, lt->tm_mday,
+		lt->tm_hour, lt->tm_min, lt->tm_sec);
+	return timestamp;
+}
+
+#define XCCDF_CURRENT_TIME_SETTER(TYPE, BOUND) \
+	int xccdf_##TYPE##_set##BOUND##time_current(struct xccdf_##TYPE *r) \
+	{ return xccdf_##TYPE##_set##BOUND##time(r, _get_timestamp()); }
+
+XCCDF_CURRENT_TIME_SETTER(rule_result,_)
+XCCDF_CURRENT_TIME_SETTER(result,_start_)
+XCCDF_CURRENT_TIME_SETTER(result,_end_)
