@@ -33,7 +33,8 @@
 #include "elements.h"
 #include "helpers.h"
 #include "xccdf_impl.h"
-
+#include "common/assume.h"
+#include "common/debug_priv.h"
 
 bool xccdf_content_parse(xmlTextReaderPtr reader, struct xccdf_item *parent)
 {
@@ -51,10 +52,9 @@ bool xccdf_content_parse(xmlTextReaderPtr reader, struct xccdf_item *parent)
 		break;
 	default:
 		assert(false);
+		return false;
 	}
 
-	assert(list != NULL);
-
 	switch (xccdf_element_get(reader)) {
 	case XCCDFE_RULE:
 		item = xccdf_rule_parse(reader, parent);
@@ -142,9 +142,12 @@ static void xccdf_items_print_id_list(struct oscap_list *items, const char *sep)
 
 static void xccdf_item_dump_deps(struct xccdf_item *item, int depth)
 {
-	struct oscap_list *conflicts, *requires;
+	struct oscap_list *conflicts = NULL, *requires = NULL;
 	xccdf_deps_get(item, &conflicts, &requires);
 
+	assume_r(conflicts != NULL &&
+		 requires  != NULL, /* void */);
+
 	if (requires->itemcount > 0) {
 		struct oscap_list_item *it;
 		xccdf_print_depth(depth);
@@ -224,7 +227,9 @@ struct xccdf_item *xccdf_group_parse(xmlTextReaderPtr reader, struct xccdf_item
 			oscap_list_add(group->sub.group.values, xccdf_value_parse(reader, group));
 			break;
 		default:
-			xccdf_item_process_element(group, reader);
+			if (!xccdf_item_process_element(group, reader))
+				dW("Encountered an unknown element '%s' while parsing XCCDF group.",
+				   xmlTextReaderConstLocalName(reader));
 		}
 		xmlTextReaderRead(reader);
 	}
@@ -349,7 +354,9 @@ struct xccdf_item *xccdf_rule_parse(xmlTextReaderPtr reader, struct xccdf_item *
 			oscap_list_add(rule->sub.rule.idents, xccdf_ident_parse(reader));
 			break;
 		default:
-			xccdf_item_process_element(rule, reader);
+			if (!xccdf_item_process_element(rule, reader))
+				dW("Encountered an unknown element '%s' while parsing XCCDF group.",
+				   xmlTextReaderConstLocalName(reader));
 		}
 		xmlTextReaderRead(reader);
 	}
@@ -387,6 +394,46 @@ void xccdf_rule_free(struct xccdf_item *rule)
 	}
 }
 
+/**
+ * Filter function returning true if the check/@selector matches with selectid
+ */
+static bool
+_xccdf_check_filter_selector(struct xccdf_check *check, char *selectid)
+{
+	return !oscap_strcmp((char*) xccdf_check_get_selector(check), selectid);
+}
+
+/**
+ * Filter all checks within a rule by their @selector attribute.
+ * @param rule which encapsulates the checks
+ * @param selector string to filter these checks. NULL or "" values might be used
+ * in order to filter checks without @selector attribute.
+ */
+struct xccdf_check_iterator *
+xccdf_rule_get_checks_filtered(struct xccdf_item *rule, char *selector)
+{
+	return (struct xccdf_check_iterator *) oscap_iterator_new_filter(rule->sub.rule.checks, (oscap_filter_func) _xccdf_check_filter_selector, selector);
+}
+
+/**
+ * Filter function returning true if the xccdf_check is complex-check
+ */
+static bool
+_xccdf_check_filter_complex(struct xccdf_check *check, void *unused)
+{
+	return xccdf_check_get_complex(check);
+}
+
+/**
+ * Filter checks and return complex-check.
+ * Note: In valid XCCDF 1.2 documents there must not be more than one check within the given rule.
+ */
+struct xccdf_check_iterator *
+xccdf_rule_get_complex_checks(struct xccdf_item *rule)
+{
+	return (struct xccdf_check_iterator *) oscap_iterator_new_filter(rule->sub.rule.checks, (oscap_filter_func) _xccdf_check_filter_complex, NULL);
+}
+
 struct xccdf_ident * xccdf_ident_clone(const struct xccdf_ident * ident)
 {
 	struct xccdf_ident * clone = xccdf_ident_new();
@@ -485,6 +532,7 @@ struct xccdf_check *xccdf_check_clone(const struct xccdf_check* old_check)
 	new_check->selector = oscap_strdup(old_check->selector);
 	new_check->content =  oscap_strdup(old_check->content);
 	new_check->oper = old_check->oper;
+	new_check->flags = old_check->flags;
 
 	new_check->imports = oscap_list_clone(old_check->imports, (oscap_clone_func) xccdf_check_import_clone);
 	new_check->exports = oscap_list_clone(old_check->exports, (oscap_clone_func) xccdf_check_export_clone);
@@ -499,6 +547,8 @@ struct xccdf_check_import *xccdf_check_import_clone(const struct xccdf_check_imp
 {
 	struct xccdf_check_import *new_import = xccdf_check_import_new();
 	new_import->name = oscap_strdup(old_import->name);
+	if (old_import->xpath)
+		new_import->xpath = oscap_strdup(old_import->xpath);
 	new_import->content = oscap_strdup(old_import->content);
 	return new_import;
 }
@@ -539,8 +589,12 @@ struct xccdf_check *xccdf_check_parse(xmlTextReaderPtr reader)
 	check->system = xccdf_attribute_copy(reader, XCCDFA_SYSTEM);
 	check->selector = xccdf_attribute_copy(reader, XCCDFA_SELECTOR);
 	check->oper = oscap_string_to_enum(XCCDF_BOOLOP_MAP, xccdf_attribute_get(reader, XCCDFA_OPERATOR));
-	if (xccdf_attribute_get_bool(reader, XCCDFA_NEGATE))
-		check->oper |= XCCDF_OPERATOR_NOT;
+	if (xccdf_attribute_has(reader, XCCDFA_MULTICHECK) && el != XCCDFE_COMPLEX_CHECK) {
+		check->flags.def_multicheck = true;
+		check->flags.multicheck = xccdf_attribute_get_bool(reader, XCCDFA_MULTICHECK);
+	}
+	check->flags.def_negate = xccdf_attribute_has(reader, XCCDFA_NEGATE);
+	check->flags.negate = xccdf_attribute_get_bool(reader, XCCDFA_NEGATE);
 
 	int depth = oscap_element_depth(reader) + 1;
 
@@ -564,14 +618,17 @@ struct xccdf_check *xccdf_check_parse(xmlTextReaderPtr reader)
 			}
 		case XCCDFE_CHECK_CONTENT:
 			if (check->content == NULL)
-				check->content = oscap_element_string_copy(reader);
+				check->content = oscap_get_xml(reader);
 			break;
 		case XCCDFE_CHECK_IMPORT:{
 				const char *name = xccdf_attribute_get(reader, XCCDFA_IMPORT_NAME);
-				if (name == NULL)
+				const char *xpath = xccdf_attribute_get(reader, XCCDFA_IMPORT_XPATH);
+				if (name == NULL) // @import-name is a required attribute
 					break;
 				struct xccdf_check_import *imp = xccdf_check_import_new();
 				imp->name = strdup(name);
+				if (xpath) // @import-xpath is just optional
+					imp->xpath = strdup(xpath);
 				imp->content = oscap_element_string_copy(reader);
 				oscap_list_add(check->imports, imp);
 				break;
@@ -618,6 +675,22 @@ void xccdf_check_dump(struct xccdf_check *check, int depth)
 	oscap_list_dump(check->content_refs, (oscap_dump_func) xccdf_check_content_ref_dump, depth + 1);
 }
 
+/**
+ * Enforce given content with given name to be the only check-content of the check.
+ * This may turn to be usefull,when processing check within rule-result --
+ * such check may (and in some cases must) refer only to the executed content.
+ */
+bool
+xccdf_check_inject_content_ref(struct xccdf_check *check, const struct xccdf_check_content_ref *content, const char *name)
+{
+	struct xccdf_check_content_ref *content_clone = xccdf_check_content_ref_clone(content);
+	if (name != NULL)
+		xccdf_check_content_ref_set_name(content_clone, name);
+	oscap_list_free(check->content_refs, (oscap_destruct_func) xccdf_check_content_ref_free);
+	check->content_refs = oscap_list_new();
+	return oscap_list_add(check->content_refs, content_clone);
+}
+
 bool xccdf_check_get_complex(const struct xccdf_check *check)
 {
 	return check->oper ? true : false;
@@ -667,6 +740,7 @@ void xccdf_check_import_free(struct xccdf_check_import *item)
 {
 	if (item) {
 		oscap_free(item->name);
+		oscap_free(item->xpath);
 		oscap_free(item->content);
 		oscap_free(item);
 	}
@@ -857,7 +931,9 @@ bool xccdf_item_add_conflicts(struct xccdf_item* item, const char* conflicts)
 
 void xccdf_rule_to_dom(struct xccdf_rule *rule, xmlNode *rule_node, xmlDoc *doc, xmlNode *parent)
 {
-	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent, XCCDF_BASE_NAMESPACE);
+	const struct xccdf_version_info* version_info = xccdf_item_get_schema_version(XITEM(rule));
+	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent,
+			(const xmlChar*)xccdf_version_info_get_namespace_uri(version_info));
 
 	/* Handle Attributes */
 	const char *extends = xccdf_rule_get_extends(rule);
@@ -883,17 +959,11 @@ void xccdf_rule_to_dom(struct xccdf_rule *rule, xmlNode *rule_node, xmlDoc *doc,
 		xmlNewProp(rule_node, BAD_CAST "role", BAD_CAST XCCDF_ROLE_MAP[role - 1].string);
 
 	xccdf_level_t severity = xccdf_rule_get_severity(rule);
-	if (severity != 0)
+	if (severity != XCCDF_LEVEL_NOT_DEFINED)
 		xmlNewProp(rule_node, BAD_CAST "severity", BAD_CAST XCCDF_LEVEL_MAP[severity - 1].string);
 
 	/* Handle Child Nodes */
-	struct oscap_text_iterator *rationales = xccdf_rule_get_rationale(rule);
-	while (oscap_text_iterator_has_more(rationales)) {
-		struct oscap_text *rationale = oscap_text_iterator_next(rationales);
-		xmlNode * child = xmlNewTextChild(rule_node, ns_xccdf, BAD_CAST "rationale", BAD_CAST oscap_text_get_text(rationale));
-                if (oscap_text_get_lang(rationale) != NULL) xmlNewProp(child, BAD_CAST "xml:lang", BAD_CAST oscap_text_get_lang(rationale));
-	}
-	oscap_text_iterator_free(rationales);
+	xccdf_texts_to_dom(xccdf_rule_get_rationale(rule), rule_node, "rationale");
 
 	struct oscap_string_iterator *platforms = xccdf_rule_get_platforms(rule);
 	while (oscap_string_iterator_has_more(platforms)) {
@@ -927,7 +997,7 @@ void xccdf_rule_to_dom(struct xccdf_rule *rule, xmlNode *rule_node, xmlDoc *doc,
 	struct xccdf_ident_iterator *idents = xccdf_rule_get_idents(rule);
 	while (xccdf_ident_iterator_has_more(idents)) {
 		struct xccdf_ident *ident = xccdf_ident_iterator_next(idents);
-		xccdf_ident_to_dom(ident, doc, rule_node);
+		xccdf_ident_to_dom(ident, doc, rule_node, version_info);
 	}
 	xccdf_ident_iterator_free(idents);
 
@@ -955,14 +1025,16 @@ void xccdf_rule_to_dom(struct xccdf_rule *rule, xmlNode *rule_node, xmlDoc *doc,
 	struct xccdf_check_iterator *checks = xccdf_rule_get_checks(rule);
 	while (xccdf_check_iterator_has_more(checks)) {
 		struct xccdf_check *check = xccdf_check_iterator_next(checks);
-		xccdf_check_to_dom(check, doc, rule_node);
+		xccdf_check_to_dom(check, doc, rule_node, version_info);
 	}
 	xccdf_check_iterator_free(checks);
 }
 
 void xccdf_group_to_dom(struct xccdf_group *group, xmlNode *group_node, xmlDoc *doc, xmlNode *parent)
 {
-	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent, XCCDF_BASE_NAMESPACE);
+	const struct xccdf_version_info* version_info = xccdf_item_get_schema_version(XITEM(group));
+	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent,
+			(const xmlChar*)xccdf_version_info_get_namespace_uri(version_info));
 
 	/* Handle Attributes */
 	const char *extends = xccdf_group_get_extends(group);
@@ -976,8 +1048,6 @@ void xccdf_group_to_dom(struct xccdf_group *group, xmlNode *group_node, xmlDoc *
 			xmlNewProp(group_node, BAD_CAST "selected", BAD_CAST "false");
 	}
 
-	xmlNewProp(group_node, BAD_CAST "hidden", BAD_CAST (xccdf_group_get_hidden(group) ? "true" : "false"));
-
 	if (XITEM(group)->item.defined_flags.weight) {
 		float weight = xccdf_group_get_weight(group);
 		char *weight_str = oscap_sprintf("%f", weight);
@@ -1071,6 +1141,8 @@ OSCAP_ACCESSOR_STRING(xccdf_check, id)
 OSCAP_ACCESSOR_STRING(xccdf_check, system)
 OSCAP_ACCESSOR_STRING(xccdf_check, selector)
 OSCAP_ACCESSOR_STRING(xccdf_check, content)
+OSCAP_ACCESSOR_EXP(bool, xccdf_check, multicheck, flags.multicheck)
+OSCAP_ACCESSOR_EXP(bool, xccdf_check, negate, flags.negate);
 OSCAP_ACCESSOR_SIMPLE(xccdf_bool_operator_t, xccdf_check, oper)
 OSCAP_IGETINS(xccdf_check_import, xccdf_check, imports, import)
 OSCAP_IGETINS(xccdf_check_export, xccdf_check, exports, export)
@@ -1080,7 +1152,8 @@ XCCDF_ITERATOR_GEN_S(check_content_ref)
 XCCDF_ITERATOR_GEN_S(check_export) XCCDF_ITERATOR_GEN_S(check_import) XCCDF_ITERATOR_GEN_S(check)
 
 OSCAP_ACCESSOR_STRING(xccdf_profile_note, reftag)    OSCAP_ACCESSOR_TEXT(xccdf_profile_note, text)
-OSCAP_ACCESSOR_STRING(xccdf_check_import, name)      OSCAP_ACCESSOR_STRING(xccdf_check_import, content)
+OSCAP_ACCESSOR_STRING(xccdf_check_import, name)      OSCAP_ACCESSOR_STRING(xccdf_check_import, xpath)
+OSCAP_ACCESSOR_STRING(xccdf_check_import, content)
 OSCAP_ACCESSOR_STRING(xccdf_check_export, name)      OSCAP_ACCESSOR_STRING(xccdf_check_export, value)
 OSCAP_ACCESSOR_STRING(xccdf_check_content_ref, name) OSCAP_ACCESSOR_STRING(xccdf_check_content_ref, href)
 
