@@ -31,12 +31,11 @@
 #ifndef XCCDF_POLICY_H_
 #define XCCDF_POLICY_H_
 
+#include <xccdf_benchmark.h>
 #include <stdbool.h>
 #include <time.h>
 #include <oscap.h>
-#include <xccdf.h>
-#include <reporter.h>
- 
+
 /**
  * @struct xccdf_policy_model
  * Handle all policies for given XCCDF benchmark
@@ -64,12 +63,49 @@ struct xccdf_value_binding_iterator;
  */
 struct xccdf_policy_iterator;
 
+/**
+ * Type of a query over checking-engine data.
+ * This allows xccdf_policy module to query checking engine and acquire comprehensive info.
+ */
+typedef enum {
+	POLICY_ENGINE_QUERY_NAMES_FOR_HREF = 1,		/// Considering xccdf:check-content-ref, what are possible @name attributes for given href?
+} xccdf_policy_engine_query_t;
+
+/**
+ * Type of function which implements queries defined within xccdf_policy_engine_query_t.
+ *
+ * Each checking engine may register its own function of the xccdf_policy_engine_query_fn
+ * type. The registered function is then used by xccdf_policy module to acquire comprehensive
+ * info about the checking-engine itself or the data fed in. First argument of the function
+ * is always user data as registered. Second argument defines the query. Third argument is
+ * dependent on query and defined as follows:
+ *  - (const char *)href -- for POLICY_ENGINE_QUERY_NAMES_FOR_HREF
+ *
+ * Expected return type depends also on query as follows:
+ *  - (struct oscap_stringlists *) -- for POLICY_ENGINE_QUERY_NAMES_FOR_HREF
+ *  - NULL shall be returned if the function doesn't understand the query.
+ */
+typedef void *(*xccdf_policy_engine_query_fn) (void *, xccdf_policy_engine_query_t, void *);
+
+/**
+ * Type of function which implements OpenSCAP checking engine.
+ *
+ * This function defines basic interface between XCCDF module and thee checking engine.
+ * For each checking engine required for evaluation there should be at least one such
+ * function registerd. The registered function is then used by xccdf_policy module to
+ * perform evaluation on the machine.
+ */
+typedef xccdf_test_result_type_t (*xccdf_policy_engine_eval_fn) (struct xccdf_policy *policy, const char *rule_id, const char *definition_id, const char *href_if, struct xccdf_value_binding_iterator *value_binding_it, struct xccdf_check_import_iterator *check_imports_it, void *user_data);
+
 /************************************************************/
 
 /**
  * Constructor of Policy Model structure
  * @param benchmark Struct xccdf_benchmark with benchmark model
  * @return new xccdf_policy_model
+ * @note
+ * The policy model will take ownership of given benchmark and free it
+ * when it's being destructed!
  * @memberof xccdf_policy_model
  */
 struct xccdf_policy_model *xccdf_policy_model_new(struct xccdf_benchmark *benchmark);
@@ -107,6 +143,48 @@ void xccdf_policy_free(struct xccdf_policy *);
  */
 void xccdf_value_binding_free(struct xccdf_value_binding *);
 
+/**
+ * Sets the Tailoring element to use in the policy.
+ *
+ * Profiles from the Benchmark *may* be shadowed by profiles in the given
+ * Tailoring element! Calling this function will affect the results of
+ * xccdf_policy_model_get_policy_by_id calls.
+ *
+ * @important This does INVALIDATE preexisting xccdf_policy pointers in
+ * this xccdf_policy_model! It is recommended to call this function before
+ * policy for any profile is queried.
+ *
+ * @note
+ * The policy model will take ownership of given tailoring and free it
+ * when it's being destructed or when new tailoring is being set.
+ */
+bool xccdf_policy_model_set_tailoring(struct xccdf_policy_model *model, struct xccdf_tailoring *tailoring);
+
+/**
+ * Retrieves the Tailoring element used in this policy.
+ * @memberof xccdf_policy_model
+ */
+struct xccdf_tailoring *xccdf_policy_model_get_tailoring(struct xccdf_policy_model *model);
+
+/**
+ * Registers an additional CPE dictionary for applicability testing
+ * The one embedded in the evaluated XCCDF take precedence!
+ */
+bool xccdf_policy_model_add_cpe_dict(struct xccdf_policy_model * model, const char * cpe_dict);
+
+/**
+ * Registers an additional CPE lang model for applicability testing
+ * The one embedded in the evaluated XCCDF take precedence!
+ */
+bool xccdf_policy_model_add_cpe_lang_model(struct xccdf_policy_model * model, const char * cpe_lang);
+
+/**
+ * Registers an additional CPE resource (either dictionary or language)
+ * Autodetects given file and acts accordingly.
+ * The one embedded in the evaluated XCCDF take precedence!
+ */
+bool xccdf_policy_model_add_cpe_autodetect(struct xccdf_policy_model *model, const char* filepath);
+
 /**
  * Function to register callback for checking system
  * @param model XCCDF Policy Model
@@ -115,8 +193,25 @@ void xccdf_value_binding_free(struct xccdf_value_binding *);
  * @param usr optional parameter for passing user data to callback
  * @memberof xccdf_policy_model
  * @return true if callback registered succesfully, false otherwise
+ *
+ * @deprecated This function is deprecated by @ref xccdf_policy_model_register_engine_and_query_callback
+ * and might be dropped from future releases.
  */
-bool xccdf_policy_model_register_engine_callback(struct xccdf_policy_model * model, char * sys, void * func, void * usr);
+OSCAP_DEPRECATED(bool xccdf_policy_model_register_engine_callback(struct xccdf_policy_model * model, char * sys, void * func, void * usr));
+
+/**
+ * Function to register callback for checking system
+ * @param model XCCDF Policy Model
+ * @param sys String representing given checking system
+ * @param eval_fn Callback - pointer to function called by XCCDF Policy system when rule parsed
+ * @param usr optional parameter for passing user data to callback
+ * @param query_fn - optional parameter for providing xccdf_policy_engine_query_fn implementation for given system.
+ * @memberof xccdf_policy_model
+ * @return true if callback registered succesfully, false otherwise
+ */
+bool xccdf_policy_model_register_engine_and_query_callback(struct xccdf_policy_model *model, char *sys, xccdf_policy_engine_eval_fn eval_fn, void *usr, xccdf_policy_engine_query_fn query_fn);
+
+typedef int (*policy_reporter_output)(struct xccdf_rule_result *, void *);
 
 /**
  * Function to register output callback for checking system that will be called AFTER each rule evaluation.
@@ -125,30 +220,10 @@ bool xccdf_policy_model_register_engine_callback(struct xccdf_policy_model * mod
  * @param usr optional parameter for passing user data to callback
  * @memberof xccdf_policy_model
  * @return true if callback registered succesfully, false otherwise
- * \par Example
- * With the first function below (register output callback) user registers the callback that will be called after
- * each rule evalution is done. Second callback is registered as callback for evaluation itself and will be called
- * during the evaluation.
- * \code
- * xccdf_policy_model_register_output_callback(policy_model, callback, NULL);
- * xccdf_policy_model_register_engine_callback(policy_model, "http://oval.mitre.org/XMLSchema/oval-definitions-5", oval_agent_eval_rule, (void *) sess);
- * \endcode
- * The example of callback:
- * \code
- * static int callback(const struct oscap_reporter_message *msg, void *arg)
- * {
- *      xccdf_test_result_type_t result = oscap_reporter_message_get_user2num(msg);
- *      if (result == XCCDF_RESULT_NOT_SELECTED) return 0;
- * 
- *      printf("\n");
- *      printf("Rule ID:\r\t\t\033[1m%s\033[0;0m\n", oscap_reporter_message_get_user1str(msg));
- *      printf("Title:\r\t\t%s\n", oscap_reporter_message_get_user3str(msg));
- *      printf("Result:\r\t\t\033[%sm%s\033[0m\n", RESULT_COLORS[result], xccdf_test_result_type_get_text((xccdf_test_result_type_t) result));
- *      return 0;
- * }
- * \endcode
  */
-bool xccdf_policy_model_register_output_callback(struct xccdf_policy_model * model, oscap_reporter func, void * usr);
+bool xccdf_policy_model_register_output_callback(struct xccdf_policy_model * model, policy_reporter_output func, void * usr);
+
+typedef int (*policy_reporter_start)(struct xccdf_rule *, void *);
 
 /**
  * Function to register start callback for checking system that will be called BEFORE each rule evaluation.
@@ -157,25 +232,8 @@ bool xccdf_policy_model_register_output_callback(struct xccdf_policy_model * mod
  * @param usr optional parameter for passing user data to callback
  * @memberof xccdf_policy_model
  * @return true if callback registered succesfully, false otherwise
- * \par Example
- * With the first function below (register start callback) user registers the callback that will be called before
- * each rule evalution is started. Second callback is registered as callback for evaluation itself and will be called
- * during the evaluation. Last callback is registered output callback.
- * \code
- * xccdf_policy_model_register_start_callback(policy_model, callback_start, NULL);
- * xccdf_policy_model_register_engine_callback(policy_model, "http://oval.mitre.org/XMLSchema/oval-definitions-5", oval_agent_eval_rule, (void *) sess);
- * xccdf_policy_model_register_output_callback(policy_model, callback_end, NULL);
- * \endcode
- * The example of callback_start:
- * \code
- * static int callback(const struct oscap_reporter_message *msg, void *arg)
- * {
- *      printf("Evaluating rule \"%s\". Please wait.". oscap_reporter_message_get_user1num(msg));
- *      return 0;
- * }
- * \endcode
  */
-bool xccdf_policy_model_register_start_callback(struct xccdf_policy_model * model, oscap_reporter func, void * usr);
+bool xccdf_policy_model_register_start_callback(struct xccdf_policy_model * model, policy_reporter_start func, void * usr);
 
 /************************************************************/
 /**
@@ -208,7 +266,8 @@ struct xccdf_benchmark * xccdf_policy_model_get_benchmark(const struct xccdf_pol
 struct xccdf_value_binding_iterator  * xccdf_policy_get_values(const struct xccdf_policy * item);
 
 /**
- * Get policies from Policy Model
+ * Get policies from Policy Model. Be aware, this function returns only a list of previously
+ * initialized policies. A policy can be initialized by xccdf_policy_model_get_policy_by_id.
  * @param model Policy Model
  * @return Iterator for list of policies
  * @memberof xccdf_policy_model
@@ -331,8 +390,12 @@ bool xccdf_policy_add_select(struct xccdf_policy *, struct xccdf_select *);
  * Set a new selector to the Policy structure
  * @memberof xccdf_policy
  * @return true if rule has been added succesfully
+ * @deprecated This function is deprecated by @ref xccdf_policy_add_select
+ * and might be dropped from future releases.
  */
-bool xccdf_policy_set_selected(struct xccdf_policy * policy, char * idref);
+OSCAP_DEPRECATED(
+bool xccdf_policy_set_selected(struct xccdf_policy * policy, char * idref)
+);
 
 /**
  * Add result to XCCDF Policy Model
@@ -348,11 +411,18 @@ bool xccdf_policy_add_result(struct xccdf_policy * policy, struct xccdf_result *
 bool xccdf_policy_add_value(struct xccdf_policy *, struct xccdf_value_binding *);
 
 /**
- * Add check export to the Value Binding structure
- * @memberof xccdf_value_binding
- * @return true if rule has been added succesfully
+ * Get the selection settings of the item.
+ * @memberof xccdf_policy
+ * @return true if the item is selected
  */
-//bool xccdf_value_binding_add_check_export(struct xccdf_value_binding *, struct xccdf_check_export *);
+bool xccdf_policy_is_item_selected(struct xccdf_policy *policy, const char *id);
+
+/**
+ * Retrieves number of selected items in the policy
+ * @note This is meant to be used to estimate scanning progress for example.
+ */
+int xccdf_policy_get_selected_rules_count(struct xccdf_policy *policy);
+
 /**
  * Get select from policy by specified ID of XCCDF Item
  * @memberof xccdf_policy
@@ -379,7 +449,7 @@ struct xccdf_select * xccdf_policy_get_select_by_id(struct xccdf_policy * policy
  * Every checking engine must have registered callback or the particular check will be skipped.
  * In the code below is used the predefined function \ref oval_agent_eval_rule for evaluation OVAL checks:
  * \code
- * xccdf_policy_model_register_engine_callback(policy_model, "http://oval.mitre.org/XMLSchema/oval-definitions-5", oval_agent_eval_rule, (void *) usr);
+ * xccdf_policy_model_register_engine_oval(policy_mode, agent_session)
  * \endcode
  * \par
  * If you use this predefined OVAL callback, user data structure (last parameter of register function) \b MUST be of type \ref\a oval_agent_session_t:
@@ -399,6 +469,18 @@ struct xccdf_result *  xccdf_policy_evaluate(struct xccdf_policy * policy);
  */
 bool xccdf_policy_resolve(struct xccdf_policy * policy);
 
+/**
+ * Generate remediation prescription (presumably a remediation script).
+ * @memberof xccdf_policy
+ * @param policy XCCDF Policy
+ * @param result XCCDF TestResult. This may be omitted to generate the prescription
+ * based solely on the XCCDF Policy (xccdf:Profile).
+ * @param sys Consider only those fixes that have @system attribute equal to sys
+ * @param output_fd write prescription to this file descriptor
+ * @returns zero on success, non-zero indicate partial (incomplete) output.
+ */
+int xccdf_policy_generate_fix(struct xccdf_policy *policy, struct xccdf_result *result, const char *sys, int output_fd);
+
 /**
  * Clone the item and tailor it against given policy (profile)
  * @param policy Policy with profile
@@ -407,6 +489,67 @@ bool xccdf_policy_resolve(struct xccdf_policy * policy);
  */
 struct xccdf_item * xccdf_policy_tailor_item(struct xccdf_policy * policy, struct xccdf_item * item);
 
+/**
+ * xccdf_policy_model_get_files and xccdf_item_get_files each return oscap_file_entries instead of raw strings
+ */
+struct oscap_file_entry;
+
+/// @memberof oscap_file_entry
+struct oscap_file_entry *oscap_file_entry_new(void);
+/// @memberof oscap_file_entry
+struct oscap_file_entry *oscap_file_entry_dup(struct oscap_file_entry* file_entry);
+/// @memberof oscap_file_entry
+void oscap_file_entry_free(struct oscap_file_entry* entry);
+/// @memberof oscap_file_entry
+const char* oscap_file_entry_get_system(struct oscap_file_entry* entry);
+/// @memberof oscap_file_entry
+const char* oscap_file_entry_get_file(struct oscap_file_entry* entry);
+
+/** @struct oscap_file_entry_iterator
+ * @see oscap_iterator
+ */
+struct oscap_file_entry_iterator;
+
+/// @memberof oscap_file_entry_iterator
+const struct oscap_file_entry *oscap_file_entry_iterator_next(struct oscap_file_entry_iterator *it);
+/// @memberof oscap_file_entry_iterator
+bool oscap_file_entry_iterator_has_more(struct oscap_file_entry_iterator *it);
+/// @memberof oscap_file_entry_iterator
+void oscap_file_entry_iterator_free(struct oscap_file_entry_iterator *it);
+/// @memberof oscap_file_entry_iterator
+void oscap_file_entry_iterator_reset(struct oscap_file_entry_iterator *it);
+
+/** @struct oscap_file_entry_list
+ * @see oscap_list
+ */
+struct oscap_file_entry_list;
+
+/// @memberof oscap_file_entry_list
+struct oscap_file_entry_list* oscap_file_entry_list_new(void);
+/// @memberof oscap_file_entry_list
+void oscap_file_entry_list_free(struct oscap_file_entry_list* list);
+/// @memberof oscap_file_entry_list
+struct oscap_file_entry_iterator* oscap_file_entry_list_get_files(struct oscap_file_entry_list* list);
+
+/**
+ * Return names of files that are used in checks of particular rules. Every check needs this file to be
+ * evaluated properly. If this file will not be imported and bind to the XCCDF Policy system the result
+ * of rule after evaluation will be "Not checked"
+ *
+ *
+ * The resulting list should be freed with oscap_filelist_free.
+ */
+struct oscap_file_entry_list * xccdf_policy_model_get_systems_and_files(struct xccdf_policy_model * policy_model);
+
+/**
+ * Return names of files that are used in checks of particular rules. Every check needs this file to be
+ * evaluated properly. If this file will not be imported and bind to the XCCDF Policy system the result
+ * of rule after evaluation will be "Not checked"
+ *
+ * The resulting list should be freed with oscap_filelist_free.
+ */
+struct oscap_file_entry_list * xccdf_item_get_systems_and_files(struct xccdf_item * item);
+
 /**
  * Return names of files that are used in checks of particular rules. Every check needs this file to be
  * evaluated properly. If this file will not be imported and bind to the XCCDF Policy system the result
@@ -421,6 +564,13 @@ struct oscap_stringlist * xccdf_policy_model_get_files(struct xccdf_policy_model
  */
 struct oscap_stringlist * xccdf_item_get_files(struct xccdf_item * item);
 
+/**
+ * Return result of the AND operation for two given attributes.
+ * For more details about the attributes A and B please consult 'Table 26: Possible Results for a Single Test' from NISTIR-7275r4.
+ * For more details about the AND operation please consult 'Table 12: Truth Table for AND' in the very same document.
+ */
+xccdf_test_result_type_t xccdf_test_result_resolve_and_operation(xccdf_test_result_type_t A, xccdf_test_result_type_t B);
+
 /************************************************************/
 /** @} End of Evaluators group */
 
@@ -497,7 +647,7 @@ char* xccdf_policy_substitute(const char *text, struct xccdf_policy *policy);
 /************************************************************/
 /** @} End of Iterators group */
 
-/*
+/**
  * @}
  */
 #endif
