@@ -29,10 +29,14 @@
 #endif
 
 #include <string.h>
+#include <fcntl.h>
+#include <unistd.h>
 
 #include "public/oscap.h"
 #include "util.h"
 #include "list.h"
+#include "_error.h"
+#include "debug_priv.h"
 #include "elements.h"
 
 
@@ -48,7 +52,6 @@ bool oscap_to_start_element(xmlTextReaderPtr reader, int depth)
 	//int olddepth = xmlTextReaderDepth(reader);
 	while (xmlTextReaderDepth(reader) >= depth) {
 		switch (xmlTextReaderNodeType(reader)) {
-			//TODO: change int values to macros XML_ELEMENT_TYPE_*
 		case XML_READER_TYPE_ELEMENT:
 			if (xmlTextReaderDepth(reader) == depth)
 				return true;
@@ -79,7 +82,8 @@ char *oscap_element_string_copy(xmlTextReaderPtr reader)
 
 const char *oscap_element_string_get(xmlTextReaderPtr reader)
 {
-	if (xmlTextReaderNodeType(reader) == 1 || xmlTextReaderNodeType(reader) == 2)
+	if (xmlTextReaderNodeType(reader) == XML_READER_TYPE_ELEMENT ||
+			xmlTextReaderNodeType(reader) == XML_READER_TYPE_ATTRIBUTE)
 		xmlTextReaderRead(reader);
 	if (xmlTextReaderHasValue(reader))
 		return (const char *)xmlTextReaderConstValue(reader);
@@ -90,9 +94,9 @@ int oscap_element_depth(xmlTextReaderPtr reader)
 {
 	int depth = xmlTextReaderDepth(reader);
 	switch (xmlTextReaderNodeType(reader)) {
-	case 2:
-	case 5:
-	case 3:
+	case XML_READER_TYPE_ATTRIBUTE:
+	case XML_READER_TYPE_TEXT:
+	case XML_READER_TYPE_ENTITY_REFERENCE:
 		return depth - 1;
 	default:
 		return depth;
@@ -159,4 +163,40 @@ const char *oscap_strlist_find_value(char ** const kvalues, const char *key)
 	return NULL;
 }
 
+int
+oscap_xml_save_filename(const char *filename, xmlDocPtr doc)
+{
+	xmlOutputBufferPtr buff;
+	int xmlCode;
+
+	if (strcmp(filename, "-") == 0) {
+		xmlCode = xmlSaveFormatFileEnc(filename, doc, "UTF-8", 1);
+	}
+	else {
+		int fd = open(filename, O_CREAT|O_TRUNC|O_WRONLY,
+				S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
+		if (fd < 0) {
+			oscap_seterr(OSCAP_EFAMILY_GLIBC, "%s '%s'", strerror(errno), filename);
+			return -1;
+		}
+
+		buff = xmlOutputBufferCreateFd(fd, NULL);
+		if (buff == NULL) {
+			close(fd);
+			oscap_setxmlerr(xmlGetLastError());
+			oscap_dlprintf(DBG_W, "xmlOutputBufferCreateFile() failed.\n");
+			return -1;
+		}
+
+		xmlCode = xmlSaveFormatFileTo(buff, doc, "UTF-8", 1);
+		close(fd);
+	}
+	if (xmlCode <= 0) {
+		oscap_setxmlerr(xmlGetLastError());
+		oscap_dlprintf(DBG_W, "No bytes exported: xmlCode: %d.\n", xmlCode);
+	}
+
+	xmlFreeDoc(doc);
+	return (xmlCode >= 1) ? 1 : -1;
+}
 
