@@ -28,8 +28,10 @@
 #include <pthread.h>
 #include <string.h>
 
-#include "public/alloc.h"
+#include "alloc.h"
+#include "util.h"
 #include "_error.h"
+#include "err_queue.h"
 #include "debug_priv.h"
 
 static pthread_key_t __key;
@@ -40,41 +42,41 @@ static void oscap_errkey_init(void)
 	(void)pthread_key_create(&__key, NULL);
 }
 
-static struct oscap_err_t *oscap_err_new(oscap_errfamily_t family, oscap_errcode_t code, const char *desc,
+static struct oscap_err_t *oscap_err_new(oscap_errfamily_t family, const char *desc,
 					 const char *func, uint32_t line, const char *file)
 {
 	struct oscap_err_t *err;
 
 	err = oscap_talloc(struct oscap_err_t);
 	err->family = family;
-	err->code = code;
-	err->desc = (desc == NULL ? NULL : strdup(desc));
+	err->desc = oscap_sprintf("%s [%s:%d]", desc, file, line);
 	err->func = func;
 	err->line = line;
 	err->file = file;
 
-	char *family_str;
-
-	switch (family) {
-	case OSCAP_EFAMILY_XML:
-		family_str = "XML";
-		break;
-	case OSCAP_EFAMILY_GLIBC:
-		family_str = "GLIBC";
-		break;
-	case OSCAP_EFAMILY_OSCAP:
-		family_str = "OSCAP";
-		break;
-	default:
-		family_str = "UNKNOWN";
-		break;
-	}
-
-	dE("\(%s:%d:%s\(\)\) %s", file, line, func, desc);
+	dE("\(%s:%d:%s\()) %s\n", file, line, func, desc);
 
 	return (err);
 }
 
+static void oscap_err_free(struct oscap_err_t *err)
+{
+	if (err->desc != NULL)
+		oscap_free(err->desc);
+	oscap_free(err);
+}
+
+static inline void _push_err(struct oscap_err_t *err)
+{
+	struct err_queue *q = pthread_getspecific(__key);
+	if (q == NULL) {
+		q = err_queue_new();
+		assert(q != NULL);
+		(void)pthread_setspecific(__key, q);
+	}
+	(void)err_queue_push(q, err);
+}
+
 void __oscap_setxmlerr(const char *file, uint32_t line, const char *func, xmlErrorPtr error)
 {
 
@@ -84,78 +86,89 @@ void __oscap_setxmlerr(const char *file, uint32_t line, const char *func, xmlErr
 	struct oscap_err_t *err;
 	(void)pthread_once(&__once, oscap_errkey_init);
 
-	oscap_clearerr();
-	err = oscap_err_new(OSCAP_EFAMILY_XML, error->code, error->message, func, line, file);
-	(void)pthread_setspecific(__key, err);
-
-	return;
-
+	/* get rid of a newline that was pass by xmlGetLastError() */
+	int len = strlen(error->message);
+	if( error->message[len-1] == '\n' )
+		error->message[len-1] = 0;
+	err = oscap_err_new(OSCAP_EFAMILY_XML, error->message, func, line, file);
+	_push_err(err);
 }
 
-void __oscap_seterr(const char *file,
-		    uint32_t line, const char *func, oscap_errfamily_t family, oscap_errcode_t code, const char *desc)
+void __oscap_seterr(const char *file, uint32_t line, const char *func, oscap_errfamily_t family, ...)
 {
 	struct oscap_err_t *err;
+	char *msg;
+	va_list ap;
+	const char *fmt;
 
 	(void)pthread_once(&__once, oscap_errkey_init);
 
-	oscap_clearerr();
-	err = oscap_err_new(family, code, desc, func, line, file);
-	(void)pthread_setspecific(__key, err);
+	va_start(ap, family);
+	fmt = va_arg(ap, const char *);
+	msg = oscap_vsprintf(fmt, ap);
+	va_end(ap);
 
-	return;
+	err = oscap_err_new(family, msg, func, line, file);
+
+	oscap_free(msg);
+	_push_err(err);
 }
 
 void oscap_clearerr(void)
 {
-	struct oscap_err_t *err;
+	struct err_queue *q;
 
 	(void)pthread_once(&__once, oscap_errkey_init);
 
-	err = pthread_getspecific(__key);
+	q = pthread_getspecific(__key);
 	(void)pthread_setspecific(__key, NULL);
-
-	if (err != NULL) {
-		if (err->desc != NULL)
-			oscap_free(err->desc);
-		oscap_free(err);
-	}
-
-	return;
+	err_queue_free(q, (oscap_destruct_func) oscap_err_free);
 }
 
 bool oscap_err(void)
 {
 	(void)pthread_once(&__once, oscap_errkey_init);
-	return (pthread_getspecific(__key) != NULL);
+	struct err_queue *q = pthread_getspecific(__key);
+	return (q != NULL && !err_queue_is_empty(q));
 }
 
 oscap_errfamily_t oscap_err_family(void)
 {
-	struct oscap_err_t *err;
+	struct err_queue *q;
 
 	(void)pthread_once(&__once, oscap_errkey_init);
-	err = pthread_getspecific(__key);
+	q = pthread_getspecific(__key);
 
-	return (err == NULL ? 0 : err->family);
-}
-
-oscap_errcode_t oscap_err_code(void)
-{
-	struct oscap_err_t *err;
-
-	(void)pthread_once(&__once, oscap_errkey_init);
-	err = pthread_getspecific(__key);
-
-	return (err == NULL ? 0 : err->code);
+	if (q == NULL || err_queue_is_empty(q))
+		return 0;
+	return err_queue_get_last(q)->family;
 }
 
 const char *oscap_err_desc(void)
 {
-	struct oscap_err_t *err;
+	struct err_queue *q;
 
 	(void)pthread_once(&__once, oscap_errkey_init);
-	err = pthread_getspecific(__key);
+	q = pthread_getspecific(__key);
 
-	return (err == NULL ? 0 : (const char *)err->desc);
+	if (q == NULL || err_queue_is_empty(q))
+		return 0;
+	return (const char *) err_queue_get_last(q)->desc;
+}
+
+char *oscap_err_get_full_error(void)
+{
+	struct err_queue *q;
+	char *res = NULL;
+
+	(void)pthread_once(&__once, oscap_errkey_init);
+	q = pthread_getspecific(__key);
+	if (q == NULL || err_queue_is_empty(q))
+		return NULL;
+	if (err_queue_to_string(q, &res) != 0)
+		return NULL;
+
+	(void)pthread_setspecific(__key, NULL);
+	err_queue_free(q, (oscap_destruct_func) oscap_err_free);
+	return res;
 }
