@@ -14,25 +14,24 @@
 
 function test_api_cpe_dict_smoke {
     ./test_api_cpe_dict --smoke-test
-    return $?
 }
 
 function test_api_cpe_dict_remove_cpe {
     require "grep" || return 255
-    ! ./test_api_cpe_dict --remove $srcdir/dict.xml "UTF-8" "cpe:/a:addsoft" | \
-	grep -s "addsoft"
-    return $?
+    ./test_api_cpe_dict --remove $srcdir/dict.xml "UTF-8" "cpe:/a:addsoft" | \
+    grep -s "addsoft"
+    return $([ $? -eq 1 ])
 }
 
 function test_api_cpe_dict_import_damaged_xml {
-    ! ./test_api_cpe_dict --list-cpe-names $srcdir/dict-damaged.xml "UTF-8"
-    return $?
+    ./test_api_cpe_dict --list-cpe-names $srcdir/dict-damaged.xml "UTF-8"
+    return $([ $? -eq 2 ])
 }
 
 function test_api_cpe_dict_match_non_existing_cpe {
-    ! ./test_api_cpe_dict --match $srcdir/dict.xml \
-	"UTF-8" "cpe:/a:3com:3c16115-usNOT_IN_THE_DICTIONARY"
-    return $?
+    ./test_api_cpe_dict --match $srcdir/dict.xml \
+    "UTF-8" "cpe:/a:3com:3c16115-usNOT_IN_THE_DICTIONARY"
+    return $([ $? -eq 1 ])
 }
 
 function test_api_cpe_dict_match_existing_cpe {
@@ -49,18 +48,15 @@ function test_api_cpe_dict_match_existing_cpe {
 function test_api_cpe_dict_export_xml {
     ./test_api_cpe_dict --export $srcdir/dict.xml "UTF-8" \
 	dict.xml.out "UTF-8" && \
-	$srcdir/../../../xmldiff.pl $srcdir/dict.xml dict.xml.out
-    return $?
+	$XMLDIFF $srcdir/dict.xml dict.xml.out
 }
 
 function test_api_cpe_dict_import_cp1250_xml {
     ./test_api_cpe_dict --list-cpe-names $srcdir/dict-cp1250-dos.xml "CP-1250"
-    return $?
 }
 
 function test_api_cpe_dict_import_utf8_xml {
     ./test_api_cpe_dict --list-cpe-names $srcdir/dict.xml "UTF8"
-    return $?
 }
 
 # Testing.
