@@ -32,7 +32,7 @@ function test_probes_runlevel_A {
     [ -f $RF ] && rm -f $RF
 
     bash ${srcdir}/test_probes_runlevel_A.xml.sh > $DF
-    ../../../utils/.libs/oscap oval eval --results $RF $DF
+    $OSCAP oval eval --results $RF $DF
         
     if [ -f $RF ]; then
 
@@ -80,10 +80,6 @@ function test_probes_runlevel_B {
 
     probecheck "runlevel" || return 255
 
-    export OVAL_PROBE_DIR=`pwd`/../../../src/OVAL/probes/
-    export LD_LIBRARY_PATH=`pwd`/../../../src/.libs
-    export OSCAP_SCHEMA_PATH=$srcdir/../../../schemas
-
     local ret_val=0;
     local DF="test_probes_runlevel_B.xml"
     local RF="results_B.xml"
@@ -92,7 +88,7 @@ function test_probes_runlevel_B {
     local SERVICE_B=`get_services_matching 3 off | head -1`
 
     bash ${srcdir}/test_probes_runlevel_B.xml.sh $SERVICE_A $SERVICE_B > $DF
-    ../../../utils/.libs/oscap oval eval --results $RF $DF
+    $OSCAP oval eval --results $RF $DF
         
     if [ -f $RF ]; then
 	verify_results "def" $DF $RF 13 && verify_results "tst" $DF $RF 40
